/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * bernimmt die Kommunikation mit dem DaV und enthlt Funktionen zum Abfragen
 * der im DaV hinterlegten Berechtigungen.
 * 
 * Diese Klasse implementiert das Singleton-Entwurfsmuster.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * @deprecated es sollte die Rechteverwaltung vom Rahmenwerk verwendet werden
 */
@Deprecated
public final class DaVKommunikation {

	/** ldie einzigste Instanz der Klasse {@link DaVKommunikation}. */
	private static DaVKommunikation instanz;

	private final Berechtigungen berechtigungen;

	private final List<IOberflaechenBerechtigungZuhoerer> listenerList = new ArrayList<IOberflaechenBerechtigungZuhoerer>();

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link DaVKommunikation}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link DaVKommunikation}.
	 */
	public static DaVKommunikation getInstanz() {
		if (DaVKommunikation.instanz == null) {
			DaVKommunikation.instanz = new DaVKommunikation();
		}
		return DaVKommunikation.instanz;
	}

	/**
	 * Privater Konstruktor des Singletons.
	 * 
	 */
	private DaVKommunikation() {
		berechtigungen = RahmenwerkService.getService().getBerechtigungen();
		berechtigungen
				.addOberflaechenFunktionsListener(new IBerechtigungListener() {

					@Override
					public void sperrung(final BerechtigungEreignis e) {
						fireSperrung(e);
					}

					@Override
					public void freigabe(final BerechtigungEreignis e) {
						fireFreigabe(e);
					}
				});
	}

	protected void fireFreigabe(final BerechtigungEreignis e) {
		ArrayList<IOberflaechenBerechtigungZuhoerer> fireList;
		synchronized (listenerList) {
			fireList = new ArrayList<IOberflaechenBerechtigungZuhoerer>(
					listenerList);
		}

		final List<OberflaechenFunktion> funktionen = new ArrayList<OberflaechenFunktion>();
		for (final FunktionMitBerechtigung fkt : e.getFunktionen()) {
			final OberflaechenFunktion funktion = OberflaechenBerechtigung
					.getInstanz().getFunktion(fkt.getId());
			if (funktion != null) {
				funktionen.add(funktion);
			}
		}

		final OberflaechenBerechtigungEreignis ereignis = new OberflaechenBerechtigungEreignis(
				e.getSource(), true, new ArrayList<SystemObject>(),
				new ArrayList<SystemObject>(), funktionen,
				new ArrayList<OberflaechenFunktion>());
		for (final IOberflaechenBerechtigungZuhoerer zuhoerer : fireList) {
			zuhoerer.freigabe(ereignis);
		}
	}

	protected void fireSperrung(final BerechtigungEreignis e) {
		ArrayList<IOberflaechenBerechtigungZuhoerer> fireList;
		synchronized (listenerList) {
			fireList = new ArrayList<IOberflaechenBerechtigungZuhoerer>(
					listenerList);
		}

		final List<OberflaechenFunktion> funktionen = new ArrayList<OberflaechenFunktion>();
		for (final FunktionMitBerechtigung fkt : e.getFunktionen()) {
			final OberflaechenFunktion funktion = OberflaechenBerechtigung
					.getInstanz().getFunktion(fkt.getId());
			if (funktion != null) {
				funktionen.add(funktion);
			}
		}

		final OberflaechenBerechtigungEreignis ereignis = new OberflaechenBerechtigungEreignis(
				e.getSource(), false, new ArrayList<SystemObject>(),
				new ArrayList<SystemObject>(),
				new ArrayList<OberflaechenFunktion>(), funktionen);
		for (final IOberflaechenBerechtigungZuhoerer zuhoerer : fireList) {
			zuhoerer.sperrung(ereignis);
		}
	}

	/**
	 * Diese Methode prft, ob auf einem Systemobjekt ein bestimmter Benutzer
	 * das Recht hat, eine Funktion durchzufhren.
	 * 
	 * @param systemObject
	 *            Systemobjekt, fr das die Berechtigung geprft werden soll.
	 *            <b>Worauf?</b>
	 * @param benutzer
	 *            Benutzer, fr den die Berechtigung auf diesem Systemobjekt
	 *            geprft werden soll. <b>Wer?</b>
	 * @param funktion
	 *            Funktion, fr welche auf dem Systemobjekt fr den gegebenen
	 *            Benutzer die Berechtigung geprft werden soll. <b>Was?</b>
	 * @return Liefert <code>true</code>, wenn der Benutzer die Berechtigung
	 *         hat, sonst false.
	 */
	public boolean hatBerechtigung(final SystemObject systemObject,
			final String benutzer, final OberflaechenFunktion funktion) {
		final FunktionMitBerechtigung funktionMitBerechtigung = berechtigungen
				.getFunktion(funktion.getFunktionsName());

		if (funktionMitBerechtigung == null) {
			return false;
		}
		return berechtigungen
				.hasBerechtigung(benutzer, funktionMitBerechtigung);
	}

	/**
	 * Fgt einen Listener zur Liste der Betriebsmeldungslistener hinzu.
	 * 
	 * @param listener
	 */
	public void hinzufuegenOberflaechenBenutzerZuhoerer(
			final IOberflaechenBerechtigungZuhoerer listener) {
		synchronized (listenerList) {
			listenerList.add(listener);
		}
	}

	/**
	 * Entfernt einen {@link IOberflaechenBerechtigungZuhoerer} von der Liste
	 * der Zuhrer.
	 * 
	 * @param benutzerZuhoerer
	 *            Zu entfernender {@link IOberflaechenBerechtigungZuhoerer}.
	 */
	public void entferneOberflaechenBenutzerZuhoerer(
			final IOberflaechenBerechtigungZuhoerer listener) {
		synchronized (listenerList) {
			listenerList.remove(listener);
		}
	}

	/**
	 * Benachrichtigt alle registrierten
	 * {@link IOberflaechenBerechtigungZuhoerer} darber, dass ein neuer
	 * Benutzer sich angemeldet hat.
	 * 
	 * @deprecated die Benachrichtigung erfolgt ber die Berechtigungsverwaltung
	 *             selbst
	 */
	@Deprecated
	public void benachrichtigeOberflaechenBenutzerZuhoerer(
			final OberflaechenBerechtigungEreignis ereignis) {
		synchronized (listenerList) {
			for (final IOberflaechenBerechtigungZuhoerer listener : listenerList) {
				if (ereignis.hatFreigabe()) {
					listener.freigabe(ereignis);
				} else {
					listener.sperrung(ereignis);
				}
			}
		}
	}

	/**
	 * Holt das Verbindungungsobjekt.
	 * 
	 * @return
	 */
	public ClientDavInterface getConnection() {
		return RahmenwerkService.getService().getRahmenWerk()
				.getDavVerbindung();
	}

	/**
	 * @deprecated die Anmeldung beim Datenverteiler erfolgt intern im
	 *             Rahmenwerk.
	 */
	@Deprecated
	public void neuAnmeldung() {
		// tut nichts, die Datenverteileranmeldung erfolgt in der neuen
		// Berechtigungsverwaltung.
	}

	/**
	 * @deprecated die Anmeldung beim Datenverteiler erfolgt intern im
	 *             Rahmenwerk.
	 */
	@Deprecated
	public void benutzerAnmeldung() {
		// tut nichts, die Datenverteileranmeldungen erfolgen in der neuen
		// Berechtigungsverwaltung.
	}

}
