/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

/**
 * Klasse OberflaechenFunktion.
 * 
 * Ersetzt die in TBuV-104 geforderte XML-Datei zur Definition von Funktionen.
 * Der Entwickler kann mit dieser Klasse eine Funktion anlegen und zustzlich
 * einen {@link IOberflaechenBerechtigungZuhoerer} angeben, der ihn ber
 * nderungen der Berechtigung informiert.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * @deprecated ersetzt durch {@link FunktionMitBerechtigung}.
 */
@Deprecated
public class OberflaechenFunktion {
	/**
	 * Name der Funktion.
	 */
	private final String funktionsName;

	/**
	 * Alias-Name der Funktion.
	 */
	private String funktionsAlias;

	/**
	 * Eindeutige ElementId fr ein Element, dessen Oberflchenberechtigung
	 * geprft werden soll.
	 */
	private final String elementId;

	/**
	 * Eindeutige ID des Dialogs oder des Views, in der die
	 * OberflaechenBerechtigungsFunktion bentigt wird.
	 */
	private final String dialogId;

	/**
	 * Listener der Funktion.
	 */
	private final IOberflaechenBerechtigungZuhoerer zuhoerer;

	/**
	 * Konstruktor fr eine OberflaechenFunktion.
	 * 
	 * @param funktionsName
	 *            Name der Funktion.
	 * @param listener
	 *            {@link IOberflaechenBerechtigungZuhoerer} der Funktion.
	 * @param dialogId
	 *            Eindeutige Identifizierung des Dialogs, fr den die
	 *            Berechtigung geprft werden soll.
	 * @param elementId
	 *            Eindeutige Id des Elements, fr dass die Berechtigung geprft
	 *            werden soll.
	 * 
	 */
	public OberflaechenFunktion(final String funktionsName,
			final IOberflaechenBerechtigungZuhoerer listener,
			final String dialogId, final String elementId) {
		this.funktionsName = funktionsName;
		this.zuhoerer = listener;
		this.dialogId = dialogId;
		this.elementId = elementId;
	}

	/**
	 * Konstruktor fr ein OberflaechenFunktion.
	 * 
	 * @param funktionsName
	 *            Name der Funktion.
	 * @param funktionsAlias
	 *            Alias-Name der Funktion.
	 * @param listener
	 *            {@link IOberflaechenBerechtigungZuhoerer} der Funktion.
	 * @param dialogId
	 *            Identifizierung des Dialogs, fr den die Berechtigung geprft
	 *            werden soll.
	 * @param elementId
	 *            Eindeutige Id des Elements, fr dass die Berechtigung geprft
	 *            werden soll.
	 * 
	 */
	public OberflaechenFunktion(final String funktionsName,
			final String funktionsAlias,
			final IOberflaechenBerechtigungZuhoerer listener,
			final String dialogId, final String elementId) {
		this.funktionsName = funktionsName;
		this.funktionsAlias = funktionsAlias;
		this.zuhoerer = listener;
		this.dialogId = dialogId;
		this.elementId = elementId;
	}

	/**
	 * Liefert den Namen der Funktion.
	 * 
	 * @return Funktionsname
	 */
	public String getFunktionsName() {
		return funktionsName;
	}

	/**
	 * Liefert den Alias der Funktion.
	 * 
	 * @return Funktionsalias
	 */
	public String getFunktionsAlias() {
		return funktionsAlias;
	}

	/**
	 * Liefert die ElementID, auf die sich diese Oberflchenfunktion bezieht.
	 * 
	 * @return ElementID.
	 */
	public String getElementId() {
		return elementId;
	}

	/**
	 * Liefert die DialogID, auf die sich diese Oberflchenfunktion bezieht.
	 * 
	 * @return DialogId
	 */
	public String getDialogId() {
		return dialogId;
	}

	/**
	 * Liefert den {@link IOberflaechenBerechtigungZuhoerer} der Funktion.
	 * 
	 * @return {@link IOberflaechenBerechtigungZuhoerer} der Funktion.
	 */
	public IOberflaechenBerechtigungZuhoerer getZuhoerer() {
		return zuhoerer;
	}

}
