/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.datenidentifikation;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse Datenidentifikation ermglicht die Auswahl von Objekten ber
 * folgende Auswahllisten. <br>
 * - Objekt-Liste<br>
 * - Objekttyp-Liste<br>
 * - Attributgruppen-Liste<br>
 * - Aspekt-Liste<br>
 * - Simulations-Variante<br>
 * 
 * <br>
 * Die Klasse erfllt folgende Technische Anforderungen des Segments Bedienung
 * und Visualisierung (BUV)<br>
 * - TBuV-96 und 97
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/14 13:15:29 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author Dambach-Werke GmbH
 * @author HofmannJ
 */
public class Datenidentifikation extends Dialog {

	// Links
	private Composite compositeLinks;

	private Composite compositeLinksObjekttypTitel;

	private Composite compositeLinksObjekttyp;

	private Composite compositeLinksAttributgruppeTitel;

	private Composite compositeLinksAttributgruppe;

	private Composite compositeLinksAspektTitel;

	private Composite compositeLinksAspekt;

	private Composite compositeLinksSimulationsvariante;

	private Button schaltflaecheLoescheObjekttyp;

	private Text textAnzahlObjekttyp;

	private Button schaltflaecheLoescheAttributgruppe;

	private Text textAnzahlAttributgruppe;

	private Button schaltflaecheLoescheAspekt;

	private Text textAnzahlAspekte;

	private Spinner spinnerSimulationsvariante;

	// Rechts
	private Composite compositeRechts;

	private Composite compositeRechtsOben;

	private Composite compositeRechtsMitte;

	private Composite compositeRechtsUnten;

	private Text textAnzahlObjekte;

	private Button schaltflaecheLoescheSelektierteObjekte;

	/**
	 * OK Schaltflche.
	 */
	private Button schaltflaecheOk;

	/**
	 * Abbrechen Schaltflche.
	 */
	private Button schaltflaecheAbbrechen;

	/**
	 * Hilfe Schaltflche.
	 */
	private Button schaltflaecheHilfe;

	// Rechts InfoBereich
	private Composite compositeRechtsInfoBereich;

	private Group gruppeInfo;

	private Text infoText;

	// SWT-Listen
	/**
	 * Enthlt die Darstellung der ObjektTypen-Liste.
	 */
	private List listeSWTObjekttypListe;

	/**
	 * Enthlt die Darstellung der Attribut-Liste.
	 */
	private List listeSWTAttributListe;

	/**
	 * Enthlt die Darstellung der Aspekt-Liste.
	 */
	private List listeSWTAspektListe;

	/**
	 * Enthlt die Darstellung der Obhekt-Liste.
	 */
	private List listeSWTObjektListe;

	/*
	 * TreeMaps fr die Objektdatenhaltung
	 */

	/**
	 * TreeMap fr die Objekte.
	 */
	private TreeMap<String, ListenElementObjekt> treeMapObjekt;

	/**
	 * TreeMap fr die ObjektTypen.
	 */
	private TreeMap<String, ListenElementObjektTyp> treeMapObjektTyp;

	/**
	 * TreeMap der Attributgruppen.
	 */
	private TreeMap<String, ListenElementAttributGruppe> treeMapAttributgruppe;

	/**
	 * TreeMap der Aspekte.
	 */
	private TreeMap<String, ListenElementAspekt> treeMapAspekt;

	/**
	 * In diesem Array werden sie selektierten SystemObjekte abgelegt und vom
	 * Dialog zurckgegeben.
	 */
	private SystemObject[] systemObjectAuswahl;

	private Shell shell;

	/**
	 * Verbindung zum Datenverteiler.
	 */
	private final ClientDavInterface connection;

	/**
	 * Breite des Dialogs.
	 */
	private static final int BREITE = 700;

	/**
	 * Hhe des Dialogs.
	 */
	private static final int HOEHE = 900;

	/**
	 * Font fr die Lschen-Buttons (X).
	 */
	private final Font buttonFont = new Font(Display.getCurrent(),
			new FontData("Arial", 8, SWT.BOLD));

	/**
	 * Standard SWT-Dialog Konstruktor.
	 * 
	 * @param parent
	 *            Parent Shell, von der ausgehend die Darstellung generiert
	 *            werden soll.
	 */
	public Datenidentifikation(final Shell parent, final ClientDavInterface con) {
		super(parent);
		this.connection = con;
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param darstellung
	 *            Gibt an, welche Bestandteile der Datenidentifikation angezeigt
	 *            werden sollen.
	 * @return SystemObject-Array mit den selektierten Objekten.
	 */
	public SystemObject[] oeffne(final Darstellung darstellung) {
		// this._darstellung = darstellung;

		final Shell parent = getParent();
		shell = new Shell(parent, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.MAX);
		shell.setText("Datenidentifikations-Eingabe");
		final GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 2;
		shell.setLayout(gridLayout);

		shell.setSize(Datenidentifikation.BREITE, Datenidentifikation.HOEHE);
		erzeugeGui();
		initialisiere();

		// _shell.pack();
		shell.open();

		final Display display = parent.getDisplay();

		// Dialog zentrieren
		final Monitor primary = display.getPrimaryMonitor();
		final Rectangle bounds = primary.getBounds();
		final Rectangle rect = shell.getBounds();
		final int x = bounds.x + ((bounds.width - rect.width) / 2);
		final int y = bounds.y + ((bounds.height - rect.height) / 2);
		shell.setLocation(x, y);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		// return true;

		/*
		 * Hier wird die Auswahl der selektierten Objekte in einem Array von
		 * SystemObjekten zurckgegeben.
		 */
		return systemObjectAuswahl;
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation, jedoch werden nur die
	 * ObjektTypen nagezeigt, die vom ObjektTyp des bergebenen Filters sind.
	 * 
	 * @param sObjektTypFilter
	 *            Gibt an, welche Objekte von welchem Typ angezeigt werden
	 *            sollen.
	 * @param darstellung
	 *            Gibt an, welche Bestandteile der Datenidentifikation angezeigt
	 *            werden sollen.
	 * @return SystemObject-Array mit den selektierten Objekten.
	 */
	public SystemObject[] oeffne(final String[] sObjektTypFilter,
			final Darstellung darstellung) {
		// this._darstellung = darstellung;

		final Shell parent = getParent();
		shell = new Shell(parent, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.MAX);
		shell.setText("Datenidentifikations-Eingabe");
		final GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 2;
		shell.setLayout(gridLayout);

		shell.setSize(Datenidentifikation.BREITE, Datenidentifikation.HOEHE);

		erzeugeGui();
		initialisiere();

		// Lschen aller nicht bentigter ObjektTypen aus TreeMapObjektTyp

		entferneObjektTypen(sObjektTypFilter);

		// Aufruf von fuellelisteSWTObjekteFiltered2
		listeSWTObjekttypListe.removeAll();
		fuellelisteSWTObjektTyp();

		fuellelisteSWTObjekteGefiltert();

		/*
		 * Initiale Darstellung der Objekte zu den vorhandenen Objekttypen.
		 */
		fuellelisteSWTObjekteFiltered(listeSWTObjekttypListe.getItems());

		// _shell.pack();
		shell.open();

		final Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		// return true;
		return systemObjectAuswahl;
	}

	/**
	 * Erstellt die Basis des SWT-Dialogs.
	 * 
	 */
	private void erzeugeGui() {
		final Composite flaeche = new Composite(shell, SWT.NONE);
		flaeche.setLayout(new FillLayout());
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		flaeche.setLayoutData(gridData);

		final SashForm formBasis = new SashForm(flaeche, SWT.HORIZONTAL);

		erzeugeCompositeLinks(formBasis);
		erzeugeCompositeRechts(formBasis);

		formBasis.setWeights(new int[] { 50, 50 });

		erzeugeCompositeRechtsUnten(shell);

	}

	/**
	 * Initialisiert die bentigten TreeMaps und startet die Basisfllung der
	 * Datenstrukturen.
	 * 
	 */
	private void initialisiere() {
		// TreeMaps initialisieren
		treeMapObjekt = new TreeMap<String, ListenElementObjekt>();
		treeMapObjektTyp = new TreeMap<String, ListenElementObjektTyp>();
		treeMapAttributgruppe = new TreeMap<String, ListenElementAttributGruppe>();
		treeMapAspekt = new TreeMap<String, ListenElementAspekt>();

		if (connection != null) {
			fuelleAlles();
		}

	}

	/**
	 * Generiert das linke Composite.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinks(final Composite parent) {
		compositeLinks = new Composite(parent, SWT.BORDER);

		compositeLinks.setLayout(new FillLayout());

		final SashForm formLinks = new SashForm(compositeLinks, SWT.VERTICAL);
		final Composite obj = new Composite(formLinks, SWT.NONE);
		final Composite atg = new Composite(formLinks, SWT.NONE);
		final Composite asp = new Composite(formLinks, SWT.NONE);
		final Composite simvar = new Composite(formLinks, SWT.NONE);
		formLinks.setWeights(new int[] { 30, 30, 30, 10 });

		final FillLayout fillLayout = new FillLayout();
		obj.setLayout(fillLayout);
		atg.setLayout(fillLayout);
		asp.setLayout(fillLayout);
		simvar.setLayout(fillLayout);

		final Composite objComplex = new Composite(obj, SWT.NONE);
		objComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksObjekttypTitel(objComplex);
		erzeugeCompositeLinksObjekttyp(objComplex);

		final Composite atgComplex = new Composite(atg, SWT.NONE);
		atgComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksAttributgruppeTitel(atgComplex);
		erzeugeCompositeLinksAttributgruppe(atgComplex);

		final Composite aspComplex = new Composite(asp, SWT.NONE);
		aspComplex.setLayout(new GridLayout());
		erzeugeCompositeLinksAspektTitel(aspComplex);
		erzeugeCompositeLinksAspekt(aspComplex);

		final Composite simvarCompex = new Composite(simvar, SWT.NONE);
		simvarCompex.setLayout(new GridLayout());
		erzeugeCompositeLinksSimulationsVariante(simvarCompex);
	}

	/**
	 * Generiert den Objekttyp-Titel im linken Composite.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksObjekttypTitel(final Composite parent) {

		compositeLinksObjekttypTitel = new Composite(parent, SWT.NONE);

		final Composite compositeLinksObjekttypTitelLinks = new Composite(
				compositeLinksObjekttypTitel, SWT.NONE);
		final Composite compositeLinksObjekttypTitelRechts = new Composite(
				compositeLinksObjekttypTitel, SWT.NONE);

		final Label labelObjekttyp = new Label(
				compositeLinksObjekttypTitelLinks, SWT.NONE);
		labelObjekttyp.setText("Objekttyp");

		textAnzahlObjekttyp = new Text(compositeLinksObjekttypTitelRechts,
				SWT.BORDER);
		textAnzahlObjekttyp.setText("0 / 0");
		textAnzahlObjekttyp.setEditable(false);
		schaltflaecheLoescheObjekttyp = new Button(
				compositeLinksObjekttypTitelRechts, SWT.PUSH);

		schaltflaecheLoescheObjekttyp.setText("X");
		schaltflaecheLoescheObjekttyp.setFont(buttonFont);

		schaltflaecheLoescheObjekttyp
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {

						listeSWTObjekttypListe.deselectAll();
						listeSWTObjektListe.setVisible(false);
						listeSWTObjektListe.removeAll();
						fuellelisteSWTObjekte();
						// listeSWTObjekttypListe.selectAll();
						// fuellelisteSWTObjekteFiltered2();
						// listeSWTObjekttypListe.deselectAll();
						listeSWTObjektListe.setVisible(true);
						fuelleTreeMapAttributgruppeFiltered(listeSWTObjekttypListe
								.getSelection());
					}
				});

		final GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 3;
		gridLayout.numColumns = 2;
		compositeLinksObjekttypTitel.setLayout(gridLayout);

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		compositeLinksObjekttypTitel.setLayoutData(gridData);

		compositeLinksObjekttypTitelLinks.setLayout(new GridLayout());
		// compositeLinksObjekttypTitelRechts

		final GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = true;
		rowLayoutRechts.justify = false;
		compositeLinksObjekttypTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksObjekttypTitelRechts.setLayoutData(gridDataRechts);
	}

	/**
	 * Generiert das linke Composite, dass die SWT-Liste mit der Darstellung der
	 * vorhandenen ObjektTypen enthlt.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksObjekttyp(final Composite parent) {
		compositeLinksObjekttyp = new Composite(parent, SWT.NONE);

		listeSWTObjekttypListe = new List(compositeLinksObjekttyp, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL);
		listeSWTObjekttypListe.setBounds(0, 0, 300, 200);

		listeSWTObjekttypListe.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				if (listeSWTObjekttypListe.getSelectionIndex() > -1) {
					// _selektionObjekttyp =
					// _listeSWTObjekttypListe.getSelectionIndices();

					berechneAnzahlObjekttypen();

					listeSWTAttributListe.removeAll();
					listeSWTAspektListe.removeAll();

					fuellelisteSWTObjekteFiltered(listeSWTObjekttypListe
							.getSelection());
					fuelleTreeMapAttributgruppeFiltered(listeSWTObjekttypListe
							.getSelection());

				}
			}
		});

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeLinksObjekttyp.setLayout(new FillLayout());
		compositeLinksObjekttyp.setLayoutData(gridData);

	}

	/**
	 * Generiert das linke Composite, dass den Attributgruppen-Titel enthlt.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksAttributgruppeTitel(final Composite parent) {

		compositeLinksAttributgruppeTitel = new Composite(parent, SWT.NONE);

		final Composite compositeLinksAttributgruppeTitelLinks = new Composite(
				compositeLinksAttributgruppeTitel, SWT.NONE);
		final Composite compositeLinksAttributgruppeTitelRechts = new Composite(
				compositeLinksAttributgruppeTitel, SWT.NONE);

		final Label labelAttributgruppe = new Label(
				compositeLinksAttributgruppeTitelLinks, SWT.NONE);
		labelAttributgruppe.setText("Attributgruppe");

		textAnzahlAttributgruppe = new Text(
				compositeLinksAttributgruppeTitelRechts, SWT.BORDER);
		textAnzahlAttributgruppe.setEditable(false);
		textAnzahlAttributgruppe.setText("0 / 0");

		schaltflaecheLoescheAttributgruppe = new Button(
				compositeLinksAttributgruppeTitelRechts, SWT.PUSH);
		schaltflaecheLoescheAttributgruppe.setText("X");
		schaltflaecheLoescheAttributgruppe.setFont(buttonFont);

		schaltflaecheLoescheAttributgruppe
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						listeSWTAttributListe.deselectAll();
					}
				});

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		compositeLinksAttributgruppeTitel.setLayout(gridLayout);

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		compositeLinksAttributgruppeTitel.setLayoutData(gridData);

		compositeLinksAttributgruppeTitelLinks.setLayout(new GridLayout());

		final GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = true;
		rowLayoutRechts.justify = false;

		compositeLinksAttributgruppeTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksAttributgruppeTitelRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung der Liste der
	 * Attributgruppen enthlt.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksAttributgruppe(final Composite parent) {
		compositeLinksAttributgruppe = new Composite(parent, SWT.NONE);
		// compositeObjekttyp.setLayout(gridLayout);
		listeSWTAttributListe = new List(compositeLinksAttributgruppe,
				SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		listeSWTAttributListe.setBounds(0, 0, 300, 200);

		listeSWTAttributListe.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				if (listeSWTAttributListe.getSelectionIndex() > -1) {
					// _selektionAttributgruppe = _listeSWTAttributListe
					// .getSelectionIndices();

					berechneAnzahlAttribute();

					fuelleTreeMapAspekteFiltered(listeSWTAttributListe
							.getSelection());
					fuellelisteSWTObjekteGefiltert();
				}

			}
		});

		final GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.grabExcessVerticalSpace = true;
		gridDataUnten.verticalAlignment = GridData.FILL;
		compositeLinksAttributgruppe.setLayout(new FillLayout());
		compositeLinksAttributgruppe.setLayoutData(gridDataUnten);

	}

	/**
	 * Generiert den Titel des linken Aspekt-Composites.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksAspektTitel(final Composite parent) {

		compositeLinksAspektTitel = new Composite(parent, SWT.NONE);

		final Composite compositeLinksAspektTitelLinks = new Composite(
				compositeLinksAspektTitel, SWT.NONE);
		final Composite compositeLinksAspektTitelRechts = new Composite(
				compositeLinksAspektTitel, SWT.NONE);

		final Label labelAspektTitel = new Label(
				compositeLinksAspektTitelLinks, SWT.NONE);
		labelAspektTitel.setText("Aspekt");

		textAnzahlAspekte = new Text(compositeLinksAspektTitelRechts,
				SWT.BORDER);
		textAnzahlAspekte.setText("0 / 0");
		textAnzahlAspekte.setEditable(false);

		schaltflaecheLoescheAspekt = new Button(
				compositeLinksAspektTitelRechts, SWT.PUSH);
		schaltflaecheLoescheAspekt.setText("X");
		schaltflaecheLoescheAspekt.setFont(buttonFont);

		schaltflaecheLoescheAspekt.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				listeSWTAspektListe.deselectAll();
			}
		});

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		compositeLinksAspektTitel.setLayout(gridLayout);
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		compositeLinksAspektTitel.setLayoutData(gridData);

		compositeLinksAspektTitelLinks.setLayout(new GridLayout());

		final GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = false;
		rowLayoutRechts.justify = false;
		compositeLinksAspektTitelRechts.setLayout(rowLayoutRechts);
		compositeLinksAspektTitelRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung der Liste der Aspekte
	 * enthlt.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeLinksAspekt(final Composite parent) {
		compositeLinksAspekt = new Composite(parent, SWT.NONE);
		// compositeObjekttyp.setLayout(gridLayout);
		listeSWTAspektListe = new List(compositeLinksAspekt, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL);
		listeSWTAspektListe.setBounds(0, 0, 300, 200);

		listeSWTAspektListe.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(final Event e) {

				if (listeSWTAspektListe.getSelectionIndex() > -1) {
					// _selektionAspekt =
					// _listeSWTAspektListe.getSelectionIndices();

					berechneAnzahlAspekte();
					fuellelisteSWTObjekteGefiltert();
				}

			}
		});

		final GridData gridDataUnten = new GridData();
		gridDataUnten.horizontalAlignment = GridData.FILL;
		gridDataUnten.grabExcessHorizontalSpace = true;
		gridDataUnten.grabExcessVerticalSpace = true;
		gridDataUnten.verticalAlignment = GridData.FILL;
		compositeLinksAspekt.setLayout(new FillLayout());
		compositeLinksAspekt.setLayoutData(gridDataUnten);

	}

	/**
	 * Generiert das linke Composite, dass die Darstellung des
	 * Simulationsvarianten-Spinners enthlt.
	 * 
	 * @param parent
	 */
	private void erzeugeCompositeLinksSimulationsVariante(final Composite parent) {

		compositeLinksSimulationsvariante = new Composite(parent, SWT.NONE);
		final Label labelSimulationsVariante = new Label(
				compositeLinksSimulationsvariante, SWT.NONE);
		labelSimulationsVariante.setText("Simulationsvariante:");

		spinnerSimulationsvariante = new Spinner(
				compositeLinksSimulationsvariante, SWT.BORDER);

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		compositeLinksSimulationsvariante.setLayout(gridLayout);

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		compositeLinksSimulationsvariante.setLayoutData(gridData);

		final GridData gridDataSpinnerSimulationsvariante = new GridData();
		gridDataSpinnerSimulationsvariante.horizontalAlignment = GridData.FILL;
		gridDataSpinnerSimulationsvariante.grabExcessHorizontalSpace = true;
		gridDataSpinnerSimulationsvariante.verticalAlignment = GridData.CENTER;

		spinnerSimulationsvariante
				.setLayoutData(gridDataSpinnerSimulationsvariante);
	}

	private void erzeugeCompositeRechts(final Composite parent) {

		compositeRechts = new Composite(parent, SWT.BORDER);
		compositeRechts.setLayout(new FillLayout());

		final SashForm formRechts = new SashForm(compositeRechts, SWT.VERTICAL);
		final Composite obj = new Composite(formRechts, SWT.NONE);
		final Composite info = new Composite(formRechts, SWT.NONE);
		// Composite kontrolle = new Composite(formRechts, SWT.NONE);
		formRechts.setWeights(new int[] { 70, 30 });

		final FillLayout fillLayout = new FillLayout();
		obj.setLayout(fillLayout);
		info.setLayout(fillLayout);
		// kontrolle.setLayout(fillLayout);

		final Composite objComplex = new Composite(obj, SWT.NONE);
		objComplex.setLayout(new GridLayout());
		erzeugeCompositeRechtsOben(objComplex);
		erzeugeCompositeRechtsMitte(objComplex);

		final Composite infoComplex = new Composite(info, SWT.NONE);
		infoComplex.setLayout(new GridLayout());
		erzeugeCompositeRechtsInfoBereich(infoComplex);

		// Composite kontrolleComplex = new Composite(kontrolle, SWT.NONE);
		// kontrolleComplex.setLayout(new GridLayout());
		// erzeugeCompositeRechtsUnten(kontrolleComplex);

		// erzeugeCompositeRechtsOben(_compositeRechts);
		// erzeugeCompositeRechtsMitte(_compositeRechts);
		// erzeugeCompositeRechtsUnten(_compositeRechts);
		// erzeugeCompositeRechtsInfoBereich(_compositeRechts);
		//
		// GridLayout gridLayout = new GridLayout();
		// gridLayout.numColumns = 1;
		// _compositeRechts.setLayout(gridLayout);
		//
		// GridData gridData = new GridData();
		// gridData.horizontalAlignment = GridData.CENTER;
		// gridData.verticalAlignment = GridData.BEGINNING;
		// _compositeRechts.setLayoutData(gridData);

	}

	/**
	 * Generiert das obere, rechte Composite, dass den Titel der Objekte
	 * enthlt.
	 * 
	 * @param parent
	 *            Parent-Composite
	 */
	private void erzeugeCompositeRechtsOben(final Composite parent) {
		compositeRechtsOben = new Composite(parent, SWT.NONE);

		final Composite compositeRechtsObenLinks = new Composite(
				compositeRechtsOben, SWT.NONE);
		final Composite compositeRechtsObenRechts = new Composite(
				compositeRechtsOben, SWT.NONE);

		final Label labelObjekte = new Label(compositeRechtsObenLinks, SWT.NONE);
		labelObjekte.setText("Objekte");

		textAnzahlObjekte = new Text(compositeRechtsObenRechts, SWT.BORDER);
		textAnzahlObjekte.setEditable(false);
		textAnzahlObjekte.setText("0 / 0");

		schaltflaecheLoescheSelektierteObjekte = new Button(
				compositeRechtsObenRechts, SWT.PUSH);
		schaltflaecheLoescheSelektierteObjekte.setText("X");
		schaltflaecheLoescheSelektierteObjekte.setFont(buttonFont);

		schaltflaecheLoescheSelektierteObjekte
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						listeSWTObjektListe.deselectAll();
					}
				});

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		compositeRechtsOben.setLayout(gridLayout);

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		compositeRechtsOben.setLayoutData(gridData);

		compositeRechtsObenLinks.setLayout(new GridLayout());

		final GridData gridDataRechts = new GridData();
		gridDataRechts.horizontalAlignment = GridData.END;
		gridDataRechts.grabExcessHorizontalSpace = true;
		gridDataRechts.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayoutRechts = new RowLayout();
		rowLayoutRechts.pack = true;
		rowLayoutRechts.fill = false;
		rowLayoutRechts.wrap = false;
		rowLayoutRechts.justify = false;

		compositeRechtsObenRechts.setLayout(rowLayoutRechts);
		compositeRechtsObenRechts.setLayoutData(gridDataRechts);

	}

	/**
	 * Erzeugt ein {@link Composite} rechts unten. Dieses beinhaltet die Buttons
	 * OK und Abbreche.
	 * 
	 * @param parent
	 *            Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *            erfolgen soll.
	 */
	private void erzeugeCompositeRechtsUnten(final Composite parent) {
		compositeRechtsUnten = new Composite(parent, SWT.NONE);

		schaltflaecheOk = new Button(compositeRechtsUnten, SWT.PUSH);
		schaltflaecheOk.setText("OK");

		schaltflaecheOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				shell.dispose();
			}
		});

		schaltflaecheAbbrechen = new Button(compositeRechtsUnten, SWT.PUSH);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				shell.dispose();
			}
		});

		schaltflaecheHilfe = new Button(compositeRechtsUnten, SWT.PUSH);
		schaltflaecheHilfe.setText("Hilfe");
		schaltflaecheHilfe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				final String hilfeKontextId = RwCompatActivator.PLUGIN_ID
						+ ".objektzusammenstellung";
				PlatformUI.getWorkbench().getHelpSystem()
						.displayHelp(hilfeKontextId);
			}
		});

		/*
		 * Erzeugen von Buttons gleicher Gre...
		 */
		final RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.pack = false;
		rowLayout.fill = false;

		/*
		 * ...und am Ende (rechts) mittels GridData in einem GridLayout
		 * platzieren
		 */
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.END;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		compositeRechtsUnten.setLayoutData(gridData);
		compositeRechtsUnten.setLayout(rowLayout);

	}

	/**
	 * Generiert das rechte, mittlere Composite, dass die Darstellung der Liste
	 * der Objekte enthlt. Diese Darstellung ist essentiell fr die Auswahl der
	 * Objekte.
	 * 
	 * @param parent
	 */
	private void erzeugeCompositeRechtsMitte(final Composite parent) {
		final RowLayout rowLayout = new RowLayout();
		rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
		rowLayout.fill = true;
		compositeRechtsMitte = new Composite(parent, SWT.NONE);
		// compositeRechtsMitte.setLayout(rowLayout);
		listeSWTObjektListe = new List(compositeRechtsMitte, SWT.BORDER
				| SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		listeSWTObjektListe.setBounds(0, 0, 300, 200);

		listeSWTObjektListe.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				if (listeSWTObjektListe.getSelectionIndex() > -1) {
					// Selektierte Elemente in selektionObjekttyp ablegen
					// _selektionObjekte =
					// _listeSWTObjektListe.getSelectionIndices();

					berechneAnzahlObjekte();
					agregiereSystemObjectAuswahl();
				}

				final String tempSelect = listeSWTObjektListe.getSelection()[0];

				final SystemObject so = treeMapObjekt.get(tempSelect)
						.getSystemObjekt();

				fuelleInfoBereich(so.getInfo().getShortInfo());

			}
		});

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeRechtsMitte.setLayout(new FillLayout());
		compositeRechtsMitte.setLayoutData(gridData);
	}

	/**
	 * Generiert das Composite Rechts mit Infobereich.
	 * 
	 * @param parent
	 *            Parent Composite
	 */
	private void erzeugeCompositeRechtsInfoBereich(final Composite parent) {
		compositeRechtsInfoBereich = new Composite(parent, SWT.NONE);
		gruppeInfo = new Group(compositeRechtsInfoBereich, SWT.NONE);
		gruppeInfo.setText("Information");

		infoText = new Text(gruppeInfo, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL
				| SWT.H_SCROLL);

		infoText.setEditable(false);

		compositeRechtsInfoBereich.pack();

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeRechtsInfoBereich.setLayout(new FillLayout());
		compositeRechtsInfoBereich.setLayoutData(gridData);

		gruppeInfo.setLayout(new FillLayout());
	}

	/**
	 * Anzeigen des Datenidentifikations-Dialog. Nach dem Schliessen des Dialogs
	 * gibt diese Methode die angewhlten System-Objekte zurck.
	 * 
	 * @return angewhlte System-Objekte oder <code>null</code> bei keiner
	 *         Auswahl bzw. Abbrechen.
	 */
	// public SystemObject[] anzeigen()
	// {
	// return null;
	// }
	/**
	 * Fllt smtliche, bentigte Listen.
	 * 
	 */
	private void fuelleAlles() {
		if (connection != null) {
			final DataModel dataModel = connection.getDataModel();

			try {
				final SystemObjectType type = dataModel
						.getType("typ.konfigurationsBereich");

				// Liste aller Systemobjekte
				final java.util.List<SystemObject> objects = type.getObjects();

				final Iterator<SystemObject> name = objects.iterator();

				while (name.hasNext()) {
					final ConfigurationArea element = (ConfigurationArea) name
							.next();

					final Collection<SystemObject> currentObjects = element
							.getCurrentObjects();

					final Iterator<SystemObject> name2 = currentObjects
							.iterator();

					while (name2.hasNext()) {
						final SystemObject element1 = name2.next();
						final ListenElementObjekt elemObjekttyp = new ListenElementObjekt(
								element1);

						// Objekte
						fuelleObjektListe(elemObjekttyp);

						// ObjektTypen
						final ListenElementObjektTyp elemObjTyp = new ListenElementObjektTyp(
								element1);
						fuelleObjektTypListe(elemObjTyp);

					}
				}
			} catch (final RuntimeException e) {
				e.printStackTrace();
			}

			// Darstellung
			fuellelisteSWTObjekte();
			fuellelisteSWTObjektTyp();

		}

	}

	/**
	 * Fllt die Liste ObjekttypListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 * 
	 */
	private void fuelleObjektListe(final ListenElementObjekt elementObjekt) {
		treeMapObjekt.put(elementObjekt.getName(), elementObjekt);
	}

	/**
	 * Fllt die Liste AttributListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleAttributListe(final ListenElementAttributGruppe lAtt) {
		if (!treeMapAttributgruppe.containsKey(lAtt.getName())) {
			treeMapAttributgruppe.put(lAtt.getName(), lAtt);
		}
	}

	/**
	 * Fllt die Liste AspektListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleAspektListe(final ListenElementAspekt lAsp) {
		if (!treeMapAspekt.containsKey(lAsp.getName())) {
			treeMapAspekt.put(lAsp.getName(), lAsp);
		}

	}

	/**
	 * Fllt die Liste ObjektListe Wird i.d.R. aufgerufen, wenn in einer der
	 * Listen ein Selektion-Event eingetreten ist.
	 */
	private void fuelleObjektTypListe(final ListenElementObjektTyp lTyp) {

		if (!treeMapObjektTyp.containsKey(lTyp.getName())) {
			treeMapObjektTyp.put(lTyp.getName(), lTyp);
		}
	}

	/**
	 * Fllt den TreeMapAttributgruppe mit gefilterten Attributen, die den in
	 * der listeSWTObjektTyp selektierten Elementen vorkommenden Attributgruppen
	 * entsprechen.
	 * 
	 */
	private void fuelleTreeMapAttributgruppeFiltered(
			final String[] filterObjektTypen) {
		// Nur die Elemente aus der Filterselektion anzeigen
		if (filterObjektTypen.length > 0) {
			// Filter enthlt die Strings von Objekttypen
			treeMapAttributgruppe.clear();
			for (final String element : filterObjektTypen) {
				final SystemObject so = treeMapObjektTyp.get(element)
						.getSystemObject();
				final java.util.List<AttributeGroup> attributeGroups = so
						.getType().getAttributeGroups();
				final Iterator<AttributeGroup> iterator = attributeGroups
						.iterator();
				while (iterator.hasNext()) {
					final AttributeGroup tempAtg = iterator.next();
					final ListenElementAttributGruppe elementAttribut = new ListenElementAttributGruppe(
							tempAtg);
					fuelleAttributListe(elementAttribut);
				}
			}

		} else {
			final Iterator<Entry<String, ListenElementObjektTyp>> it = treeMapObjektTyp
					.entrySet().iterator();
			treeMapAttributgruppe.clear();
			while (it.hasNext()) {
				final Entry<String, ListenElementObjektTyp> entry = it.next();
				// Map.Entry entry = (Map.Entry) it.next();
				final ListenElementObjektTyp tempObjektTyp = (entry.getValue());
				final SystemObject so = tempObjektTyp.getSystemObject();
				final java.util.List<AttributeGroup> attributeGroups = so
						.getType().getAttributeGroups();
				final Iterator<AttributeGroup> iterator = attributeGroups
						.iterator();
				while (iterator.hasNext()) {
					final AttributeGroup tempAtg = iterator.next();
					final ListenElementAttributGruppe elementAttribut = new ListenElementAttributGruppe(
							tempAtg);
					fuelleAttributListe(elementAttribut);
				}

			}

		}

		listeSWTAttributListe.setVisible(false);
		listeSWTAttributListe.removeAll();
		fuellelisteSWTAttribute();
		listeSWTAttributListe.setVisible(true);

		final int itemCount = listeSWTAttributListe.getItemCount();
		textAnzahlAttributgruppe.setText("0 / " + itemCount);
		textAnzahlAttributgruppe.getParent().pack();

		fuelleTreeMapAspekteFiltered(listeSWTAttributListe.getSelection());

	}

	private void fuelleTreeMapAspekteFiltered(
			final String[] filterAttributgruppen) {
		treeMapAspekt.clear();

		if (filterAttributgruppen.length > 0) {
			for (final String element : filterAttributgruppen) {
				final ListenElementAttributGruppe listenElementAttributGruppe = treeMapAttributgruppe
						.get(element);

				final Collection<Aspect> aspects = listenElementAttributGruppe
						.getAttributGruppe().getAspects();

				final Iterator<Aspect> aspectIterator = aspects.iterator();
				while (aspectIterator.hasNext()) {
					final Aspect tempAspect = aspectIterator.next();
					final ListenElementAspekt elementAspekt = new ListenElementAspekt(
							tempAspect);
					fuelleAspektListe(elementAspekt);
				}
			}
		} else {
			final Iterator<Entry<String, ListenElementAttributGruppe>> it = treeMapAttributgruppe
					.entrySet().iterator();
			// Iterator it = _treeMapAttributgruppe.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, ListenElementAttributGruppe> entry = it
						.next();
				// Map.Entry entry = (Map.Entry) it.next();
				final ListenElementAttributGruppe tempAttributGruppe = (entry
						.getValue());
				final Collection<Aspect> aspects = tempAttributGruppe
						.getAttributGruppe().getAspects();
				final Iterator<Aspect> aspectIterator = aspects.iterator();
				while (aspectIterator.hasNext()) {
					final Aspect tempAspect = aspectIterator.next();
					final ListenElementAspekt elementAspekt = new ListenElementAspekt(
							tempAspect);
					fuelleAspektListe(elementAspekt);
				}
			}

		}

		listeSWTAspektListe.removeAll();
		fuellelisteSWTAspekte();
	}

	/*
	 * Fllen der SWT-Listen im GUI
	 */

	/**
	 * Fllt die Liste listeSWTOAttributLise.
	 */
	private void fuellelisteSWTAttribute() {
		final Iterator<Entry<String, ListenElementAttributGruppe>> it = treeMapAttributgruppe
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, ListenElementAttributGruppe> entry = it.next();
			listeSWTAttributListe.add(entry.getKey());
		}

		berechneAnzahlAttribute();
	}

	/**
	 * Fllt die Liste listeSWTAspektListe.
	 * 
	 */
	private void fuellelisteSWTAspekte() {
		final Iterator<Entry<String, ListenElementAspekt>> it = treeMapAspekt
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, ListenElementAspekt> entry = it.next();
			listeSWTAspektListe.add(entry.getKey());
		}

		berechneAnzahlAspekte();
	}

	/**
	 * Fllt die Liste listeSWTObjektListe.
	 */
	private void fuellelisteSWTObjekte() {
		final Iterator<Entry<String, ListenElementObjekt>> it = treeMapObjekt
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, ListenElementObjekt> entry = it.next();
			listeSWTObjektListe.add(entry.getKey());
		}

		berechneAnzahlObjekte();
	}

	/**
	 * Fllt die Objektliste mit Objekten vom Typ der in der Liste Objekttyp
	 * selektierten Objekttypen.
	 * 
	 * @param filterObjektTypen
	 */
	private void fuellelisteSWTObjekteFiltered(final String[] filterObjektTypen) {
		// temporre TreeMap anlegen, in der die anzuzeigenden Werte enthalten
		// sind
		final TreeMap<String, ListenElementObjekt> tempObjektMap = new TreeMap<String, ListenElementObjekt>();

		for (final String element : filterObjektTypen) {
			final Iterator<Entry<String, ListenElementObjekt>> it = treeMapObjekt
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, ListenElementObjekt> entry = it.next();
				final ListenElementObjekt tempObjektElement = entry.getValue();
				final SystemObject so = tempObjektElement.getSystemObjekt();

				// ListenElementObjektTyp tempObjektTypElement =
				// TreeMapObjektTyp.get(filterObjektTypen[i]);

				if ((so.getType().toString()).equals(element)) {

					if (!tempObjektMap.containsKey(tempObjektElement.getName())) {
						tempObjektMap.put(tempObjektElement.getName(),
								tempObjektElement);
					}

				}
			}
		}

		listeSWTObjektListe.setVisible(false);
		listeSWTObjektListe.removeAll();

		final Iterator<Entry<String, ListenElementObjekt>> it = tempObjektMap
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, ListenElementObjekt> entry = it.next();
			listeSWTObjektListe.add(entry.getKey());
		}

		listeSWTObjektListe.setVisible(true);

		final int itemCount = listeSWTObjektListe.getItemCount();
		textAnzahlObjekte.setText("0 / " + itemCount);
		textAnzahlObjekte.getParent().pack();

	}

	private void fuellelisteSWTObjekteGefiltert() {

		final String[] filterObjektTypen = listeSWTObjekttypListe
				.getSelection();
		final String[] filterAttributgruppen = listeSWTAttributListe
				.getSelection();
		final String[] filterAspekt = listeSWTAspektListe.getSelection();

		// temporre TreeMap anlegen, in der die anzuzeigenden Werte enthalten
		// sind
		final TreeMap<String, ListenElementObjekt> tempObjektTypMap = new TreeMap<String, ListenElementObjekt>();
		final TreeMap<String, ListenElementObjekt> tempAspektMap = new TreeMap<String, ListenElementObjekt>();
		final TreeMap<String, ListenElementObjekt> tempAttributGruppeMap = new TreeMap<String, ListenElementObjekt>();
		final TreeMap<String, ListenElementObjekt> tempResultMap = new TreeMap<String, ListenElementObjekt>();

		for (final String element : filterObjektTypen) {
			final Iterator<Entry<String, ListenElementObjekt>> it = treeMapObjekt
					.entrySet().iterator();
			while (it.hasNext()) {
				final Entry<String, ListenElementObjekt> entry = it.next();
				final ListenElementObjekt tempObjektElement = entry.getValue();
				final SystemObject so = tempObjektElement.getSystemObjekt();

				// ListenElementObjektTyp tempObjektTypElement =
				// TreeMapObjektTyp.get(filterObjektTypen[i]);

				// Prfung auf Objekt vom gleichen ObjektTyp
				if ((so.getType().toString()).equals(element)) {

					// Prfung auf gleiche Attributgruppe

					if (!tempObjektTypMap.containsKey(tempObjektElement
							.getName())) {
						tempObjektTypMap.put(tempObjektElement.getName(),
								tempObjektElement);
					}

				}
			}

		}

		// Filter ber die Attributgruppen tempAttributGruppeMap
		// Iterator ber alle Objekte
		Iterator<Entry<String, ListenElementObjekt>> it = treeMapObjekt
				.entrySet().iterator();
		while (it.hasNext()) {
			// System Objekte aus TreeMap holen
			final Entry<String, ListenElementObjekt> entry = it.next();
			final ListenElementObjekt tempObjektElement = entry.getValue();
			final SystemObject so = tempObjektElement.getSystemObjekt();

			// Iterator zu den Attributen der SystemObjekte bilden
			final java.util.List<AttributeGroup> listAttributeGroups = so
					.getType().getAttributeGroups();
			final Iterator<AttributeGroup> itAttributeGroups = listAttributeGroups
					.iterator();

			while (itAttributeGroups.hasNext()) {
				final AttributeGroup tempAttributgruppeSystemObjekt = itAttributeGroups
						.next();
				for (final String element : filterAttributgruppen) {
					// AttributGruppe der selektierten Attributgruppe in der
					// SWT-Liste
					final ListenElementAttributGruppe tempAttributGruppe = treeMapAttributgruppe
							.get(element);

					if (tempAttributGruppe.getAttributGruppe().getName()
							.equals(tempAttributgruppeSystemObjekt.getName())) {
						if (!tempAttributGruppeMap
								.containsKey(tempObjektElement.getName())) {
							tempAttributGruppeMap.put(
									tempObjektElement.getName(),
									tempObjektElement);
						}
					}
				}
			}

		}

		// ////
		// Filter ber Aspekte tempAspektMap
		// Iterator ber alle Objekte

		it = treeMapObjekt.entrySet().iterator();

		while (it.hasNext()) {

			// System Objekte aus TreeMap holen
			final Entry<String, ListenElementObjekt> entry = it.next();
			final ListenElementObjekt tempObjektElement = entry.getValue();
			final SystemObject so = tempObjektElement.getSystemObjekt();

			// Iterator zu den Attributen der SystemObjekte bilden
			final java.util.List<AttributeGroup> listAttributeGroups = so
					.getType().getAttributeGroups();
			final Iterator<AttributeGroup> itAttributeGroups = listAttributeGroups
					.iterator();

			while (itAttributeGroups.hasNext()) {
				final AttributeGroup tempAttributgruppeSystemObjekt = itAttributeGroups
						.next();
				final Collection<Aspect> aspects = tempAttributgruppeSystemObjekt
						.getAspects();
				final Iterator<Aspect> itAspects = aspects.iterator();
				while (itAspects.hasNext()) {
					final Aspect tempAspect = itAspects.next();
					for (final String element : filterAspekt) {
						final ListenElementAspekt asp = treeMapAspekt
								.get(element);

						if (asp.getAspekt().equals(tempAspect)) {
							if (!tempAspektMap.containsKey(tempObjektElement
									.getName())) {
								tempAspektMap.put(tempObjektElement.getName(),
										tempObjektElement);
							}
						}

					}
				}

			}

		}

		// ////
		// Ergebnisse in den Maps zusammenfgen
		// TreeMap<String, ListenElementObjekt> tempObjektTypMap = new
		// TreeMap<String, ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempAspektMap = new
		// TreeMap<String,
		// ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempAttributGruppeMap = new
		// TreeMap<String, ListenElementObjekt>();
		// TreeMap<String, ListenElementObjekt> tempResultMap = new
		// TreeMap<String,
		// ListenElementObjekt>();

		// Ergebnisse aus tempObjektTypMap
		Iterator<Entry<String, ListenElementObjekt>> mergeIt = tempObjektTypMap
				.entrySet().iterator();
		while (mergeIt.hasNext()) {
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			final Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = entry.getValue();

			if (tempAspektMap.size() > 0) {
				if (!tempAspektMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAttributGruppeMap.size() > 0) {
				if (!tempAttributGruppeMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps) {
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		// Ergebnisse aus tempAttributGruppeMap
		mergeIt = tempAttributGruppeMap.entrySet().iterator();
		while (mergeIt.hasNext()) {
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			final Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = entry.getValue();

			if (tempObjektTypMap.size() > 0) {
				if (!tempObjektTypMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAspektMap.size() > 0) {
				if (!tempAspektMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps) {
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		// Ergebnisse aus tempAspektMap
		mergeIt = tempAspektMap.entrySet().iterator();
		while (mergeIt.hasNext()) {
			boolean vorhandenInAnderenMaps = true;
			ListenElementObjekt tempListenElementObjekt = null;
			final Entry<String, ListenElementObjekt> entry = mergeIt.next();
			tempListenElementObjekt = entry.getValue();

			if (tempObjektTypMap.size() > 0) {
				if (!tempObjektTypMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if (tempAttributGruppeMap.size() > 0) {
				if (!tempAttributGruppeMap.containsKey(tempListenElementObjekt
						.getName())) {
					vorhandenInAnderenMaps = false;
				}
			}

			if ((!tempResultMap.containsKey(tempListenElementObjekt.getName()))
					&& vorhandenInAnderenMaps) {
				tempResultMap.put(tempListenElementObjekt.getName(),
						tempListenElementObjekt);
			}
		}

		listeSWTObjektListe.setVisible(false);
		listeSWTObjektListe.removeAll();

		final Iterator<Entry<String, ListenElementObjekt>> outIt = tempResultMap
				.entrySet().iterator();
		while (outIt.hasNext()) {
			// System Objekte aus TreeMap holen
			final Entry<String, ListenElementObjekt> entry = outIt.next();
			final ListenElementObjekt tempObjektElement = entry.getValue();
			listeSWTObjektListe.add(tempObjektElement.getName());
		}

		listeSWTObjektListe.setVisible(true);

		// int itemCount = _listeSWTObjektListe.getItemCount();
		// _textAnzahlObjekte.setText("0 / " + itemCount);
		// _textAnzahlObjekte.getParent().pack();

		berechneAnzahlObjekte();

	}

	/**
	 * Fllt die Liste listeSWTObjektTyp.
	 */
	private void fuellelisteSWTObjektTyp() {
		final Iterator<Entry<String, ListenElementObjektTyp>> it = treeMapObjektTyp
				.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, ListenElementObjektTyp> entry = it.next();
			listeSWTObjekttypListe.add(entry.getKey());
		}

		fuelleTreeMapAttributgruppeFiltered(listeSWTObjekttypListe
				.getSelection());

		berechneAnzahlObjekttypen();
	}

	/**
	 * Entfernt aus TreeMapObjektTyp alle ObjektTypen, die nicht denjenigen aus
	 * sFilter entsprechen.
	 * 
	 * @param sFilter
	 *            String-Arrays, enthlt die Pids der bentigten ObjektTypen
	 */
	private void entferneObjektTypen(final String[] sFilter) {
		// Enthlt die Keys der zu lschenden ObjektTypen
		final LinkedList<String> loeschListe = new LinkedList<String>();

		// Prfen, welche Eintrge in TreeMapObjektTyp gelscht werden sollen
		final Iterator<Entry<String, ListenElementObjektTyp>> itMapObjTyp = treeMapObjektTyp
				.entrySet().iterator();
		while (itMapObjTyp.hasNext()) {
			final Entry<String, ListenElementObjektTyp> entry = itMapObjTyp
					.next();
			final ListenElementObjektTyp tempLEObjTyp = entry.getValue();

			boolean loeschen = true;
			for (final String element : sFilter) {
				if (tempLEObjTyp.getSystemObject().getType().getPid()
						.equals(element)) {
					// nicht lschen
					loeschen = false;
				}

			}

			if (loeschen) {
				loeschListe.add(entry.getKey());
			}

		}

		// Lschen der Entsprechenden Eintrge
		final Iterator<String> it = loeschListe.iterator();
		while (it.hasNext()) {
			treeMapObjektTyp.remove(it.next());
		}
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen
	 * Objekttypen im Feld _textAnzahlObjekttyp an.
	 */
	private void berechneAnzahlObjekttypen() {
		final int selektiert = listeSWTObjekttypListe.getSelectionCount();
		final int anzahlGesamt = listeSWTObjekttypListe.getItemCount();

		final String printString = "" + selektiert + " / " + anzahlGesamt;
		textAnzahlObjekttyp.setText(printString);
		textAnzahlObjekttyp.getParent().pack();
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen
	 * Attributgruppen im Feld _textAnzahlAttributgruppe an.
	 */
	private void berechneAnzahlAttribute() {
		final int selektiert = listeSWTAttributListe.getSelectionCount();
		final int anzahlGesamt = listeSWTAttributListe.getItemCount();

		final String printString = "" + selektiert + " / " + anzahlGesamt;
		textAnzahlAttributgruppe.setText(printString);
		textAnzahlAttributgruppe.getParent().pack();
	}

	/**
	 * Zeigt die Anzahl der selektierten und der insgesamt vorhandenen Aspekte
	 * im Feld _textAnzahlAspekte an.
	 */
	private void berechneAnzahlAspekte() {
		final int selektiert = listeSWTAspektListe.getSelectionCount();
		final int anzahlGesamt = listeSWTAspektListe.getItemCount();

		final String printString = "" + selektiert + " / " + anzahlGesamt;
		textAnzahlAspekte.setText(printString);
		textAnzahlAspekte.getParent().pack();
	}

	/**
	 * Zeigt die anzahl der selektierten und der insgesamt vorhandenen Objekte
	 * im Feld _textAnzahlObjekte an.
	 */
	private void berechneAnzahlObjekte() {
		final int selektiert = listeSWTObjektListe.getSelectionCount();
		final int anzahlGesamt = listeSWTObjektListe.getItemCount();

		final String printString = "" + selektiert + " / " + anzahlGesamt;
		textAnzahlObjekte.setText(printString);
		textAnzahlObjekte.getParent().pack();
	}

	/*
	 * Hilfsmethoden
	 */
	/**
	 * Fllt den Infobereich.
	 * 
	 * @param Text
	 *            , der im Infobereich erscheinen soll
	 */
	private void fuelleInfoBereich(final String text) {
		infoText.setText(text);
		shell.layout();
	}

	/**
	 * Legt in der Variable systemObjectAuswahl alle in der Liste SystemObjekte
	 * selektierten Elemente an.
	 */
	private void agregiereSystemObjectAuswahl() {
		final String[] auswahl = listeSWTObjektListe.getSelection();
		final SystemObject[] returnSystemObject = new SystemObject[auswahl.length];
		for (int i = 0; i < auswahl.length; i++) {
			returnSystemObject[i] = treeMapObjekt.get(auswahl[i])
					.getSystemObjekt();
		}

		systemObjectAuswahl = returnSystemObject;
	}

	/**
	 * Enummerator fr die Darstellung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Darstellung {
		/**
		 * Folgende Auswahllisten werden angezeigt.<br>
		 * - Objekt-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE,

		/**
		 * Folgende Auswahllisten werden angezeigt.<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE_OBJEKTTYP,

		/**
		 * Folgende Auswahllisten werden angezeigt.<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		OBJEKTELISTE_OBJEKTTYP_ATG,

		/**
		 * Folgende Auswahllisten werden angezeigt.<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Aspekt-Liste<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		// OBJEKTELISTE_OBJEKTTYP_ATG_ASP,
		OBJEKTELISTE_OBJEKTTYP_ATG_ASP

		/**
		 * Folgende Auswahllisten werden angezeigt:<br>
		 * - Objekt-Liste<br>
		 * - Objekttyp-Liste<br>
		 * - Attributgruppen-Liste<br>
		 * - Aspekt-Liste<br>
		 * - Simulations-Variante<br>
		 * - Filter, wenn <code>filter</code> ungleich <code>null</code>
		 */
		// OBJEKTELISTE_OBJEKTTYP_ATG_ASP_SIMUVAR,
	}

	/*
	 * Reprsentation der jeweiligen Elemente in den Listen
	 */

	/**
	 * Reprsentiert die Listenelemente vom Typ Objekttyp.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListenElementObjekt {
		private String name;

		private SystemObject systemObject;

		public ListenElementObjekt(final SystemObject sysObj) {
			setSystemObject(sysObj);
		}

		public String getName() {
			return name;
		}

		private void setName(final String name) {
			this.name = name;
		}

		public SystemObject getSystemObjekt() {
			return systemObject;
		}

		private void setSystemObject(final SystemObject systemObject) {
			this.systemObject = systemObject;
			String tempName = "";
			if ((systemObject.getName() != null)
					&& (!systemObject.getName().isEmpty())) {
				tempName = tempName + systemObject.getName() + " ";
			}
			if ((systemObject.getPid() != null)
					&& (!systemObject.getPid().isEmpty())) {
				tempName = tempName + "(" + systemObject.getPid() + ") ";
			}
			if (systemObject.getId() > 0) {
				tempName = tempName + "["
						+ String.valueOf(systemObject.getId()) + "]";
			}

			this.setName(tempName);

		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ Attributgruppe.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListenElementAttributGruppe {
		/**
		 * Name dieses Elements.
		 */
		private String name;

		/**
		 * Attribute dieses Elements.
		 */
		private AttributeGroup attributeGroup;

		public ListenElementAttributGruppe(final AttributeGroup wert) {
			setAttributGruppe(wert);
		}

		public String getName() {
			return name;
		}

		private void setName(final String name) {
			this.name = name;
		}

		/**
		 * Liefert das Attribut.
		 * 
		 * @return Attribut
		 */
		public AttributeGroup getAttributGruppe() {
			return attributeGroup;
		}

		/**
		 * Setzt das Attribut.
		 * 
		 * @param attributeGroup
		 */
		private void setAttributGruppe(final AttributeGroup wert) {
			this.attributeGroup = wert;
			String tempName = "";
			if ((attributeGroup.getName() != null)
					&& (!attributeGroup.getName().isEmpty())) {
				tempName = tempName + attributeGroup.getName() + " ";
			}
			if ((attributeGroup.getPid() != null)
					&& (!attributeGroup.getPid().isEmpty())) {
				tempName = tempName + "(" + attributeGroup.getPid() + ") ";
			}
			this.setName(tempName);
		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ Aspekt.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListenElementAspekt {
		/**
		 * Name dieses Elements.
		 */
		private String name;

		/**
		 * Aspect dieses Elements.
		 */
		private Aspect aspect;

		public ListenElementAspekt(final Aspect aspekt) {
			setAspekt(aspekt);
		}

		public String getName() {
			return name;
		}

		private void setName(final String name) {
			this.name = name;
		}

		public Aspect getAspekt() {
			return aspect;
		}

		private void setAspekt(final Aspect aspekt) {
			this.aspect = aspekt;
			String tempName = "";
			if ((aspect.getName() != null) && (!aspect.getName().isEmpty())) {
				tempName = tempName + aspect.getName();
			}
			if (aspect.getPid() != null) {
				tempName = tempName + " (" + aspekt.getPid() + ")";
			}
			if (aspect.getId() > 0) {
				tempName = tempName + " " + "["
						+ String.valueOf(aspect.getId()) + "]";
			}
			setName(tempName);
		}
	}

	/**
	 * Reprsentiert die Listenelemente vom Typ ObjektTyp.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListenElementObjektTyp {
		/**
		 * Liefert den Namen des ObjektTyps.
		 */
		private String name;

		/**
		 * Hlt ein SystemObjekt zwecks Auswertung und Anzeige der
		 * Attributgruppen zur Verfgung.
		 */
		private final SystemObject systemObject;

		public ListenElementObjektTyp(final SystemObject sysObj) {
			setName(sysObj.getType().toString());
			this.systemObject = sysObj;
		}

		/**
		 * Liefert den Namen des ObjektTyps.
		 * 
		 * @return Name des Objekttyps
		 */
		public String getName() {
			return name;
		}

		private void setName(final String name) {
			this.name = name;
		}

		/**
		 * Liefert das SystemObject.
		 * 
		 * @return SystemObject
		 */
		public SystemObject getSystemObject() {
			return systemObject;
		}

	}

}
