/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.datenidentifikation;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Liefert einen Dialog zur Identifiezierung von Daten.
 * 
 * @author HofmannJ
 * 
 */
public class DatenidentifikationGtm extends Dialog {
	/**
	 * Liste der darzustellenden SystemObjektTypen.
	 */
	private List<SystemObjectType> typesListe;

	/**
	 * Liste der SystemObjekte, wird fr die Rckgabe bentigt.
	 */
	private LinkedList<SystemObject> systemObjects = new LinkedList<SystemObject>();

	/**
	 * Shell zur Darstellung.
	 */
	private Shell shell;

	/**
	 * Datendarstellung.
	 */
	private GtmListeComposite compGtmListe;

	/**
	 * Verbindung zum DaV. Wird mit Hilfe der Klasse {@link DaVVerbindung}
	 * bestimmt.
	 */
	private ClientDavInterface connection;

	/**
	 * Darstellungsart.
	 */
	private int darstellung = GtmListeComposite.DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE;

	/**
	 * Dialogtitel.
	 */
	private String dialogTitel = "Datenidentifikation";

	/**
	 * ffnet den Dialog zur Daten Identifikation.
	 * 
	 * @param parent
	 *            Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *            soll.
	 * @param dialogTitel
	 *            Titel des des Dialoges. Bei <code>null</code> wird der
	 *            Default-Titel <code>Datenidentifikation</code> gesetzt.
	 */
	public DatenidentifikationGtm(final Shell parent, final String dialogTitel) {
		super(parent, 0);

		if (dialogTitel != null) {
			this.dialogTitel = dialogTitel;
		}
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param types
	 *            Liste von {@link SystemObjectType}s, die dargestellt werden
	 *            soll.
	 * @param darstellungsTyp
	 *            Art der Darstellung. Die Darstellung ist whlbar ber
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEM_OBJEKTE},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKT_TYPEN},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OJEKTTYPEN_ATG}
	 *            ,
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE}
	 *            sowie
	 *            {@link GtmListeComposite#darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}
	 *            .
	 * @return Eine Liste der ausgewhlten {@link SystemObject}s. Wurden keine
	 *         selektiert, so ist die Liste leer.
	 */
	public LinkedList<SystemObject> oeffnen(final List<SystemObjectType> types,
			final int darstellungsTyp) {
		this.typesListe = types;
		return oeffnen(darstellungsTyp);
	}

	/**
	 * ffnet den Dialog zur Datenidentifikation.
	 * 
	 * @param darstellungsTyp
	 *            Art der Darstellung. Die Darstellung ist whlbar ber
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEM_OBJEKTE},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKT_TYPEN},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OJEKTTYPEN_ATG}
	 *            ,
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE}
	 *            sowie
	 *            {@link GtmListeComposite#darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}
	 *            .
	 * @return Eine Liste der ausgewhlten {@link SystemObject}s. Wurden keine
	 *         selektiert, so ist die Liste leer.
	 */
	public LinkedList<SystemObject> oeffnen(final int darstellungsTyp) {
		this.darstellung = darstellungsTyp;

		connection = RahmenwerkService.getService().getRahmenWerk()
				.getDavVerbindung();

		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		shell.setText(dialogTitel);
		final Image shellImage = RwCompatActivator.getDefault().getImage(
				"icons/sonstiges/find_16x16.png");
		shell.setImage(shellImage);

		shell.setSize(new Point(800, 600));
		shell.setLayout(new GridLayout());

		erzeugeDarstellung();

		shell.open();

		final Display display = getParent().getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		final int displayWidth = Display.getDefault().getBounds().width;
		final int displayHeight = Display.getDefault().getBounds().height;

		final Rectangle bounds = shell.getBounds();
		final Point cursorLocation = Display.getDefault().getCursorLocation();

		int y = cursorLocation.y;
		if ((cursorLocation.y + bounds.height) > displayHeight) {
			y = displayHeight - bounds.height;
			if (y < 0) {
				y = 0;
			}
		}

		int x = cursorLocation.x;
		if (((cursorLocation.x + bounds.width) > displayWidth)) {
			x = displayWidth - bounds.width;
			if (x < 0) {
				x = 0;
			}
		}

		final Rectangle newBounds = new Rectangle(x, y, bounds.width,
				bounds.height);
		shell.setBounds(newBounds);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return systemObjects;
	}

	private void erzeugeDarstellung() {
		// GtmDatenBasis datenBasis = new GtmDatenBasis(_connection, false,
		// null);
		GtmDatenBasis datenBasis = DaVPreLader.getInstanz().getGtmDatenBasis();

		final DataModel dataModel = connection.getDataModel();

		/*
		 * SystemObjekte nach Typen bestimmen
		 */
		final SystemObjectType type = dataModel.getType("typ.messQuerschnitt");
		final LinkedList<SystemObjectType> types = new LinkedList<SystemObjectType>();
		types.add(type);

		if (typesListe == null) {
			final DaVPreLader daVPreLader = DaVPreLader.getInstanz();
			datenBasis = daVPreLader.getGtmDatenBasis();
		} else {
			datenBasis.setTypes(typesListe);
		}

		final Composite compositeOben = new Composite(shell, SWT.NONE);
		final Composite compositeUnten = new Composite(shell, SWT.NONE);

		compGtmListe = new GtmListeComposite(compositeOben, SWT.NONE,
				datenBasis, darstellung);

		final Button buttonOk = new Button(compositeUnten, SWT.NONE);
		buttonOk.setText("OK");

		final Button buttonAbbrechen = new Button(compositeUnten, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");

		buttonOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionOk();
			}
		});

		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionAbbrechen();
			}
		});

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeOben.setLayout(new GridLayout());
		compositeOben.setLayoutData(gridData);

		final RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeUnten.setLayoutData(gridData1);
		compositeUnten.setLayout(rowLayout);

		final GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = GridData.FILL;
		compGtmListe.setLayoutData(gd);
	}

	private void aktionOk() {
		systemObjects = compGtmListe.getSelektierteSystemObjekte();
		shell.dispose();
	}

	private void aktionAbbrechen() {
		systemObjects.clear();
		shell.dispose();
	}
}
