/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.datenidentifikation;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Stellt eine Listendarstellung der Daten des GTM als {@link Composite} bereit.
 * 
 * @author HofmannJ
 * 
 */
public class GtmListeComposite extends Composite {
	/**
	 * Darstellung von SystemObjekten.
	 */
	public static final int DARSTELLUNG_SYSTEM_OBJEKTE = 0;

	/**
	 * Darstellung von SystemObjekten und ObjektTypen.
	 */
	public static final int DARSTELLUNG_SYSTEMOBJEKT_TYPEN = 1;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen und AttributGruppen.
	 */
	public static final int DARSTELLUNG_SYSTEMOBJEKTE_OJEKTTYPEN_ATG = 2;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen, AttributGruppen und
	 * Aspekten.
	 */
	public static final int DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE = 3;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen, AttributGruppen, Aspekten
	 * und der Simulationsvariante.
	 */
	public static final int DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE_SIMVAR = 4;

	private int darstellungsArt = 3;

	/** Verbindung zum DaV. */
	private final GtmDatenBasis gtmDatenBasis;
	private SashForm formBasis;
	private Composite compositeLinks;
	private Composite compositeRechts;
	private Table tableSystemObjekte;
	private TableViewer tvSystemObjekte;
	private ISelectionChangedListener listenerTvSystemObjekte;
	private Table tableObjektTypen;
	private TableViewer tvObjektTypen;
	private ISelectionChangedListener listenerTvObjektTypen;
	private Table tableAttributGruppen;
	private TableViewer tvAttributGruppen;
	private ISelectionChangedListener listenerTvAttributGruppen;
	private Table tableAspekte;
	private TableViewer tvAspekte;
	private ISelectionChangedListener listenerTvAspekte;

	private GtmListeSelektionsKopf selektionObjektListe;
	private GtmListeSelektionsKopf selektionObjektTypen;
	private GtmListeSelektionsKopf selektionAttributGruppen;
	private GtmListeSelektionsKopf selektionAspekte;
	private int simulationsVariante = -1;

	/**
	 * Konstruktor zur Darstellung eines Composites zur Datenidentifikation.
	 * 
	 * @param parent
	 *            Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *            erfolgen soll.
	 * @param stil
	 *            {@link SWT}-Stil dieses {@link Composite}s.
	 * @param gtmDatenBasis
	 *            {@link GtmDatenBasis}, auf dessen Basis die Darstellung
	 *            erfolgen soll.
	 * @param darstellungsArt
	 *            Die Art der Darstellung ist whlbar ber
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEM_OBJEKTE},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKT_TYPEN},
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OJEKTTYPEN_ATG}
	 *            ,
	 *            {@link GtmListeComposite#DARSTELLUNG_SYSTEMOBJEKTE_OBJEKTTYPEN_ATTRIBUTGRUPPEN_ASPEKTE}
	 *            sowie
	 *            {@link GtmListeComposite#darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}
	 *            .
	 */
	public GtmListeComposite(final Composite parent, final int stil,
			final GtmDatenBasis gtmDatenBasis, final int darstellungsArt) {
		super(parent, stil);

		this.darstellungsArt = darstellungsArt;
		this.gtmDatenBasis = gtmDatenBasis;

		initialisierung();
	}

	private void initialisierung() {
		setLayout(new FillLayout());
		formBasis = new SashForm(this, SWT.HORIZONTAL);

		erzeugeCompositeLinks(formBasis);
		erzeugeCompositeRechts(formBasis);

		if (darstellungsArt > 0) {
			formBasis.setWeights(new int[] { 50, 50 });
		} else {
			formBasis.setWeights(new int[] { 0, 100 });
		}

		fuelleAlleListenSelektionsKoepfe();
	}

	/**
	 * Erzeugt das {@link Composite} auf der linken Seite.
	 * 
	 * @param parent
	 *            Eltern-{@link Composite}
	 */
	private void erzeugeCompositeLinks(final Composite parent) {
		compositeLinks = new Composite(parent, SWT.BORDER);
		compositeLinks.setLayout(new FillLayout());

		final SashForm formLinks = new SashForm(compositeLinks, SWT.VERTICAL);

		Composite objTyp = null;
		Composite atg = null;
		Composite asp = null;
		Composite simvar = null;

		final FillLayout fillLayout = new FillLayout();

		if (darstellungsArt >= 1) {
			objTyp = new Composite(formLinks, SWT.NONE);
			objTyp.setLayout(fillLayout);
			erzeugeCompositeObjektTypen(objTyp);
		}
		if (darstellungsArt >= 2) {
			atg = new Composite(formLinks, SWT.NONE);
			atg.setLayout(fillLayout);
			erzeugeCompositeAttributGruppen(atg);
		}
		if (darstellungsArt >= 3) {
			asp = new Composite(formLinks, SWT.NONE);
			asp.setLayout(fillLayout);
			erzeugeCompositeAspekte(asp);
		}
		if (darstellungsArt >= 4) {
			simvar = new Composite(formLinks, SWT.NONE);
			simvar.setLayout(fillLayout);
			erzeugeCompositeSimulation(simvar);
		}

		switch (darstellungsArt) {
		case 0:
			break;
		case 1:
			formLinks.setWeights(new int[] { 100 });
			break;
		case 2:
			formLinks.setWeights(new int[] { 50, 50 });
			break;
		case 3:
			formLinks.setWeights(new int[] { 33, 33, 33 });
			break;
		case 4:
			formLinks.setWeights(new int[] { 30, 30, 30, 10 });
			break;
		default:
			break;
		}

	}

	/**
	 * Erzeugt das {@link Composite} auf der rechten Seite.
	 * 
	 * @param parent
	 *            Eltern {@link Composite}
	 */
	private void erzeugeCompositeRechts(final Composite parent) {
		compositeRechts = new Composite(parent, SWT.BORDER);
		compositeRechts.setLayout(new FillLayout());

		erzeugeCompositeObjektListe(compositeRechts);
	}

	/**
	 * Erzeugt das Composite zur Darstellung der Objekt-Liste.
	 * 
	 * @param parent
	 *            Eltern-Composite
	 */
	private void erzeugeCompositeObjektListe(final Composite parent) {
		final Composite objektListe = new Composite(parent, SWT.NONE);

		final Runnable run = new Runnable() {

			@Override
			public void run() {
				System.out.println("RUN!");
				if (tvSystemObjekte != null) {
					tvSystemObjekte.setSelection(null, true);
				}
			}

		};

		selektionObjektListe = new GtmListeSelektionsKopf(objektListe,
				SWT.BORDER, "Objekte:", run);

		tableSystemObjekte = new Table(objektListe, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.V_SCROLL | SWT.H_SCROLL);
		tableSystemObjekte.setHeaderVisible(true);
		tableSystemObjekte.setLinesVisible(true);
		final TableColumn tc = new TableColumn(tableSystemObjekte, SWT.LEAD);
		tc.setText("SystemObjekte:");
		tc.setWidth(300);
		tvSystemObjekte = new TableViewer(tableSystemObjekte);
		tvSystemObjekte.setContentProvider(new ArrayContentProvider());
		tvSystemObjekte.setLabelProvider(new GtmLabelProvider());
		tvSystemObjekte.setSorter(new ViewerSorter());

		listenerTvSystemObjekte = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				fuelleAlleListenSelektionsKoepfe();
			}

		};
		tvSystemObjekte.addSelectionChangedListener(listenerTvSystemObjekte);

		inputSystemObjekte();

		final GridLayout gridLayoutCompositeObjektListe = new GridLayout(1,
				false);
		objektListe.setLayout(gridLayoutCompositeObjektListe);

		final GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		selektionObjektListe.setLayoutData(gdSelektion);

		final GridData gdListSystemObjekte = new GridData();
		gdListSystemObjekte.grabExcessHorizontalSpace = true;
		gdListSystemObjekte.horizontalAlignment = GridData.FILL;
		gdListSystemObjekte.grabExcessVerticalSpace = true;
		gdListSystemObjekte.verticalAlignment = GridData.FILL;
		// _listSystemObjekte.setLayoutData(gdListSystemObjekte);
		tableSystemObjekte.setLayoutData(gdListSystemObjekte);
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung von ObjektTypen.
	 * 
	 * @param parent
	 *            Eltern {@link Composite}
	 */
	private void erzeugeCompositeObjektTypen(final Composite parent) {
		final Composite objektTypen = new Composite(parent, SWT.NONE);

		final Runnable run = new Runnable() {

			@Override
			public void run() {
				if (tvObjektTypen != null) {
					// removeAllListeners();
					tvObjektTypen.setSelection(null, true);
					// inputSystemObjekte();
					// inputAttributGruppen();
					// inputAspekte();
					// addAllListeners();
				}
			}

		};

		selektionObjektTypen = new GtmListeSelektionsKopf(objektTypen,
				SWT.BORDER, "Objekttyp:", run);

		tableObjektTypen = new Table(objektTypen, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		tableObjektTypen.setHeaderVisible(true);
		tableObjektTypen.setLinesVisible(true);
		final TableColumn tc = new TableColumn(tableObjektTypen, SWT.LEAD);
		tc.setText("ObjektTypen:");
		tc.setWidth(300);
		tvObjektTypen = new TableViewer(tableObjektTypen);
		tvObjektTypen.setContentProvider(new ArrayContentProvider());
		tvObjektTypen.setLabelProvider(new GtmLabelProvider());
		tvObjektTypen.setSorter(new ViewerSorter());

		listenerTvObjektTypen = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					inputAttributGruppen();
					inputAspekte();
					filtere();
				}
			}
		};
		tvObjektTypen.addSelectionChangedListener(listenerTvObjektTypen);

		inputObjektTypen();

		final GridLayout glObjektTypen = new GridLayout(1, false);
		objektTypen.setLayout(glObjektTypen);

		final GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		selektionObjektTypen.setLayoutData(gdSelektion);

		final GridData gdListObjektTypen = new GridData();
		gdListObjektTypen.grabExcessHorizontalSpace = true;
		gdListObjektTypen.horizontalAlignment = GridData.FILL;
		gdListObjektTypen.grabExcessVerticalSpace = true;
		gdListObjektTypen.verticalAlignment = GridData.FILL;
		tableObjektTypen.setLayoutData(gdListObjektTypen);
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung von Attributgruppen.
	 * 
	 * @param parent
	 *            Eltern {@link Composite}
	 */
	private void erzeugeCompositeAttributGruppen(final Composite parent) {
		final Composite attributGruppen = new Composite(parent, SWT.NONE);

		final Runnable run = new Runnable() {

			@Override
			public void run() {
				if (tvAttributGruppen != null) {
					tvAttributGruppen.setSelection(null, true);
				}
			}

		};

		selektionAttributGruppen = new GtmListeSelektionsKopf(attributGruppen,
				SWT.BORDER, "Attributgruppe:", run);

		tableAttributGruppen = new Table(attributGruppen, SWT.BORDER
				| SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		tableAttributGruppen.setHeaderVisible(true);
		tableAttributGruppen.setLinesVisible(true);
		final TableColumn tc = new TableColumn(tableAttributGruppen, SWT.LEAD);
		tc.setWidth(300);
		tc.setText("Attributgruppen:");
		tvAttributGruppen = new TableViewer(tableAttributGruppen);
		tvAttributGruppen.setContentProvider(new ArrayContentProvider());
		tvAttributGruppen.setLabelProvider(new GtmLabelProvider());
		tvAttributGruppen.setSorter(new ViewerSorter());

		listenerTvAttributGruppen = new ISelectionChangedListener() {
			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					inputAspekte();
					filtere();
				}

			}

		};
		tvAttributGruppen
				.addSelectionChangedListener(listenerTvAttributGruppen);

		inputAttributGruppen();

		final GridLayout glAttributGruppen = new GridLayout(1, false);
		attributGruppen.setLayout(glAttributGruppen);

		final GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		selektionAttributGruppen.setLayoutData(gdSelektion);

		final GridData gdListAttributGruppen = new GridData();
		gdListAttributGruppen.grabExcessHorizontalSpace = true;
		gdListAttributGruppen.horizontalAlignment = GridData.FILL;
		gdListAttributGruppen.grabExcessVerticalSpace = true;
		gdListAttributGruppen.verticalAlignment = GridData.FILL;
		tableAttributGruppen.setLayoutData(gdListAttributGruppen);
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung von Aspekten.
	 * 
	 * @param parent
	 *            Eltern {@link Composite}
	 */
	private void erzeugeCompositeAspekte(final Composite parent) {
		final Composite compAspekte = new Composite(parent, SWT.NONE);

		final Runnable run = new Runnable() {

			@Override
			public void run() {
				if (tvAspekte != null) {
					tvAspekte.setSelection(null, true);
				}
			}

		};

		selektionAspekte = new GtmListeSelektionsKopf(compAspekte, SWT.BORDER,
				"Aspekt:", run);

		tableAspekte = new Table(compAspekte, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		tableAspekte.setLinesVisible(true);
		tableAspekte.setHeaderVisible(true);
		final TableColumn tc = new TableColumn(tableAspekte, SWT.LEAD);
		tc.setWidth(300);
		tc.setText("Aspekte:");
		tvAspekte = new TableViewer(tableAspekte);
		tvAspekte.setContentProvider(new ArrayContentProvider());
		tvAspekte.setLabelProvider(new GtmLabelProvider());
		tvAspekte.setSorter(new ViewerSorter());

		listenerTvAspekte = new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection) {
					final StructuredSelection ss = (StructuredSelection) selection;
					if (ss.size() > 0) {
						filtere();
					}
				}

			}

		};
		tvAspekte.addSelectionChangedListener(listenerTvAspekte);

		inputAspekte();

		final GridLayout glCompAspekte = new GridLayout(1, false);
		compAspekte.setLayout(glCompAspekte);

		final GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		selektionAspekte.setLayoutData(gdSelektion);

		final GridData gdListAspekte = new GridData();
		gdListAspekte.grabExcessHorizontalSpace = true;
		gdListAspekte.horizontalAlignment = GridData.FILL;
		gdListAspekte.grabExcessVerticalSpace = true;
		gdListAspekte.verticalAlignment = GridData.FILL;
		tableAspekte.setLayoutData(gdListAspekte);
	}

	private void erzeugeCompositeSimulation(final Composite parent) {
		final Composite simulation = new Composite(parent, SWT.NONE);

		final Label labelSim = new Label(simulation, SWT.NONE);
		labelSim.setText("Simulationsvariante:");

		final Spinner spinnerSim = new Spinner(simulation, SWT.NONE);
		spinnerSim.setIncrement(1);
		spinnerSim.setMinimum(0);
		spinnerSim.setMaximum(999);

		spinnerSim.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(final ModifyEvent e) {
				final int spinnerWert = spinnerSim.getSelection();
				simulationsVariante = spinnerWert;
			}
		});

		this.simulationsVariante = spinnerSim.getSelection();

		/*
		 * Layout
		 */
		final GridLayout glSimulation = new GridLayout(2, false);
		simulation.setLayout(glSimulation);

		// GridData gdSimulation = new GridData();
		// gdSimulation.grabExcessHorizontalSpace = true;
		// gdSimulation.horizontalAlignment = GridData.CENTER;
		// simulation.setLayoutData(gdSimulation);
	}

	protected void removeAllListeners() {
		if ((listenerTvSystemObjekte != null) && (tvSystemObjekte != null)) {
			tvSystemObjekte
					.removeSelectionChangedListener(listenerTvSystemObjekte);
		}

		if ((listenerTvObjektTypen != null) && (tvObjektTypen != null)) {
			tvObjektTypen.removeSelectionChangedListener(listenerTvObjektTypen);
		}

		if ((listenerTvAttributGruppen != null) && (tvAttributGruppen != null)) {
			tvAttributGruppen
					.removeSelectionChangedListener(listenerTvAttributGruppen);
		}

		if ((listenerTvAspekte != null) && (tvAspekte != null)) {
			tvAspekte.removeSelectionChangedListener(listenerTvAspekte);
		}
	}

	protected void addAllListeners() {
		if ((listenerTvSystemObjekte != null) && (tvSystemObjekte != null)) {
			tvSystemObjekte
					.addSelectionChangedListener(listenerTvSystemObjekte);
		}

		if ((listenerTvObjektTypen != null) && (tvObjektTypen != null)) {
			tvObjektTypen.addSelectionChangedListener(listenerTvObjektTypen);
		}

		if ((listenerTvAttributGruppen != null) && (tvAttributGruppen != null)) {
			tvAttributGruppen
					.addSelectionChangedListener(listenerTvAttributGruppen);
		}

		if ((listenerTvAspekte != null) && (tvAspekte != null)) {
			tvAspekte.addSelectionChangedListener(listenerTvAspekte);
		}
	}

	/**
	 * Stellt alles {@link SystemObject}s aus der {@link GtmDatenBasis} dar.
	 */
	private void inputSystemObjekte() {
		if (darstellungsArt >= 0) {
			if (gtmDatenBasis != null) {
				if (tvSystemObjekte != null) {
					tableSystemObjekte.setVisible(false);
					tvSystemObjekte.setInput(null);
					tvSystemObjekte.setInput(gtmDatenBasis.getSystemObjects());
					tableSystemObjekte.setVisible(true);
				}
			}
		}
	}

	/**
	 * Stellt bergebene {@link SystemObject}s dar.
	 * 
	 * @param systemObjects
	 *            Darzustellende {@link SystemObject}s
	 */
	private void inputSystemObjekte(
			final java.util.List<SystemObject> systemObjects) {
		if (darstellungsArt >= 0) {
			if (tvSystemObjekte != null) {
				tableSystemObjekte.setVisible(false);
				tvSystemObjekte.setInput(null);
				tvSystemObjekte.setInput(systemObjects);
				tableSystemObjekte.setVisible(true);
			}
		}
	}

	/**
	 * Stellt die in der {@link GtmDatenBasis} hinterlegten SytemObjectTypen
	 * dar.
	 */
	private void inputObjektTypen() {
		if (darstellungsArt >= 1) {
			if (gtmDatenBasis != null) {
				if (tvObjektTypen != null) {
					tableObjektTypen.setVisible(false);
					tvObjektTypen.setInput(null);
					tvObjektTypen
							.setInput(gtmDatenBasis.getSystemObjectTypes());
					tableObjektTypen.setVisible(true);
				}
			}
		}
	}

	/**
	 * Stellt an Hand der selektierten Objekttypen die zugehrigen
	 * Attributgruppen dar. Sind keine Objekttypen selektiert, werden alle
	 * Attributgruppen dargestellt.
	 */
	private void inputAttributGruppen() {
		if (darstellungsArt >= 2) {
			if (gtmDatenBasis != null) {
				if (tvAttributGruppen != null) {

					final LinkedList<SystemObjectType> sot = getSelektierteObjektTypenOderAlle();

					final java.util.List<AttributeGroup> atgs = gtmDatenBasis
							.getAtg(sot);

					if (sot.size() > 0) {
						tableAttributGruppen.setVisible(false);
						tvAttributGruppen.setInput(null);
						tvAttributGruppen.setInput(atgs);
						tableAttributGruppen.setVisible(true);
					}

					// if(_listAttributGruppen != null)
					// {
					// _listAttributGruppen.selectAll();
					// }

				}
			}
		}
	}

	/**
	 * Stellt an Hand der selektierten Attributgruppen die zugehrigen Aspekte
	 * dar. Sind keine Attributgruppen selektiert, werden alle Aspekte
	 * dargestellt.
	 */
	private void inputAspekte() {
		if (darstellungsArt >= 3) {
			if (gtmDatenBasis != null) {
				if (tvAspekte != null) {
					java.util.List<Aspect> asps = null;
					/*
					 * Wenn Attributgruppen selektiert sind
					 */
					final LinkedList<AttributeGroup> selektierteAttributGruppen = getSelektierteAttributGruppen();
					if (selektierteAttributGruppen.size() > 0) {
						asps = gtmDatenBasis.getAsp(selektierteAttributGruppen);
					} else {
						asps = gtmDatenBasis.getAsp(gtmDatenBasis
								.getAtg(getSelektierteObjektTypenOderAlle()));
					}
					if (asps.size() > 0) {
						tableAspekte.setVisible(false);
						tvAspekte.setInput(null);
						tvAspekte.setInput(asps);
						tableAspekte.setVisible(true);
					}
				}
			}
		}
	}

	/**
	 * Fhrt die Filterung der SystemObjekte an Hand der in den Listen gewhlten
	 * Objekten durch.
	 */
	private void filtere() {
		// long start = new Date().getTime();

		LinkedList<SystemObjectType> selObjTyp = null;
		LinkedList<AttributeGroup> selAtg = null;
		LinkedList<Aspect> selAsp = null;

		final LinkedList<SystemObjectType> selektierteObjektTypen = getSelektierteObjektTypenOderAlle();
		if (selektierteObjektTypen.size() > 0) {
			selObjTyp = selektierteObjektTypen;
		}

		final LinkedList<AttributeGroup> selektierteAttributGruppen = getSelektierteAttributGruppen();
		if (selektierteAttributGruppen.size() > 0) {
			selAtg = selektierteAttributGruppen;
		}

		final LinkedList<Aspect> selektierteAspekte = getSelektierteAspekte();
		if (selektierteAspekte.size() > 0) {
			selAsp = selektierteAspekte;
		}

		if (gtmDatenBasis != null) {
			final LinkedList<SystemObject> leos = gtmDatenBasis
					.filtere(gtmDatenBasis.getSystemObjects(), selObjTyp,
							selAtg, selAsp);

			inputSystemObjekte(leos);
		}

		fuelleAlleListenSelektionsKoepfe();

		// long ende = new Date().getTime();
		// System.out.println("Dauer Filtern GtmListeComposite: " + (ende -
		// start)
		// + " ms.");
	}

	/**
	 * Fllt alle vorhandenen {@link GtmListeSelektionsKpfe}.
	 */
	private void fuelleAlleListenSelektionsKoepfe() {
		if (selektionObjektListe != null) {
			// flleListenSelektionsKopf(_selektionObjektListe,
			// _listSystemObjekte);
			fuelleListenSelektionsKopf(selektionObjektListe, tableSystemObjekte);
		}

		if (selektionObjektTypen != null) {
			fuelleListenSelektionsKopf(selektionObjektTypen, tableObjektTypen);
		}

		if (selektionAttributGruppen != null) {
			fuelleListenSelektionsKopf(selektionAttributGruppen,
					tableAttributGruppen);
		}

		if (selektionAspekte != null) {
			fuelleListenSelektionsKopf(selektionAspekte, tableAspekte);
		}
	}

	private void fuelleListenSelektionsKopf(
			final GtmListeSelektionsKopf header, final Table liste) {
		header.setAnzahl(liste.getItemCount(), liste.getSelectionCount());
	}

	/**
	 * Liefert eine Liste der selektierten {@link SystemObjectType}s. Ist kein
	 * {@link SystemObjectType} selektiert, werden alle {@link SystemObjectType}
	 * der {@link GtmDatenBasis} zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link SystemObjectType}s
	 */
	private LinkedList<SystemObjectType> getSelektierteObjektTypenOderAlle() {
		final LinkedList<SystemObjectType> sot = new LinkedList<SystemObjectType>();
		if (tvObjektTypen != null) {
			final ISelection selection = tvObjektTypen.getSelection();
			if (!selection.isEmpty()) {
				final StructuredSelection ss = (StructuredSelection) selection;
				final Iterator<SystemObjectType> it = ss.iterator();
				while (it.hasNext()) {
					sot.add(it.next());
				}
			} else if (gtmDatenBasis != null) {
				final Iterator<SystemObjectType> it = gtmDatenBasis
						.getSystemObjectTypes().iterator();
				while (it.hasNext()) {
					sot.add(it.next());
				}
			}
		} else if (gtmDatenBasis != null) {
			final Iterator<SystemObjectType> it = gtmDatenBasis
					.getSystemObjectTypes().iterator();
			while (it.hasNext()) {
				sot.add(it.next());
			}
		}

		return sot;
	}

	/**
	 * Liefert eine Liste der selektierten {@link AttributeGroup}s. Ist keine
	 * Attributgruppe selektiert, werden alle in der Liste dargestellten
	 * Attributgruppen zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link AttributeGroup}s
	 */
	private LinkedList<AttributeGroup> getSelektierteAttributGruppen() {
		final LinkedList<AttributeGroup> atgs = new LinkedList<AttributeGroup>();

		if (tvAttributGruppen != null) {
			final ISelection selection = tvAttributGruppen.getSelection();

			if (!selection.isEmpty()) {
				final StructuredSelection ss = (StructuredSelection) selection;
				final Iterator<AttributeGroup> it = ss.iterator();
				while (it.hasNext()) {
					atgs.add(it.next());
				}
			}
		}

		return atgs;
	}

	/**
	 * Liefert eine Liste der selektierten {@link Aspect}s zurck. Wurde kein
	 * {@link Aspect} selektiert, werden alle in der Liste dargestellten
	 * {@link Aspect}s zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link Aspect}s
	 */
	private LinkedList<Aspect> getSelektierteAspekte() {
		final LinkedList<Aspect> asps = new LinkedList<Aspect>();

		if (tvAspekte != null) {
			final ISelection selection = tvAspekte.getSelection();
			if (!selection.isEmpty()) {
				final StructuredSelection ss = (StructuredSelection) selection;
				final Iterator<Aspect> it = ss.iterator();
				while (it.hasNext()) {
					asps.add(it.next());
				}
			}
		} else if (gtmDatenBasis != null) {
			if (tableAspekte != null) {
				for (final TableItem tableItem : tableAspekte.getItems()) {
					final Object aspect = tableItem.getData();
					if (aspect instanceof Aspect) {
						asps.add((Aspect) aspect);
					}
				}
			}
		}

		return asps;
	}

	/**
	 * Liefert eine Liste der selektierten SystemObjekte zurck.
	 * 
	 * @return
	 */
	public LinkedList<SystemObject> getSelektierteSystemObjekte() {
		final LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		if (tvSystemObjekte != null) {
			final ISelection selection = tvSystemObjekte.getSelection();
			if (!selection.isEmpty()) {
				final StructuredSelection ss = (StructuredSelection) selection;
				final Iterator<SystemObject> it = ss.iterator();
				while (it.hasNext()) {
					re.add(it.next());
				}
			}
		}

		return re;
	}

	/**
	 * Liefert die mit Hilfe des Spinners gewhlte Simulationsvariante zurck.
	 * 
	 * @return Simulationsvariante oder -1, falls es keinen Whler der
	 *         Simulationsvariante in der Darstellung gab.
	 */
	public int getSimulationsVariante() {
		return simulationsVariante;
	}

	private static class GtmLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			return element.toString();
		}
	}

	/**
	 * Klasse zur Darstellung der ListenKpfe.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class GtmListeSelektionsKopf extends Composite {
		private final String titel;
		private int anzMax = -1;
		private int anzAkt = -1;
		private final Runnable buttonRun;
		private Label labelTitle;
		private Text textAnzahl;

		public GtmListeSelektionsKopf(final Composite parent, final int style,
				final String titel, final Runnable buttonRun) {
			super(parent, style);

			this.titel = titel;
			this.buttonRun = buttonRun;

			initialize();
		}

		private void initialize() {
			final GridLayout glHeader = new GridLayout(3, false);
			setLayout(glHeader);

			labelTitle = new Label(this, SWT.NONE);
			labelTitle.setText(titel);

			textAnzahl = new Text(this, SWT.BORDER);
			textAnzahl.setEditable(false);
			setAnzahl(anzMax, anzAkt);

			final Button buttonReset = new Button(this, SWT.PUSH);
			buttonReset.setText("X");
			buttonReset.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(
						final org.eclipse.swt.events.SelectionEvent e) {
					if (buttonRun != null) {
						buttonRun.run();
					}
				}
			});

			final GridData gdTitel = new GridData();
			gdTitel.grabExcessHorizontalSpace = true;
			gdTitel.horizontalAlignment = GridData.FILL;
			labelTitle.setLayoutData(gdTitel);
		}

		public void setAnzahl(final int anzahlMax, final int anzahlAktuell) {
			this.anzMax = anzahlMax;
			this.anzAkt = anzahlAktuell;

			textAnzahl.setText(anzAkt + " / " + anzahlMax);
			this.layout(true);
		}
	}

}
