/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.dav;

import java.util.EventObject;

import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Eventobjekt fr die Kapselung der Daten die einen
 * {@link DavVerbindungsListener} bergeben werden.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DavVerbindungsEvent.java 45393 2013-08-07 21:32:57Z peuker $
 */
public class DavVerbindungsEvent extends EventObject {

	private static final long serialVersionUID = 1L;

	/** Definition der Tyoen des Events. */
	public enum Type {
		/** Verbing wird getrnnt bzw. soll getrennt werden. */
		DISCONNECT,
		/** Verbindung wurde hergestellt. */
		CONNECT;
	}

	private final Type type;
	private final ClientDavInterface davVerbindung;

	/**
	 * Konstruktor.
	 * 
	 * @param source
	 *            die Quelle des Events
	 * @param type
	 *            der Typ
	 * @param davVerbindung
	 *            die beteiligte Datenverteilerverbindung
	 */
	public DavVerbindungsEvent(final Object source, final Type type,
			final ClientDavInterface davVerbindung) {
		super(source);
		this.type = type;
		this.davVerbindung = davVerbindung;
	}

	/**
	 * liefert den Typ des Events.
	 * 
	 * @return den Typ
	 */
	public Type getType() {
		return type;
	}

	/**
	 * liefert die beteiligte Datenverteilerverbindung.
	 * 
	 * @return die Verbindung
	 */
	public ClientDavInterface getDavVerbindung() {
		return davVerbindung;
	}
}
