/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.dav;

/**
 * Schnittstelle fr einen Listener fr den Zustand der
 * Datenverteilerverbindung.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DavVerbindungsListener.java 45393 2013-08-07 21:32:57Z peuker $
 */
public interface DavVerbindungsListener {

	/**
	 * es wurde eine Datenverteilerverbindung hergestellt.
	 * 
	 * @param event
	 *            die Daten des Ereignisses
	 */
	void verbindungHergestellt(DavVerbindungsEvent event);

	/**
	 * es wurde eine Datenverteilerverbindung getrennt.
	 * 
	 * @param event
	 *            die Daten des Ereignisses
	 */
	void verbindungGetrennt(DavVerbindungsEvent event);

	/**
	 * eine Datenverteilerverbindung soll getrennt werden. Der Listener kann
	 * hier ein Veto einlegen, sollte aber versuchen einen Zustand zu erreichen,
	 * der beim einem der nchsten Aufrufe kein Veto mehr erfordert.
	 * 
	 * @param event
	 *            die Daten des Ereignisses
	 * @return <code>true</code>, wenn die Datenverteilerverbindung offen
	 *         gehalten werden soll, <code>false</code> sonst (kein Veto)
	 */
	boolean verbindungHalten(DavVerbindungsEvent event);
}
