/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.internal;

import java.util.Timer;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Aktivator des Rahmenwerk-Basislib-Plugins.
 * 
 * Der Aktivator stellt einen global nutzbaren Timer zur Verfgung und bietet
 * eine integrierte Image-Registry.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwBasisLibActivator.java 45656 2013-08-23 07:43:01Z peuker $
 */
public class RwBasisLibActivator extends AbstractUIPlugin {

	/** die Plugin-ID. */
	public static final String PLUGIN_ID = "de.bsvrz.buv.rw.basislib";

	/** die globale Instanz des Plugins. */
	private static RwBasisLibActivator plugin;

	/** ein global verfgbarer Timer. */
	private static final Timer TIMER = new Timer("RwBasislibTimer", true);

	/** Standardkonstruktor. */
	public RwBasisLibActivator() {
		final String[] args = Platform.getApplicationArgs();
		final String[] argCopy = new String[args.length];
		System.arraycopy(args, 0, argCopy, 0, args.length);
		Debug.init(RwBasisLibActivator.PLUGIN_ID, new ArgumentList(argCopy));
	}

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		RwBasisLibActivator.plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		RwBasisLibActivator.plugin = null;
		super.stop(context);
	}

	/**
	 * liefert die globale Instanz des Plugins.
	 * 
	 * @return die Instanz
	 */
	public static RwBasisLibActivator getDefault() {
		return RwBasisLibActivator.plugin;
	}

	/**
	 * liefert den Image-Descriptor fr eine Image-Ressource innerhalb des
	 * Plug-ins.
	 * 
	 * @param path
	 *            der Pfad innerhalb des Plug-ins
	 * @return der Descriptor
	 */
	public static ImageDescriptor getImageDescriptor(final String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				RwBasisLibActivator.PLUGIN_ID, path);
	}

	/**
	 * liefert das Image fr den bergebenen Key aus der Image-Registry des
	 * Plug-ins. Gegebenefalls wird das Bild erzeugt und in die Registry
	 * eingetragen.
	 * 
	 * @param key
	 *            der Schlssel
	 * @return das Image oder <code>null</code>, wenn kein entsprechendes Image
	 *         gefunden werden konnte
	 */
	public Image getImage(final String key) {
		Image image = getImageRegistry().get(key);
		if (image == null) {
			final ImageDescriptor imageDescriptor = RwBasisLibActivator
					.getImageDescriptor(key);
			if (imageDescriptor != null) {
				image = imageDescriptor.createImage();
				getImageRegistry().put(key, image);
			}
		}
		return image;
	}

	/**
	 * liefert den global verfgbaren Timer des Plug-ins.
	 * 
	 * @return den Timer
	 */
	public static Timer getPluginTimer() {
		return RwBasisLibActivator.TIMER;
	}
}
