/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.kalender.internal.SelektierInfo;
import de.bsvrz.buv.rw.basislib.kalender.internal.ValidierungsInfo;
import de.bsvrz.buv.rw.basislib.kalender.internal.ZeitDauerGranularitaet;
import de.bsvrz.buv.rw.basislib.kalender.internal.ZeitKlasse;

/**
 * Composite zur Darstellung und zum Bearbeiten einer Zeitdauer.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: CompositeZeitDauer.java 47112 2013-11-12 15:25:14Z peuker $
 * 
 */
public class CompositeZeitDauer extends Composite {

	private List<SelectionListener> selectionListener;

	/**
	 * Feld zur Eingabe der Zeitdauer.
	 */
	private Text textEingabe;

	/**
	 * Erhhen der selektierten Zeit.
	 */
	private Button schaltflaechePlus;

	/**
	 * Verringern der selektierten Zeit.
	 */
	private Button schaltflaecheMinus;

	/**
	 * Zeitdauer-Hilfsklasse.
	 */
	private final ZeitDauer zeitdauer;

	private final List<ModifyListener> modifyListeners = Collections
			.synchronizedList(new ArrayList<ModifyListener>());

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            der umgebende Container
	 * @param style
	 *            der Stil der Darstellung
	 * @param ausgangsZeitpunkt
	 *            der Anfangszeitpunkt
	 * @param kurzDarstellung
	 *            Kurzdarstellung verwenden?
	 */
	public CompositeZeitDauer(final Composite parent, final int style,
			final long ausgangsZeitpunkt, final boolean kurzDarstellung) {
		super(parent, style);

		zeitdauer = new ZeitDauer(ausgangsZeitpunkt);
		zeitdauer.setAllesEditierbar(true);
		zeitdauer.setKurzDarstellung(kurzDarstellung);

		init();
		listenerAnhaengen();
	}

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            der umgebende Container
	 * @param style
	 *            der Stil der Darstellung
	 * @param ausgangsZeitpunkt
	 *            der Anfangszeitpunkt
	 * @param intervall
	 *            die Lnge des Intervalls
	 * @param kurzDarstellung
	 *            Kurzdarstellung verwenden?
	 */
	public CompositeZeitDauer(final Composite parent, final int style,
			final long ausgangsZeitpunkt, final long intervall,
			final boolean kurzDarstellung) {
		super(parent, style);
		zeitdauer = new ZeitDauer(ausgangsZeitpunkt);
		zeitdauer.setAllesEditierbar(true);
		zeitdauer.setKurzDarstellung(kurzDarstellung);

		init();
		listenerAnhaengen();

		setZeitDauerInBezugAufAnfangsZeitpunkt(intervall);
	}

	private void listenerAnhaengen() {
		textEingabe.addListener(SWT.Verify, new Listener() {

			@Override
			public void handleEvent(final Event e) {

				e.doit = false;

				if ((e.keyCode != SWT.TAB) && (e.keyCode != SWT.ARROW_LEFT)
						&& (e.keyCode != SWT.ARROW_RIGHT)) {
					// berlesen der
					// Tab-Taste
					// if (e.keyCode != SWT.TAB) // berlesen der Tab-Taste
					schaltflaechePlus.setVisible(false);
					schaltflaecheMinus.setVisible(false);

					/*
					 * Verarbeitung von Texten, die z.B. per Copy & Paste
					 * eingefgt wurden.
					 */
					if (e.text.length() > 1) {
						if (zeitdauer.verifyEingabe(e.text)) {
							e.doit = true;
						}
					} else if (zeitdauer.verifyEingabe(e.text, e.start,
							textEingabe.getText(), true)) {
						e.doit = true;
					}
				}

			}

		});

		textEingabe.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDown(final MouseEvent e) {
				aktionMausSelektion();
			}
		});

		textEingabe.addTraverseListener(new TraverseListener() {

			@Override
			public void keyTraversed(final TraverseEvent e) {
				if (e.detail == SWT.TRAVERSE_TAB_NEXT) {
					selektiereTab(true);
					textEingabe.setFocus();
					e.doit = false;
				} else if (e.detail == SWT.TRAVERSE_TAB_PREVIOUS) {
					selektiereTab(false);
					textEingabe.setFocus();
					e.doit = false;
				}
			}
		});
	}

	/**
	 * Aktion hinter der Selektion eines Zahlenfeldes mit der Maus.
	 */
	private void aktionMausSelektion() {
		final int pos = textEingabe.getCaretPosition();
		// System.out.println("Position in Text: " + pos);
		final int[] bestimmeMarkierung = zeitdauer.bestimmeMarkierung(
				textEingabe.getText(), pos);
		if (bestimmeMarkierung == null) {
			schaltflaechePlus.setVisible(false);
			schaltflaecheMinus.setVisible(false);
		} else {
			if (bestimmeMarkierung.length == 2) {
				/*
				 * Setzen der Selektion
				 */
				textEingabe.setSelection(bestimmeMarkierung[0],
						bestimmeMarkierung[1] + 1);

				schaltflaechePlus.setVisible(true);
				schaltflaecheMinus.setVisible(true);
			} else {
				schaltflaechePlus.setVisible(false);
				schaltflaecheMinus.setVisible(false);
			}
		}
	}

	private void init() {

		textEingabe = new Text(this, SWT.BORDER);
		textEingabe.addListener(SWT.DefaultSelection, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				aktionNormalisieren();
			}
		});
		textEingabe.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(final ModifyEvent e) {
				fireModifyListeners(e);
			}
		});

		final Button schaltflaecheZeitDauerWaehlerOeffnen = new Button(this,
				SWT.ARROW | SWT.DOWN);
		schaltflaecheZeitDauerWaehlerOeffnen
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						aktionZeitDauerWaehler();
					}
				});

		schaltflaechePlus = new Button(this, SWT.NONE);
		schaltflaechePlus.setText("+");
		schaltflaechePlus.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionPlus();
			}
		});
		schaltflaechePlus.setVisible(false);

		schaltflaecheMinus = new Button(this, SWT.NONE);
		schaltflaecheMinus.setText("-");
		schaltflaecheMinus.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionMinus();
			}
		});
		schaltflaecheMinus.setVisible(false);

		final GridData gridDataText = new GridData();
		gridDataText.horizontalAlignment = GridData.FILL;
		gridDataText.grabExcessHorizontalSpace = true;
		gridDataText.verticalAlignment = GridData.CENTER;
		gridDataText.horizontalIndent = 0;
		textEingabe.setLayoutData(gridDataText);

		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;

		gridLayout.horizontalSpacing = 0;

		this.setLayout(gridLayout);
		// setSize(new Point(300, 200));
	}

	/**
	 * Fhrt die Selektion des nchsten Zahlenfeldet mit Hilfe der TAB-Taste
	 * durch.
	 * 
	 * @param naechstesFeld
	 *            Wenn <code>true</code>, wenn wird das nchste Zahlenfeld nach
	 *            rechts hin gesucht, bei <code>false</code> wird in linker
	 *            Richtung vorgegangen.
	 */
	private void selektiereTab(final boolean naechstesFeld) {
		// System.out.println("Selektiere Tab: ");
		// textEingabe.setText("123 Jahre");

		final int caretPosition = textEingabe.getCaretPosition();

		SelektierInfo naechstesZahlenfeld = null;

		if (naechstesFeld) {
			naechstesZahlenfeld = zeitdauer.bestimmeNaechstesZahlenfeld(
					textEingabe.getText(), caretPosition,
					textEingabe.getSelection().x, textEingabe.getSelection().y);
		} else {
			naechstesZahlenfeld = zeitdauer.bestimmeVorhergehendesZahlenfeld(
					textEingabe.getText(), caretPosition,
					textEingabe.getSelection().x, textEingabe.getSelection().y);
		}

		if (naechstesZahlenfeld != null) {
			textEingabe.setSelection(naechstesZahlenfeld.getStartPos(),
					naechstesZahlenfeld.getEndPos() + 1);
			schaltflaechePlus.setVisible(true);
			schaltflaecheMinus.setVisible(true);
		}
		textEingabe.setFocus();
	}

	private void aktionZeitDauerWaehler() {
		aktionNormalisieren();
		final ZeitDauerDialog zdd = new ZeitDauerDialog(PlatformUI
				.getWorkbench().getActiveWorkbenchWindow().getShell(), this);
		zdd.open();
	}

	/**
	 * Fhrt auf einem Zahlenfeld einer Zeitklasse die Addition von 1 durch.
	 */
	private void aktionPlus() {
		final String text = textEingabe.getText();
		final int caretPosition = textEingabe.getCaretPosition();
		final String selectionText = textEingabe.getSelectionText();
		final SelektierInfo selektierInfo = zeitdauer.wertAendern(text,
				selectionText, caretPosition, 1);

		if (selektierInfo != null) {
			textEingabe.setText(selektierInfo.getNeuerText());
			textEingabe.setSelection(selektierInfo.getStartPos(),
					selektierInfo.getEndPos() + 1);
		}

		schaltflaechePlus.setVisible(true);
		schaltflaecheMinus.setVisible(true);

		textEingabe.setFocus();
	}

	/**
	 * Fhrt auf einem Zahlenfeld einer Zeitklasse die Subtraktion von 1 durch.
	 */
	private void aktionMinus() {

		final String text = textEingabe.getText();
		final int caretPosition = textEingabe.getCaretPosition();
		final String selectionText = textEingabe.getSelectionText();

		final int wert = -1;
		final SelektierInfo selektierInfo = zeitdauer.wertAendern(text,
				selectionText, caretPosition, wert);

		if (selektierInfo != null) {
			textEingabe.setText(selektierInfo.getNeuerText());
			textEingabe.setSelection(selektierInfo.getStartPos(),
					selektierInfo.getEndPos() + 1);
		}

		schaltflaechePlus.setVisible(true);
		schaltflaecheMinus.setVisible(true);

		textEingabe.setFocus();
	}

	/**
	 * Normalisiert den Zeitdauer-String des Textfeldes an Hand des
	 * gregorianischen Kalenders. Es wird vorher eine Validierung durchgefhrt.
	 */
	final void aktionNormalisieren() {
		if ((zeitdauer != null) && (textEingabe.getText().length() > 0)) {
			final ValidierungsInfo validierungsInfo = zeitdauer
					.validiereFelder(textEingabe.getText());

			if (validierungsFehlerauswertung(validierungsInfo)) {
				final String normalisiert = zeitdauer
						.normalisieren(validierungsInfo.getFeldText());
				textEingabe.setText(normalisiert);
			}
		}
	}

	private boolean validierungsFehlerauswertung(
			final ValidierungsInfo validierungsInfo) {
		boolean re = true;

		final StringBuilder meldung = new StringBuilder(150);

		final List<String> nichtBestimmbareFelder = validierungsInfo
				.getNichtBestimmbareFelder();
		if ((nichtBestimmbareFelder != null)
				&& (!nichtBestimmbareFelder.isEmpty())) {
			meldung.append("Nicht bestimmbare Felder:\n=========================\n");
			int i = 0;
			final Iterator<String> it = nichtBestimmbareFelder.iterator();
			while (it.hasNext()) {
				final String string = it.next();
				meldung.append(i);
				meldung.append(". Feld: ");
				meldung.append(string);
				meldung.append('\n');
				i++;
			}

		}

		final String sonstigeInfos = validierungsInfo.getSonstigeInfos();
		if ((sonstigeInfos != null) && (!sonstigeInfos.isEmpty())) {
			meldung.append("\nSonstige Informationen:\n=======================\n");
			meldung.append(sonstigeInfos);

		}

		if (meldung.length() > 0) {
			meldung.insert(
					0,
					"Die Validierung konnte nicht komplett durchgefhrt werden,\n"
							+ "da einige Angaben unvollstndig oder fehlerhaft sind. Nachfolgend\n"
							+ "erhalten Sie eine Auflistung der unvollstndigen oder fehlerhafen\n"
							+ "Angaben:\n");

			meldung.append("\n\nDrcken Sie OK um fortzufahren, fehlerhafte Eingaben gehen\n"
					+ "dabei verloren, oder Abbrechen um Ihre fehlerhaften Eingaben\n"
					+ "zu korrigieren.");

			final MessageBox msg = new MessageBox(PlatformUI.getWorkbench()
					.getDisplay().getActiveShell(), SWT.OK | SWT.CANCEL
					| SWT.ICON_QUESTION);
			msg.setText("Validierung nicht erfolgreich");
			msg.setMessage(meldung.toString());

			final int open = msg.open();

			if (open == SWT.OK) {
				re = true;
			} else if (open == SWT.CANCEL) {
				re = false;
			}
		}

		return re;
	}

	/**
	 * Liefert den Ausgangszeitpunkt in Millisekunden seit dem 1.1.1970.
	 * 
	 * @return Ausgangszeitpunkt
	 */
	public long getAusgangsZeitpunkt() {
		return zeitdauer.getAusgangsZeitpunkt();
	}

	/**
	 * Setzt den Ausgangszeitpunkt in Millisekunden seit dem 1.1.1970. Wird der
	 * Wert gendert, muss der Entwickler auch die Zeitdauer erneut setzen.
	 * 
	 * @param ausgangsZeitpunkt
	 *            Ausgangszeitpunkt
	 */
	public void setAusgangsZeitpunkt(final long ausgangsZeitpunkt) {
		zeitdauer.setAusgangsZeitpunkt(ausgangsZeitpunkt);
		fireChangeEvent();
	}

	/**
	 * Prft, ob die Kurzdarstellung gewhlt wurde oder nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn die Kurzdarstellung gewhlt
	 *         wurde, sonst <code>false</code>.
	 */
	public boolean istKurzDarstellung() {
		return zeitdauer.istKurzDarstellung();
	}

	/**
	 * Setzt, ob die Darstellung in der Kurzdarstellung erfolgen soll. Per
	 * Default ist dieser Wert false.
	 * 
	 * @param kurzDarstellung
	 *            Muss <code>true</code> sein, wenn eine Kurzdarstellung
	 *            erfolgen soll, sonst <code>false</code>.
	 */
	public void setKurzDarstellung(final boolean kurzDarstellung) {
		zeitdauer.setKurzDarstellung(kurzDarstellung);
	}

	/**
	 * Liefert die Zeitdauer in Millisekunden zurck.
	 * 
	 * @return Zeitdauer in Millisekunden
	 */
	public long getZeitDauer() {
		long re = -1;

		if ((zeitdauer != null) && (textEingabe.getText().length() > 0)) {
			re = zeitdauer.getZeitdauerInBezugAufAnfangszeitpunkt(textEingabe
					.getText());
		}

		return re;
	}

	/**
	 * Setzt die Zeitdauer in Bezug zum gesetzten Anfangszeitpunkt.
	 * 
	 * @param zeitDauer
	 *            Zeitdauer in Millisekunden
	 */
	public void setZeitDauerInBezugAufAnfangsZeitpunkt(final long zeitDauer) {
		final String inBezugAufAnfangszeitpunkt = zeitdauer
				.setZeitdauerInBezugAufAnfangszeitpunkt(
						zeitdauer.getAusgangsZeitpunkt(), zeitDauer);
		textEingabe.setText(inBezugAufAnfangszeitpunkt);
		fireChangeEvent();
	}

	/**
	 * Setzt auf direktem Wege die Zeitdauer, in dem Jahre, Monate, Tage,
	 * Stunden, Minuten, Sekunden und Tausendstel der Zeitdauer direkt
	 * angegebene werden. Die Werte werden ausgehend vom gewhlten
	 * Ausgangszeitpunkt normalisiert.
	 * 
	 * @param jahre
	 *            Anzahl der Jahre
	 * @param monate
	 *            Anzahl der Monate
	 * @param tage
	 *            Anzahl der Tage
	 * @param stunden
	 *            Anzahl der Stunden
	 * @param minuten
	 *            Anzahl der Minuten
	 * @param sekunden
	 *            Anzahl der Sekunden
	 * @param tausendstel
	 *            Anzahl der Tausendstel
	 */
	public void setZeitDauer(final int jahre, final int monate, final int tage,
			final int stunden, final int minuten, final int sekunden,
			final int tausendstel) {
		final String zeitString = zeitdauer.baueZeitString(jahre, monate, tage,
				stunden, minuten, sekunden, tausendstel);
		textEingabe.setText(zeitdauer.normalisieren(zeitString));
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Jahre zurck.
	 * 
	 * @return Anzahl der Jahre oder -1, wenn keine Jahre gesetzt wurden.
	 */
	public int getJahre() {
		return zeitdauer.getJahre(textEingabe.getText());
	}

	/**
	 * setzt den Jahre-Anteil der Zeitdauer.
	 * 
	 * @param jahre
	 *            die Anzahl an Jahren
	 */
	public void setJahre(final int jahre) {
		final String wert = zeitdauer.setJahre(textEingabe.getText(), jahre);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Monate zurck.
	 * 
	 * @return Anzahl der Monate oder -1, wenn keine Monate gesetzt wurden.
	 */
	public int getMonate() {
		return zeitdauer.getMonate(textEingabe.getText());
	}

	/**
	 * setzt den Monate-Anteil der Zeitdauer.
	 * 
	 * @param monate
	 *            die Anzahl an Monaten
	 */
	public void setMonate(final int monate) {
		final String wert = zeitdauer.setMonate(textEingabe.getText(), monate);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Tage zurck.
	 * 
	 * @return Anzahl der Tage oder -1, wenn keine Tage gesetzt wurden.
	 */
	public int getTage() {
		return zeitdauer.getTage(textEingabe.getText());
	}

	/**
	 * setzt den Tage-Anteil der Zeitdauer.
	 * 
	 * @param tage
	 *            die Anzahl an Tagen
	 */
	public void setTage(final int tage) {
		final String wert = zeitdauer.setTage(textEingabe.getText(), tage);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Stunden zurck.
	 * 
	 * @return Anzahl der Stunden oder -1, wenn keine Stunden gesetzt wurden.
	 */
	public int getStunden() {
		return zeitdauer.getStunden(textEingabe.getText());
	}

	/**
	 * setzt den Stunden-Anteil der Zeitdauer.
	 * 
	 * @param stunden
	 *            die Anzahl an Stunden
	 */
	public void setStunden(final int stunden) {
		final String wert = zeitdauer
				.setStunden(textEingabe.getText(), stunden);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Minuten zurck.
	 * 
	 * @return Anzahl der Minuten oder -1, wenn keine Minuten gesetzt wurden.
	 */
	public int getMinuten() {
		return zeitdauer.getMinuten(textEingabe.getText());
	}

	/**
	 * setzt den Minuten-Anteil der Zeitdauer.
	 * 
	 * @param minuten
	 *            die Anzahl an Minuten
	 */
	public void setMinuten(final int minuten) {
		final String wert = zeitdauer
				.setMinuten(textEingabe.getText(), minuten);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Sekunden zurck.
	 * 
	 * @return Anzahl der Sekunden oder -1, wenn keine Sekunden gesetzt wurden.
	 */
	public int getSekunden() {
		return zeitdauer.getSekunden(textEingabe.getText());
	}

	/**
	 * setzt den Sekunden-Anteil der Zeitdauer.
	 * 
	 * @param sekunden
	 *            die Anzahl an Sekunden
	 */
	public void setSekunden(final int sekunden) {
		final String wert = zeitdauer.setSekunden(textEingabe.getText(),
				sekunden);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Liefert die Anzahl der Tausendstel zurck.
	 * 
	 * @return Anzahl der Tausendstel oder -1, wenn keine Tausendstel gesetzt
	 *         wurden.
	 */
	public int getTausendstel() {
		return zeitdauer.getTausendstel(textEingabe.getText());
	}

	/**
	 * setzt den Millisekunden-Anteil der Zeitdauer.
	 * 
	 * @param tausendstell
	 *            die Anzahl an Millisekunden
	 */
	public void setTausendstel(final int tausendstell) {
		final String wert = zeitdauer.setTausendstel(textEingabe.getText(),
				tausendstell);
		textEingabe.setText(wert);
		fireChangeEvent();
	}

	/**
	 * Setzt, ob Jahre dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Jahre dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setJahreDarstellen(final boolean darstellen) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.JAHR, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Monate dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Monate dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setMonateDarstellen(final boolean darstellen) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MONAT, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Tage dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Tage dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setTageDarstellen(final boolean darstellen) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.TAG, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Stunden dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Stunden dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setStundenDarstellen(final boolean darstellen) {
		this.zeitdauer
				.setZeitElementDarstellung(ZeitKlasse.STUNDEN, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Minuten dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Minuten dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setMinutenDarstellen(final boolean darstellen) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MINUTE, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Sekunden dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Sekunden dargestellt, bei
	 *            <code>false</code> nicht.
	 */
	public void setSekundenDarstellen(final boolean darstellen) {
		this.zeitdauer
				.setZeitElementDarstellung(ZeitKlasse.SEKUNDE, darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt, ob Tausendstel dargestellt werden sollen oder nicht.
	 * 
	 * @param darstellen
	 *            Wenn <code>true</code>, dann werden Tausendstel dargestellt,
	 *            bei <code>false</code> nicht.
	 */
	public void setTausendstelDarstellen(final boolean darstellen) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MILLSEKUNDE,
				darstellen);
		aktionNormalisieren();
	}

	/**
	 * Setzt welche Zeitklassen dargestellt werden sollen. Wird diese Funktion
	 * nachtrglich aufgerufen sollte eine Normalisierung durchgefhrt werden.
	 * 
	 * @param jahre
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param monate
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param tage
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param stunden
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param minuten
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param sekunden
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 * @param tausendstel
	 *            Werden bei <code>true</code> dargestellt, sonst nicht.
	 */
	public void setZeitklassenDarstellen(final boolean jahre,
			final boolean monate, final boolean tage, final boolean stunden,
			final boolean minuten, final boolean sekunden,
			final boolean tausendstel) {
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.JAHR, jahre);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MONAT, monate);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.TAG, tage);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.STUNDEN, stunden);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MINUTE, minuten);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.SEKUNDE, sekunden);
		this.zeitdauer.setZeitElementDarstellung(ZeitKlasse.MILLSEKUNDE,
				tausendstel);
		aktionNormalisieren();
	}

	/**
	 * Prft, ob der Text der Zeitklassen im Eingabefeld editierbar ist oder
	 * nicht.
	 * 
	 * @return Liefert <code>true</code>, wenn der Text im Eingabefeld
	 *         editierbar ist, sonst <code>false</code>.
	 */
	public boolean istZeitKlassenTextEditierbar() {
		return zeitdauer.istAllesEditierbar();
	}

	/**
	 * Setzt die Editierbarkeit der Zeitklassentexte im Eingabefeld.
	 * 
	 * @param editierbar
	 *            Wenn <code>true</code>, dann ist der Text der Zeitklassentexte
	 *            editierbar, bei <code>false</code> nicht.
	 */
	public void setZeitKlassenTextEditierbar(final boolean editierbar) {
		this.zeitdauer.setAllesEditierbar(editierbar);
	}

	/**
	 * Liefert die Granularitt dieses Eingabefeldes.
	 * 
	 * @return die Granularitt
	 */
	public ZeitDauerGranularitaet getGranularitaet() {
		return zeitdauer.getGranularitaet(textEingabe.getText());
	}

	/**
	 * fgt einen auf die nderung des Wertes angemeldeten Listener hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addModifyListener(final ModifyListener listener) {
		modifyListeners.add(listener);
	}

	/**
	 * entfernt einen auf die nderung des Wertes angemeldeten Listener.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeModifyListener(final ModifyListener listener) {
		modifyListeners.remove(listener);
	}

	private void fireModifyListeners(final ModifyEvent event) {
		synchronized (modifyListeners) {
			for (final ModifyListener listener : modifyListeners) {
				listener.modifyText(event);
			}
		}
	}

	/**
	 * fgt einen auf die Benachrichtigung einer Auswahlnderung angemeldeten
	 * Listener hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addSelectionListener(final SelectionListener listener) {
		if (selectionListener == null) {
			selectionListener = new ArrayList<SelectionListener>();
		}
		if (!selectionListener.contains(listener)) {
			selectionListener.add(listener);
		}
	}

	/**
	 * entfernt einen auf die Benachrichtigung einer Auswahlnderung
	 * angemeldeten Listener.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeSelectionListener(final SelectionListener listener) {
		if (selectionListener == null) {
			return;
		}
		selectionListener.remove(listener);
		if (selectionListener.isEmpty()) {
			selectionListener = null;
		}
	}

	private void fireChangeEvent() {
		if (selectionListener == null) {
			return;
		}
		final List<SelectionListener> tempList = new ArrayList<SelectionListener>(
				selectionListener);
		for (final SelectionListener listener : tempList) {
			listener.widgetSelected(null);
		}
	}

}
