/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;
import de.bsvrz.buv.rw.basislib.internal.RwBasisLibActivator;
import de.bsvrz.buv.rw.basislib.kalender.internal.EintragEreigniskalender;
import de.bsvrz.buv.rw.basislib.kalender.internal.ErkaKapsel;
import de.bsvrz.buv.rw.basislib.kalender.internal.IEintragKapsel;
import de.bsvrz.buv.rw.basislib.kalender.internal.SkeKapsel;
import de.bsvrz.buv.rw.basislib.kalender.internal.SysKalAnfrageArbeiter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Dialog zur Auswahl von Ereignissen aus dem Ereignis-/Systemkalender.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: KalenderBereichDialog.java 47112 2013-11-12 15:25:14Z peuker $
 * 
 */
public class KalenderBereichDialog extends TitleAreaDialog {

	/**
	 * Enumerator zur Auswahl der Quelle (Ereignis- oder Systemkalender).
	 * 
	 * @author HofmannJ
	 * 
	 */
	public enum Quelle {
		/** Quelle der Eintrge ist der Ereigniskalender. */
		ereigniskalender("Ereigniskalender",
				"Eintrge aus dem Ereigniskalender"),
		/** Quelle der Eintrge ist der Systemkalender. */
		systemkalender("Systemkalender", "Eintrge aus dem Systemkalender");

		/**
		 * Bezeichnung des Eintrags.
		 */
		private String bezeichnung;

		/**
		 * Beschreibung des Eintrags.
		 */
		private String beschreibung;

		/**
		 * Konstruktor. Setzt Bezeichnung und Beschreibung des Eintrags.
		 * 
		 * @param bezeichnung
		 *            Bezeichnung des Eintrags
		 * @param beschreibung
		 *            Beschreibung des Eintrags
		 */
		private Quelle(final String bezeichnung, final String beschreibung) {
			this.bezeichnung = bezeichnung;
			this.beschreibung = beschreibung;
		}

		/**
		 * Liefert die Beschreibung.
		 * 
		 * @return Beschreibung
		 */
		public String getBeschreibung() {
			return beschreibung;
		}

		/**
		 * Liefert die Bezeichnung.
		 * 
		 * @return Bezeichnung
		 */
		public String getBezeichnung() {
			return bezeichnung;
		}

		@Override
		public String toString() {
			return bezeichnung + " (" + beschreibung + ")";
		}

	}

	/**
	 * Contentprovider fr Ereignis-/Systemkalenderbereiche.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class SysErKalBereichContentProvider implements
			IStructuredContentProvider {

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;
			if (inputElement instanceof IEintragKapsel) {
				final IEintragKapsel skeKapsel = (IEintragKapsel) inputElement;
				re = skeKapsel.getBereiche().toArray();
			}

			return re;
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird global bahandelt
		}
	}

	/**
	 * Labelprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class SysErKalBereichLabelProvider implements
			ITableLabelProvider {

		@Override
		public void addListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			String re = "";
			if (element instanceof IEintragBereich) {
				final IEintragBereich skeBereich = (IEintragBereich) element;
				if (columnIndex == 0) {
					re = skeBereich.getVonLesbar();
				} else if (columnIndex == 1) {
					re = skeBereich.getBisLesbar();
				}

			}

			return re;
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}

		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

	}

	/**
	 * Contentprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class SysErKalEintragContentProvider implements
			IStructuredContentProvider {
		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;

			if (inputElement instanceof LinkedList) {
				final LinkedList<IEintragKapsel> kapseln = (LinkedList<IEintragKapsel>) inputElement;
				re = new Object[kapseln.size()];
				int i = 0;
				final Iterator<IEintragKapsel> it = kapseln.iterator();
				while (it.hasNext()) {
					final IEintragKapsel skeKapsel = it.next();
					re[i] = skeKapsel;
					i++;
				}
			}

			return re;
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird global behandelt?
		}

	}

	/**
	 * Labelprovider fr Ereignis-/Systemkalendereintrge.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class SysErKalEintragLabelProvider implements
			ITableLabelProvider {

		@Override
		public void addListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			String re = "";
			if (element instanceof IEintragKapsel) {
				final IEintragKapsel skeKapsel = (IEintragKapsel) element;
				re = skeKapsel.getPid();
			}
			return re;
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}

		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}
	}

	private Text textJahr;

	private Text textMonat;

	private TableViewer tableViewerEintraege;

	private Label labelDaVVerbindung;

	private TableViewer tableViewerBereiche;

	private static final String[] MONATE = new String[] { "Januar", "Februar",
			"Mrz", "April", "Mai", "Juni", "Juli", "August", "September",
			"Oktober", "November", "Dezember" };

	/** Index zur Bestimmung des Monats. */
	private int idxMonat;

	/** Index zur Bestimmung des Jahres. */
	private int idxJahr = 2008;

	/**
	 * Flag das Anzeigt, ob der Systemkalender-Arbeiter gestartet wurde oder
	 * nicht.
	 */
	private boolean sysKalArbeiterGestartet;

	/**
	 * Arbeiter fr Anfragen an den Systemkalender.
	 */
	private SysKalAnfrageArbeiter sysKalAnfrageArbeiter;

	/**
	 * Quelle.
	 */
	private Quelle quelle = Quelle.systemkalender;

	/**
	 * Liste der selektierten Bereiche.
	 */
	private final List<IEintragBereich> selektierteBereiche = new LinkedList<IEintragBereich>();

	/**
	 * TableColumn fr die Eintragsart im jeweiligen Bereich.
	 */
	private TableColumn tcEintragsArt;

	private Quelle initialeQuelle;

	/**
	 * Titel fr Systemkalendereintrge.
	 */
	private static final String EINTRAGS_ART_SYSTEMKALENDER = "Systemkalendereintrag";

	/**
	 * Titel fr Ereigniskalendereintrge.
	 */
	private static final String EINTRAGS_ART_EREIGNISKALENDER = "Ereigniskalendereintrag";

	/**
	 * Konstruktor dieses Dialoges.
	 * 
	 * @param parent
	 *            {@link Shell}.
	 */
	public KalenderBereichDialog(final Shell parent) {
		this(parent, Quelle.systemkalender);
	}

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            die bergeordnete Shell
	 * @param initialeQuelle
	 *            die initiale Quelle fr Kalendereintrge
	 */
	public KalenderBereichDialog(final Shell parent, final Quelle initialeQuelle) {
		super(parent);
		this.initialeQuelle = initialeQuelle;
	}

	private void bestimmeBereich() {
		final GregorianCalendar gcStart = new GregorianCalendar();
		gcStart.set(idxJahr, idxMonat, 1, 0, 0, 0);
		gcStart.set(Calendar.MILLISECOND, 0);

		final GregorianCalendar gcEnde = new GregorianCalendar();
		gcEnde.setTime(new Date(gcStart.getTimeInMillis()));
		gcEnde.add(Calendar.MONTH, 1);
		// gcEnde.set(Calendar.DAY_OF_MONTH, 0);
		gcEnde.add(Calendar.MILLISECOND, -1);

		final ClientDavInterface veb = RahmenwerkService.getService()
				.getRahmenWerk().getDavVerbindung();
		if (veb != null) {
			if ((quelle != null) && (quelle.equals(Quelle.systemkalender))) {
				tcEintragsArt
						.setText(KalenderBereichDialog.EINTRAGS_ART_SYSTEMKALENDER);

				starteSysKaArbeiter();

				final List<SkeKapsel> list = sysKalAnfrageArbeiter
						.berechneGueltigVonBisS(gcStart.getTimeInMillis(),
								gcEnde.getTimeInMillis());

				Collections.sort(list);
				tableViewerEintraege.setInput(list);

				/*
				 * Selektion in Tabelle Eintrge setzen
				 */
				if (tableViewerEintraege.getTable().getItemCount() > 0) {
					final StructuredSelection ss = new StructuredSelection(
							tableViewerEintraege.getElementAt(0));
					tableViewerEintraege.setSelection(ss, true);

				}
			} else if ((quelle != null)
					&& (quelle.equals(Quelle.ereigniskalender))) {
				tcEintragsArt
						.setText(KalenderBereichDialog.EINTRAGS_ART_EREIGNISKALENDER);

				starteSysKaArbeiter();

				final List<EintragEreigniskalender> eintraege = getEreigniskalenderEintraege();
				if (!eintraege.isEmpty()) {
					final LinkedList<IEintragKapsel> kapseln = new LinkedList<IEintragKapsel>();
					final Iterator<EintragEreigniskalender> it = eintraege
							.iterator();
					while (it.hasNext()) {
						final EintragEreigniskalender ereigniskalender = it
								.next();
						final ErkaKapsel kapsel = new ErkaKapsel(
								ereigniskalender, sysKalAnfrageArbeiter,
								gcStart.getTimeInMillis(),
								gcEnde.getTimeInMillis());
						kapseln.add(kapsel);
					}
					tableViewerEintraege.setInput(kapseln);

					/*
					 * Selektion in Tabelle Eintrge setzen
					 */
					if (tableViewerEintraege.getTable().getItemCount() > 0) {
						final StructuredSelection ss = new StructuredSelection(
								tableViewerEintraege.getElementAt(0));
						tableViewerEintraege.setSelection(ss, true);
					}
				}
			}
		}
	}

	private List<EintragEreigniskalender> getEreigniskalenderEintraege() {
		final List<EintragEreigniskalender> result = new LinkedList<EintragEreigniskalender>();

		final Rahmenwerk rahmenWerk = RahmenwerkService.getService()
				.getRahmenWerk();
		if (rahmenWerk.isOnline()) {
			final ConfigurationObject konfigurationsObjekt = rahmenWerk
					.getDavVerbindung().getLocalConfigurationAuthority();

			final List<SystemObject> objekte = konfigurationsObjekt
					.getObjectSet("Ereignisse").getElements();

			if (!objekte.isEmpty()) {
				final DataModel dataModel = rahmenWerk.getDavVerbindung()
						.getDataModel();
				final DataDescription desc = new DataDescription(
						dataModel.getAttributeGroup("atg.ereignisParameter"),
						dataModel.getAspect("asp.parameterSoll"));

				for (final SystemObject objekt : objekte) {
					final ResultData data = rahmenWerk.getDavVerbindung()
							.getData(objekt, desc, 0L);
					if (data.hasData()) {
						result.add(new EintragEreigniskalender(objekt, data
								.getData()));
					}
				}
			}
		}

		return result;
	}

	@Override
	protected void cancelPressed() {
		selektierteBereiche.clear();
		super.cancelPressed();
	}

	@Override
	protected final Control createDialogArea(final Composite parent) {

		final Composite panel = new Composite(parent, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);
		panel.setLayout(new GridLayout());

		setTitle("Ereignis- und Systemkalenderbernahme");
		setTitleImage(AbstractUIPlugin.imageDescriptorFromPlugin(
				RwBasisLibActivator.PLUGIN_ID,
				"icons/wizban/bitctrl_wizard.png").createImage());

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;

		if (!RahmenwerkService.getService().getRahmenWerk().isOnline()) {
			labelDaVVerbindung = new Label(panel, SWT.NONE);
			labelDaVVerbindung.setForeground(new Color(PlatformUI
					.getWorkbench().getDisplay(), 255, 0, 0));
			labelDaVVerbindung.setText("Keine DaV-Verbindung aufgebaut!");
		}

		erzeugeGruppeQuelle(panel);
		erzeugeGruppeEintraege(panel);
		erzeugeGruppeBereiche(panel);

		final GregorianCalendar gc = new GregorianCalendar();
		idxJahr = gc.get(Calendar.YEAR);
		idxMonat = gc.get(Calendar.MONTH);

		/*
		 * Felder fr Jahr und Monat vorbelegen
		 */
		textJahr.setText(Integer.toString(idxJahr));
		textMonat.setText(KalenderBereichDialog.MONATE[idxMonat]);

		/*
		 * Bereich bestimmen
		 */
		bestimmeBereich();

		/* TODO Positionieren des Dialogs bei der Position des Mauszeigers ? */
		return panel;
	}

	/**
	 * This method initializes compositeDatum.
	 * 
	 */
	private void erzeugeCompositeDatum(final Composite parent) {

		final Composite compositeDatum = new Composite(parent, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, false)
				.applyTo(compositeDatum);
		compositeDatum.setLayout(new GridLayout(3, false));

		final Button schaltflaecheLinksJahr = new Button(compositeDatum,
				SWT.NONE);
		schaltflaecheLinksJahr.setText("<");
		schaltflaecheLinksJahr.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				idxJahr = idxJahr - 1;
				textJahr.setText(Integer.toString(idxJahr));
				bestimmeBereich();
			}
		});

		textJahr = new Text(compositeDatum, SWT.BORDER | SWT.CENTER);
		textJahr.setEditable(false);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(textJahr);
		final Button schaltflaecheRechtsJahr = new Button(compositeDatum,
				SWT.NONE);
		schaltflaecheRechtsJahr.setText(">");
		schaltflaecheRechtsJahr.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				idxJahr = idxJahr + 1;
				textJahr.setText(Integer.toString(idxJahr));
				bestimmeBereich();
			}
		});

		final Button schaltflaecheLinksMonat = new Button(compositeDatum,
				SWT.NONE);
		schaltflaecheLinksMonat.setText("<");
		schaltflaecheLinksMonat.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (idxMonat >= 0) {
					idxMonat = idxMonat - 1;
				}
				if (idxMonat < 0) {
					idxMonat = 11;
				}

				textMonat.setText(KalenderBereichDialog.MONATE[idxMonat]);
				bestimmeBereich();
			}
		});

		textMonat = new Text(compositeDatum, SWT.BORDER | SWT.CENTER);
		textMonat.setEditable(false);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(textMonat);
		final Button schaltflaecheRechtsMonat = new Button(compositeDatum,
				SWT.NONE);
		schaltflaecheRechtsMonat.setText(">");
		schaltflaecheRechtsMonat.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (idxMonat <= 11) {
					idxMonat = idxMonat + 1;
				}
				if (idxMonat > 11) {
					idxMonat = 0;
				}

				textMonat.setText(KalenderBereichDialog.MONATE[idxMonat]);
				bestimmeBereich();
			}
		});
	}

	private void erzeugeGruppeBereiche(final Composite parent) {

		final Group gruppeBereiche = new Group(parent, SWT.NONE);
		gruppeBereiche.setLayout(new GridLayout());
		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 200)
				.applyTo(gruppeBereiche);
		gruppeBereiche.setText("Bereiche:");

		tableViewerBereiche = new TableViewer(gruppeBereiche, SWT.MULTI
				| SWT.FULL_SELECTION);
		tableViewerBereiche.getTable().setHeaderVisible(true);
		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 200)
				.applyTo(tableViewerBereiche.getTable());
		tableViewerBereiche.getTable().setLinesVisible(true);

		final TableColumn tcVon = new TableColumn(
				tableViewerBereiche.getTable(), SWT.LEFT);
		tcVon.setText("Von:");
		tcVon.setWidth(175);

		final TableColumn tcBis = new TableColumn(
				tableViewerBereiche.getTable(), SWT.LEFT);
		tcBis.setText("Bis:");
		tcBis.setWidth(175);

		final SysErKalBereichContentProvider cp = new SysErKalBereichContentProvider();
		tableViewerBereiche.setContentProvider(cp);
		final SysErKalBereichLabelProvider lp = new SysErKalBereichLabelProvider();
		tableViewerBereiche.setLabelProvider(lp);

		tableViewerBereiche
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection) {
							final IStructuredSelection iss = (IStructuredSelection) selection;

							/* Liste fllen */
							selektierteBereiche.clear();
							for (final Object object : iss.toArray()) {
								if (object instanceof IEintragBereich) {
									final IEintragBereich bereich = (IEintragBereich) object;
									selektierteBereiche.add(bereich);
								}
							}
						}
					}

				});

	}

	/**
	 * Initialisiert gruppeEintraege.
	 * 
	 */
	private void erzeugeGruppeEintraege(final Composite parent) {

		final Group gruppeEintraege = new Group(parent, SWT.NONE);
		gruppeEintraege.setLayout(new GridLayout());
		gruppeEintraege.setText("Eintrge:");
		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 200)
				.applyTo(gruppeEintraege);

		tableViewerEintraege = new TableViewer(gruppeEintraege,
				SWT.FULL_SELECTION);
		tableViewerEintraege.getTable().setHeaderVisible(true);
		GridDataFactory.fillDefaults().grab(true, true).hint(SWT.DEFAULT, 200)
				.applyTo(tableViewerEintraege.getTable());
		tableViewerEintraege.getTable().setLinesVisible(true);

		tcEintragsArt = new TableColumn(tableViewerEintraege.getTable(),
				SWT.LEFT);
		tcEintragsArt.setText("Systemkalendereintrag");
		tcEintragsArt.setWidth(345);

		final SysErKalEintragContentProvider cp = new SysErKalEintragContentProvider();
		tableViewerEintraege.setContentProvider(cp);

		final SysErKalEintragLabelProvider lp = new SysErKalEintragLabelProvider();
		tableViewerEintraege.setLabelProvider(lp);

		tableViewerEintraege
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection) {
							final IStructuredSelection iss = (IStructuredSelection) selection;
							final Object firstElement = iss.getFirstElement();
							if (firstElement instanceof IEintragKapsel) {
								final IEintragKapsel kapsel = (IEintragKapsel) firstElement;
								tableViewerBereiche.setInput(kapsel);

								/*
								 * Erstes Element selektieren
								 */
								if (tableViewerBereiche.getTable()
										.getItemCount() > 0) {

									tableViewerBereiche.getControl().setFocus();
									tableViewerBereiche.getTable().selectAll();
								}
							}
						}
					}

				});

	}

	/**
	 * Initialisiert gruppeQuelle.
	 * 
	 */
	private void erzeugeGruppeQuelle(final Composite parent) {

		final Group gruppeQuelle = new Group(parent, SWT.NONE);
		gruppeQuelle.setText("Quelle:");
		gruppeQuelle.setLayout(new GridLayout());
		GridDataFactory.fillDefaults().grab(true, false).applyTo(gruppeQuelle);

		final ComboViewer comboViewer = new ComboViewer(gruppeQuelle);
		final Quelle[] quellen = new Quelle[] { Quelle.systemkalender,
				Quelle.ereigniskalender };
		comboViewer.add(quellen);
		comboViewer.setSelection(new StructuredSelection(initialeQuelle));
		/*
		 * Selektion der Quelle
		 */
		comboViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection) {
							final IStructuredSelection iss = (IStructuredSelection) selection;

							if (iss.getFirstElement() instanceof Quelle) {
								final Quelle elementQuelle = (Quelle) iss
										.getFirstElement();
								quelle = elementQuelle;
								bestimmeBereich();
							}
						}

					}

				});
		GridDataFactory.fillDefaults().grab(true, false)
				.applyTo(comboViewer.getControl());

		erzeugeCompositeDatum(gruppeQuelle);
	}

	/**
	 * liefert eine Liste der im DIalog ausgewhlten Bereiche.
	 * 
	 * @return die Liste der Bereiche
	 */
	public final List<IEintragBereich> getSelektierteBereiche() {
		return selektierteBereiche;
	}

	@Override
	protected int getShellStyle() {
		return SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE;
	}

	@Override
	protected void okPressed() {
		final ISelection selection = tableViewerBereiche.getSelection();

		if (selection.isEmpty()) {
			selektierteBereiche.clear();
		} else if (selection instanceof IStructuredSelection) {

			final IStructuredSelection iss = (IStructuredSelection) selection;

			/* Liste fllen. */
			selektierteBereiche.clear();
			for (final Object object : iss.toArray()) {
				if (object instanceof IEintragBereich) {
					final IEintragBereich bereich = (IEintragBereich) object;
					selektierteBereiche.add(bereich);
				}
			}
		}
		super.okPressed();
	}

	/**
	 * setzt die initiale Quelle fr die bernahme von Kalendereintrgen.
	 * 
	 * @param initialeQuelle
	 *            die Quelle
	 */
	public void setInitialeQuelle(final Quelle initialeQuelle) {
		this.initialeQuelle = initialeQuelle;
	}

	/**
	 * Setzt das Jahr.
	 * 
	 * @param jahr
	 *            Jahr
	 */
	public void setJahr(final int jahr) {
		this.idxJahr = jahr;
		textJahr.setText(Integer.toString(idxJahr));
	}

	/**
	 * Setzt den Monat. 0 = Januar, 1 = Februar, ..., 11 = Dezember.
	 * 
	 * @param monat
	 *            Index des Monat >= 0 und <= 11
	 */
	public void setMonat(final int monat) {
		if ((monat >= 0) && (monat <= 11)) {
			this.idxMonat = monat;
			textMonat.setText(KalenderBereichDialog.MONATE[idxMonat]);
		}
	}

	private void starteSysKaArbeiter() {
		final ClientDavInterface veb = RahmenwerkService.getService()
				.getRahmenWerk().getDavVerbindung();
		final String kalenderObj = veb.getLocalConfigurationAuthority()
				.getPid();

		if (sysKalAnfrageArbeiter == null) {
			sysKalAnfrageArbeiter = new SysKalAnfrageArbeiter(veb, kalenderObj);
		}

		if (!sysKalArbeiterGestartet) {
			sysKalAnfrageArbeiter.getSyskalArbeiter()
					.starteSystemKalenderArbeiter();
			sysKalArbeiterGestartet = true;
		}
	}

}
