/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;

import de.bsvrz.buv.rw.basislib.kalender.internal.FeldInfo;
import de.bsvrz.buv.rw.basislib.kalender.internal.SelektierInfo;
import de.bsvrz.buv.rw.basislib.kalender.internal.ValidierungsInfo;
import de.bsvrz.buv.rw.basislib.kalender.internal.ZeitDauerGranularitaet;
import de.bsvrz.buv.rw.basislib.kalender.internal.ZeitKlasse;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Reprsentation einer Zeitdauer.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ZeitDauer.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class ZeitDauer {

	/** Ausgangszeitpunkt. */
	private long ausgangsZeitpunkt = new Date().getTime();

	/** Map der gltigen Zeichen. */
	private Map<String, String> tmGueltigeZeichen;

	/** Gibt an, ob auch Text editiert werden darf. */
	private boolean vollEditierbar = true;

	/** Kurzdarstellung. */
	private boolean kurzDarstellung;

	private final Set<ZeitKlasse> darzustellendeElemente = new HashSet<ZeitKlasse>(
			Arrays.asList(ZeitKlasse.values()));

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 * 
	 * @param ausgangsZeitpunkt
	 *            Ausgangszeitpunkt
	 */
	ZeitDauer(final long ausgangsZeitpunkt) {
		this.ausgangsZeitpunkt = ausgangsZeitpunkt;
		init();
	}

	/**
	 * Prft, ob sich auch die Texte der {@link ZeitKlasse}n editieren lassen
	 * oder nur die Zahlenfelder.
	 * 
	 * @return Liefert <code>true</code>, wenn alles editierbar ist, sonst
	 *         <code>false</code>.
	 */
	boolean istAllesEditierbar() {
		return vollEditierbar;
	}

	/**
	 * Setzt die Editierbarkeit aller Felder. Es knnen sowohl Zahlen als auch
	 * {@link ZeitKlasse}nfelder editiert werden, oder nur die Zahlenfelder.
	 * 
	 * @param allesEditierbar
	 *            Wenn <code>true</code>, dann kann alles editiert werden, bei
	 *            <code>false</code> knnen nur die Zahlenfelder editiert
	 *            werden.
	 */
	void setAllesEditierbar(final boolean allesEditierbar) {
		this.vollEditierbar = allesEditierbar;
	}

	/**
	 * ermittelt, ob die Kurzdarstellung verwednet werden soll.
	 * 
	 * @return den Status
	 */
	boolean istKurzDarstellung() {
		return kurzDarstellung;
	}

	/**
	 * legt fest, ob die Kurzdarstellung verwendet wedren soll.
	 * 
	 * @param kurzdarstellung
	 *            der gewnschte Zustand
	 */
	void setKurzDarstellung(final boolean kurzdarstellung) {
		this.kurzDarstellung = kurzdarstellung;
	}

	/**
	 * Fhrt die Initialisierung durch.
	 */
	private void init() {
		erstelleListeGueltigerStrings();
	}

	ZeitKlasse bestimmeZeitklassenZugehoerigkeit(final String wert) {
		ZeitKlasse re = null;

		for (final ZeitKlasse zeitKlasse : ZeitKlasse.values()) {
			if (wert.equalsIgnoreCase(zeitKlasse.getBezeichnung())) {
				re = zeitKlasse;
				break;
			} else if (wert.equalsIgnoreCase(zeitKlasse.getMehrzahl())) {
				re = zeitKlasse;
				break;
			} else if (wert.equals(zeitKlasse.getKurzform())) {
				re = zeitKlasse;
				break;
			}
		}

		return re;
	}

	boolean verifyEingabe(final String text) {
		boolean re = true;

		for (int i = 0; i < text.length(); i++) {
			final String zeichen = Character.toString(text.charAt(i));
			if (!verifyEingabe(zeichen, i, text, true)) {
				re = false;
				break;
			}
		}

		return re;
	}

	/**
	 * verifiziert die eingegebenen Daten.
	 * 
	 * @param zeichen
	 *            Hinzuzufgendes Zeichen
	 * @param position
	 *            Position, an der das Zeichen hinzugefgt werden soll
	 * @param gesamterText
	 *            Der gesamte Text des jeweiligen Textfeldes
	 * @param checkAllesEditierbar
	 *            Wenn <code>true</code>, kann alles editiert werden, wenn
	 *            <code>false</code> knnen nur Zahlen bearbeitet werden.
	 * @return Liefert <code>true</code> wenn die Eingabe verifiziert wurde,
	 *         ansonsten <code>false</code>.
	 */
	boolean verifyEingabe(final String zeichen, final int position,
			final String gesamterText, final boolean checkAllesEditierbar) {
		this.vollEditierbar = checkAllesEditierbar;

		/*
		 * Gltig-Flag
		 */
		boolean gueltig = true;

		if (zeichen.length() > 0) {
			/*
			 * Alles (inkl. Texte) editierbar?
			 */
			if (vollEditierbar) {

				// ///////////////////////////
				// ////Allgemeine Regeln//////
				// ///////////////////////////

				/*
				 * Prfen auf gltige Zeichen
				 */
				for (int i = 0; i < zeichen.length(); i++) {
					String vlg = Character.toString(zeichen.charAt(i));
					vlg = vlg.toLowerCase();
					if (tmGueltigeZeichen.containsKey(vlg)) {
						gueltig = true;
					} else {
						gueltig = false;
						break;
					}
				}

				/*
				 * Sicherstellen, dass das erste Zeichen eine Zahl ist.
				 */
				if ((position == 0) && gueltig) {
					try {
						Integer.valueOf(zeichen);
					} catch (final NumberFormatException e) {
						ZeitDauer.LOGGER.finer(e.getLocalizedMessage());
						gueltig = false;
					}
				}

				/*
				 * Sicherstellen, dass keine zwei Leerzeichen hintereinander
				 * eingegeben werden
				 */
				if (gueltig && (position > 0)) {
					/*
					 * Auf Vorgngerleerzeichen Prfen
					 */
					final String vorgaengerZeichen = Character
							.toString(gesamterText.charAt(position - 1));

					if ((" ".equals(zeichen))
							&& (zeichen.equals(vorgaengerZeichen))) {
						gueltig = false;
					}

					/*
					 * Auf Nachfolgerleerzeichen (also 2 Leerzeichen
					 * hintereinander) prfen
					 */
					if (gueltig && (position < gesamterText.length())) {
						final String zeichenAktuell = Character
								.toString(gesamterText.charAt(position));
						if ((position + 1) < gesamterText.length()) {
							final String zeichenNachfolgend = Character
									.toString(gesamterText.charAt(position + 1));
							if ((" ".equals(zeichenAktuell))
									&& (" ".equals(zeichenNachfolgend))) {
								gueltig = false;
								ZeitDauer.LOGGER
										.warning("Keine zwei Leerzeichen nacheinander");
							}
						}

					}

				}

				/*
				 * Vor einem Buchstaben darf keine Zahl, vor einer Zahl kein
				 * Buchstabe stehen
				 */
				if (gueltig && (gesamterText.length() > 2)
						&& ((position - 1) > 0)) {
					final String vglVorgaenger = Character
							.toString(gesamterText.charAt(position - 1));

					if (zeichen.matches("\\d")
							&& vglVorgaenger.matches("[a-zA-Z]")) {
						gueltig = false;
					} else if (zeichen.matches("[a-zA-Z]")
							&& vglVorgaenger.matches("\\d")) {
						gueltig = false;
					}
				}

				/*
				 * Nach einer Zeitklasse muss eine Zahl folgen. Nach einer Zahl
				 * muss eine Zeitklasse folgen.
				 */
				if (gueltig && (position > 0) && (gesamterText.length() > 2)) {
					/*
					 * Ist Vorgnger ein Leerzeichen?
					 */
					final String vglLeerzeichen = Character
							.toString(gesamterText.charAt(position - 1));
					if (" ".equals(vglLeerzeichen)) {
						final String vglVorgaenger = Character
								.toString(gesamterText.charAt(position - 2));

						if (zeichen.matches("\\d")
								&& vglVorgaenger.matches("\\d")) {
							gueltig = false;
						} else if (zeichen.matches("[a-zA-Z]")
								&& vglVorgaenger.matches("[a-zA-Z]")) {
							gueltig = false;
						}

					}

				}

				// //////////////////////////////
				// ////Spezielle Regeln fr//////
				// ////die Zeichenklassen //////
				// //////////////////////////////

				/*
				 * Prfen, ob die eingegebene Zeichenklasse gltig ist -
				 * Verwendete Zeichen
				 */
				if (gueltig) {
					String vgl = null;
					if (zeichen.matches("[a-zA-Z]")) {
						// Ist Vorgnger Leerzeichen
						final String vorg = Character.toString(gesamterText
								.charAt(position - 1));
						if (" ".equals(vorg)) {
							vgl = zeichen;
						} else {
							int startOffset = -1;
							for (int i = position - 1; i > 0; i--) {
								final String wert = Character
										.toString(gesamterText.charAt(i));
								if (" ".equals(wert)) {
									startOffset = i + 1;
									break;
								}
							}

							vgl = gesamterText.substring(startOffset, position)
									+ zeichen;

						}

						if (!pruefeVorkommenInZeitKlassen(vgl)) {
							gueltig = false;
						}

					}
				}

				/*
				 * Prfen auf korrekte Schreibweise einer Zeitklasse
				 * (Case-Insensetive)
				 */
				if (gueltig && zeichen.matches("[a-zA-Z]")) {
					final String bestimmeString = bestimmeStringZeitKlassenEingabe(
							gesamterText, zeichen, position);
					istGueltigeZeitklasse(bestimmeString);
				}

				/*
				 * Prfen, ob die Zeichenklasse schon mal am Stck verwendet
				 * wurde
				 */

			} else {
				/*
				 * Nur Zahlen editierbar public boolean verifyEingabe(String
				 * zeichen, int position, String gesamterText, boolean
				 * allesEditierbar)
				 */

				/*
				 * Zeichen ist Zahl?
				 */
				if (zeichen.matches("\\d")) {
					final String vorZeichen = Character.toString(gesamterText
							.charAt(position));
					final String nachZeichen = Character.toString(gesamterText
							.charAt(position + 1));

					if (!(((" ".equals(vorZeichen)) && (nachZeichen
							.matches("\\d"))) || ((vorZeichen.matches("\\d")) && (" "
							.equals(nachZeichen))))) {
						gueltig = false;
					}

				} else {
					gueltig = false;
				}

			}
		}

		return gueltig;
	}

	boolean pruefeVorkommenInZeitKlassen(final String string) {
		boolean re = false;

		for (final ZeitKlasse zeitKlasse : ZeitKlasse.values()) {
			if (zeitKlasse.getBezeichnung().toLowerCase()
					.contains(string.toLowerCase())) {
				re = true;
				break;
			} else if (zeitKlasse.getMehrzahl().toLowerCase()
					.contains(string.toLowerCase())) {
				re = true;
				break;
			} else if (zeitKlasse.getKurzform().toLowerCase()
					.contains(string.toLowerCase())) {
				re = true;
				break;
			}
		}

		return re;
	}

	/**
	 * Bestimmt die Position des nchsten Zahlenfeldes.
	 * 
	 * @param text
	 *            Zu prfendens Textfeld
	 * @param carretPos
	 *            Aktuelle Position des Carrets
	 * @param aktSelStartPos
	 *            Start-Position der Selektion
	 * @param aktSelEndPos
	 *            End-Position der Selektion
	 * @return Zu selektierende Position
	 */
	SelektierInfo bestimmeNaechstesZahlenfeld(final String text,
			final int carretPos, final int aktSelStartPos,
			final int aktSelEndPos) {

		SelektierInfo re = null;

		if ((text != null) && (text.length() > 0) && (carretPos >= 0)) {
			int startPos = -1;
			int endPos = -1;

			int pos = 0;

			/*
			 * Ausgehend von der Carretpositon (Zeichenpostion + 1) wird
			 * sichergestellt, dass das nchste Feld selektiert wird.
			 */
			if (carretPos > 0) {
				if (carretPos == text.length()) {
					pos = 0;
				} else {
					pos = carretPos;
				}
			}

			/*
			 * Aktuelles Zeichen ist Zahl? Wenn ja, Anfangsposition der Zahl
			 * suchen
			 */
			String startZeichen = Character.toString(text.charAt(pos));
			if (startZeichen.matches("\\d")) {
				startPos = pos;

				for (int i = pos; i >= 0; i--) {
					startZeichen = Character.toString(text.charAt(i));

					if (startZeichen.matches("\\d")) {
						startPos = i;
					} else {
						break;
					}
				}
			}

			/*
			 * Wenn noch keine Startposition gefunden wurde, dann StartPosition
			 * nach hinten bestimmen.
			 */
			if (startPos == -1) {
				for (int i = pos; i < text.length(); i++) {

					startZeichen = Character.toString(text.charAt(i));

					if (startZeichen.matches("\\d")) {
						startPos = i;
						break;
					}
				}
			}

			/*
			 * Bestimmen der Endposition
			 */
			String endZeichen = "";
			if (startPos >= 0) {
				for (int i = startPos; i < text.length(); i++) {
					endZeichen = Character.toString(text.charAt(i));
					endPos = i;
					if (!endZeichen.matches("\\d")) {
						endPos--;
						break;
					}
				}
			}

			/*
			 * Wenn Start- und Endposition noch nicht bestimmt sind dann ist
			 * davon auszugehen, das wir uns am Ende des Feldes befinden. Um den
			 * durchlauf zu garantieren, wird von vorne angefangen, wenn die
			 * CarretPosition > 0 war.
			 */
			if ((!((startPos >= 0) && (endPos >= 0))) && (carretPos > 0)) {
				final SelektierInfo naechstesZahlenfeld = bestimmeNaechstesZahlenfeld(
						text, 0, aktSelStartPos, aktSelEndPos);
				startPos = naechstesZahlenfeld.getStartPos();
				endPos = naechstesZahlenfeld.getEndPos();
			}

			if ((startPos >= 0) && (endPos >= 0)) {
				re = new SelektierInfo(startPos, endPos, text);
			} else {
				ZeitDauer.LOGGER
						.warning("Start & Endposition konnten nicht bestimmt werden");
			}

		}

		return re;
	}

	/**
	 * Bestimmt die Position des nchsten Zahlenfeldes.
	 * 
	 * @param text
	 *            Zu prfendens Textfeld
	 * @param carretPos
	 *            Aktuelle Position des Carrets
	 * @param aktSelStartPos
	 *            Start-Position der Selektion
	 * @param aktSelEndPos
	 *            End-Position der Selektion
	 * @return Zu selektierende Position
	 */
	SelektierInfo bestimmeVorhergehendesZahlenfeld(final String text,
			final int carretPos, final int aktSelStartPos,
			final int aktSelEndPos) {
		SelektierInfo re = null;

		if ((text != null) && (text.length() > 0) && (carretPos >= 0)) {
			int startPos = -1;
			int endPos = -1;

			int pos = 0;

			/*
			 * Ausgehend von der Carretpositon (Zeichenpostion + 1) wird
			 * sichergestellt, dass das nchste Feld selektiert wird.
			 */
			if (carretPos > 0) {
				if (carretPos == text.length()) {
					pos = 0;
				} else {
					pos = carretPos;
				}
			}

			/*
			 * Aktuelles Zeichen ist Zahl? Wenn ja, Anfangsposition der Zahl
			 * suchen
			 */

			String startZeichen = null;
			if (pos > 0) {
				startZeichen = Character.toString(text.charAt(pos - 1));

				if (startZeichen.matches("\\d")) {
					startPos = pos - 1;

					for (int i = pos - 1; i >= 0; i--) {
						startZeichen = Character.toString(text.charAt(i));

						if (startZeichen.matches("\\d")) {
							startPos = i;
						} else {
							break;
						}
					}
				}
			}

			/*
			 * Wenn noch keine Startposition gefunden wurde, dann StartPosition
			 * nach vorne bestimmen.
			 */
			boolean zahlGefunden = false;
			if (startPos == -1) {
				for (int i = pos; i >= 0; i--) {

					startZeichen = Character.toString(text.charAt(i));

					if (zahlGefunden) {
						if (startZeichen.matches("\\d")) {
							startPos = i;
						} else {
							startPos = i + 1;
							break;
						}
					} else {
						if (startZeichen.matches("\\d")) {
							startPos = i;
							zahlGefunden = true;
						}
					}

				}
			}

			/*
			 * Bestimmen der Endposition
			 */
			String endZeichen = "";
			if (startPos >= 0) {
				for (int i = startPos; i < text.length(); i++) {
					endZeichen = Character.toString(text.charAt(i));
					endPos = i;
					if (!endZeichen.matches("\\d")) {
						endPos--;
						break;
					}
				}
			}

			if ((startPos >= 0) && (endPos >= 0)) {
				re = new SelektierInfo(startPos, endPos, text);
			}

			/*
			 * Befinden wir uns ganz am Anfang, dann wollen wir wieder von
			 * hinten beginnen.
			 */
			if ((aktSelStartPos == startPos) && ((aktSelEndPos - 1) == endPos)) {
				int neuPos = 0;
				if (startPos > 0) {
					neuPos = startPos - 1;
				} else {
					neuPos = text.length() - 1;
				}

				re = bestimmeVorhergehendesZahlenfeld(text, neuPos, neuPos,
						neuPos);
			}

		}

		return re;
	}

	/**
	 * Bestimmt die Markierung eine Zahl in der direkten Nhe des Carrets.
	 * 
	 * @param text
	 *            Text, in dem die Markierung bestimmt werden soll
	 * @param pos
	 *            Position des Carrets
	 * @return Array mit Start und Endposition der Markierung oder
	 *         <code>null</code>, wenn keine Bestimmung statt finden konnte.
	 */
	int[] bestimmeMarkierung(final String text, final int pos) {
		int[] re = null;

		int startPos = pos;
		int endPos = -1;

		/*
		 * Ist Zeichen Zahl? Wenn ja, dann gehe in der Folge der Zahlen weiter
		 * vor.
		 */
		if (startPos > 0) {
			for (int i = (pos - 1); i >= 0; i--) {
				final String zeichen = Character.toString(text.charAt(i));

				startPos = i;

				if (!zeichen.matches("\\d")) {
					i = i + 1;
					startPos = i;
					break;
				}
			}
		}

		if (startPos == pos) {
			if (pos < text.length()) {
				final String zeichen = Character
						.toString(text.charAt(startPos));
				if (!zeichen.matches("\\d")) {
					startPos = -1;
				}
			} else {
				startPos = -1;
			}
		}

		/*
		 * Bestimmung nach hinten
		 */
		for (int i = pos; i < text.length(); i++) {
			final String zeichen = Character.toString(text.charAt(i));

			if (!zeichen.matches("\\d")) {
				i = i - 1;
				endPos = i;
				break;
			}
		}

		if (endPos == pos) {
			final String zeichen = Character.toString(text.charAt(endPos));
			if (!zeichen.matches("\\d")) {
				endPos = -1;
			}
		}

		/*
		 * Prfen, ob Strings an Start & Endpos auch Zeichen sind
		 */
		if (startPos >= 0) {
			final String zeichenStartPos = Character.toString(text
					.charAt(startPos));
			if (!zeichenStartPos.matches("\\d")) {
				startPos = -1;
			}
		}

		if ((endPos >= 0) && (endPos < text.length())) {
			final String zeichenEndPos = Character
					.toString(text.charAt(endPos));
			if (!zeichenEndPos.matches("\\d")) {
				endPos = -1;
			}
		}

		/*
		 * Rckgabewert zusammenstellen:
		 */
		if ((startPos >= 0) && (endPos >= 0)) {
			re = new int[] { startPos, endPos };
		}

		return re;
	}

	/**
	 * ndert den Wert einer Zahl in dem bergebenen Textfeld.
	 * 
	 * @param text
	 *            Zu bearbeitender Text
	 * @param selectionText
	 *            Selektierter Text im Textfeld
	 * @param carretPos
	 *            Carret-Position, wird am Ende der Selektion im Textfeld
	 *            postiert
	 * @param wert
	 *            Wert, der zu dem Zahlenfeld addiert / subtrahiert werden soll
	 * @return Informationen zur Selektion
	 */
	SelektierInfo wertAendern(final String text, final String selectionText,
			final int carretPos, final int wert) {
		SelektierInfo re = null;

		if ((text != null) && (!text.isEmpty()) && (selectionText != null)
				&& (!selectionText.isEmpty())) {

			final int startPos = carretPos - selectionText.length();
			final int endPosAlt = (startPos + selectionText.length()) - 1;

			/* Ist selektierter Text eine Zahl */
			if (selectionText.matches("\\d*")) {
				final int zahl = Integer.valueOf(selectionText);
				int zahlNeu = zahl + wert;

				if (zahlNeu < 0) {
					zahlNeu = 0;
				}

				final String strZahlNeu = Integer.toString(zahlNeu);

				String ersterTeil = "";
				if (startPos > 0) {
					ersterTeil = text.substring(0, startPos);

				}

				String letzterTeil = "";
				if ((endPosAlt + 1) < text.length()) {
					letzterTeil = text.substring(endPosAlt + 1);
				}

				final String neuerText = ersterTeil + strZahlNeu + letzterTeil;
				final int endPosNeu = (startPos + strZahlNeu.length()) - 1;
				re = new SelektierInfo(startPos, endPosNeu, neuerText);
			}
		}

		return re;
	}

	/**
	 * Bestimmt die {@link ZeitKlasse} eines Textes nach der Endposition einer
	 * Zahl.
	 * 
	 * @param text
	 *            Text, an Hand dem die {@link ZeitKlasse} bestimmt werden soll.
	 * @param endPosZahl
	 *            Endposition der letzten Zahl des Wertes der {@link ZeitKlasse}
	 *            .
	 * @return Liefert die {@link ZeitKlasse}, die an Hand des eingegebenen
	 *         Textes bestimmt werden konnte.
	 */
	ZeitKlasse bestimmeZeitKlasse(final String text, final int endPosZahl) {
		ZeitKlasse re = null;

		if ((endPosZahl < text.length()) && ((endPosZahl + 1) < text.length())) {

			final int startPos = endPosZahl + 2;
			int endPos = startPos;

			/*
			 * Bereich des Zeitklassen-Strings bestimmen
			 */
			for (int i = startPos; i < text.length(); i++) {

				final String aktZeichen = Character.toString(text.charAt(i));
				if (aktZeichen.matches("[a-zA-Z]")) {
					endPos = i;
				} else {
					break;
				}
			}

			final String pruefling = text.substring(startPos, endPos + 1);

			/*
			 * Passende Zeitklasse fr diesen String finden
			 */

			for (final ZeitKlasse zeitKlasse : ZeitKlasse.values()) {
				boolean gefunden = false;

				if (zeitKlasse.getBezeichnung().equalsIgnoreCase(pruefling)) {
					gefunden = true;
				} else if (zeitKlasse.getMehrzahl().equalsIgnoreCase(pruefling)) {
					gefunden = true;
				} else if (zeitKlasse.getKurzform().equals(pruefling)) {
					gefunden = true;
				}

				if (gefunden) {
					re = zeitKlasse;
					break;
				}
			}

		}

		return re;
	}

	/**
	 * Liefert eine Liste von Gltigen Zeichen, welche erfasst werden drfen.
	 * 
	 * @return Liste gltiger Zeichen.
	 */
	Map<String, String> getListeGueltigerStrings() {
		return tmGueltigeZeichen;
	}

	/**
	 * Bestimmt die Zeichen, die eingegeben werden drfen.
	 * 
	 * @return {@link TreeMap} von eingebbaren Zeichen.
	 */
	private Map<String, String> erstelleListeGueltigerStrings() {
		if (tmGueltigeZeichen == null) {

			/*
			 * Gltige Zeichen aus den vorhandenen ZeitKlassen erstellen
			 */

			tmGueltigeZeichen = new TreeMap<String, String>();
			final LinkedList<String[]> strgs = new LinkedList<String[]>();
			for (final ZeitKlasse zeitKlasse : ZeitKlasse.values()) {
				strgs.add(separiereZeichenAusString(zeitKlasse.getBezeichnung()));
				strgs.add(separiereZeichenAusString(zeitKlasse.getMehrzahl()));
				strgs.add(separiereZeichenAusString(zeitKlasse.getKurzform()));
			}

			final Iterator<String[]> it = strgs.iterator();
			while (it.hasNext()) {
				final String[] werte = it.next();
				for (final String zeichen : werte) {
					if (!tmGueltigeZeichen.containsKey(zeichen)) {
						tmGueltigeZeichen.put(zeichen, zeichen);
					}
				}
			}

			/*
			 * Zahlen 0-9 hinzufgen
			 */
			for (int i = 0; i < 10; i++) {
				final String wert = Integer.toString(i);
				if (!tmGueltigeZeichen.containsKey(wert)) {
					tmGueltigeZeichen.put(wert, wert);
				}
			}

			/*
			 * Leerzeichen
			 */
			tmGueltigeZeichen.put(" ", " ");
		}

		return tmGueltigeZeichen;
	}

	/**
	 * Liefert ein Array mit den einzelnen Strings eines Strings. Jedes Zeichen
	 * kommt dabei nur ein mal vor.
	 * 
	 * @param wert
	 *            String, dessen Zeichen bestimmt werden sollen.
	 * @return Zeichen des Strings in einem Array
	 */
	private String[] separiereZeichenAusString(final String wert) {
		String[] re = null;
		final HashMap<String, String> hm = new HashMap<String, String>();

		/*
		 * String Parsen
		 */
		for (int i = 0; i < wert.length(); i++) {
			String zeichen = Character.toString(wert.charAt(i));
			zeichen = zeichen.toLowerCase();

			if (!hm.containsKey(zeichen)) {
				hm.put(zeichen, zeichen);
			}
		}

		/*
		 * Rckgabefeld erstellen
		 */
		re = new String[hm.size()];
		final Iterator<Entry<String, String>> it = hm.entrySet().iterator();
		int i = 0;
		while (it.hasNext()) {
			final Entry<String, String> entry = it.next();
			re[i] = entry.getValue();
			i++;
		}

		return re;
	}

	/**
	 * Bestimmt an Hand der angegebenen einfuegePosition den bist dato dort
	 * vorhandenen kompletten String, der sich gegen [a-zA-Z] matchen lsst. Er
	 * wird von Anfang bist Ende bestimmt. <br>
	 * Es ist hiermit eine Prfung daraufhin mglich, zu welcher Zeitklasse der
	 * String wohl gehren wird.
	 * 
	 * @param gesammt
	 *            Der gesamte String des Feldes
	 * @param einfuegen
	 *            Einzufgender String
	 * @param einfuegePosition
	 *            Die Position, an der der neue String eingefgt werden soll.
	 * @return Liefert den jeweiligen String oder <code>null</code>, wenn an
	 *         dieser Position keiner gematched werden konnte
	 */
	String bestimmeStringZeitKlassenEingabe(final String gesammt,
			final String einfuegen, final int einfuegePosition) {
		String re = null;

		int startPosition = -1;
		int endPosition = -1;

		String vorEinfuegePosition = null;
		String nachEinfuegePosition = null;

		if (!(einfuegePosition >= (gesammt.length() + 1))) {
			/*
			 * Nach vorne parsen
			 */
			if (einfuegePosition > 0) {
				for (int i = einfuegePosition - 1; i >= 0; i--) {
					final String pruefling = Character.toString(gesammt
							.charAt(i));

					if (!pruefling.matches("[a-zA-Z]")) {
						startPosition = i + 1;
						break;
					}
					startPosition = i;
				}
			} else {
				startPosition = 0;
			}

			if ((startPosition >= 0) && (einfuegePosition >= 0)) {
				vorEinfuegePosition = gesammt.substring(startPosition,
						einfuegePosition);
			}

			/*
			 * Nach hinten Parsen von der einfuegePosition an
			 */
			if (einfuegePosition >= 0) {
				if (einfuegePosition < gesammt.length()) {
					for (int i = einfuegePosition; i < gesammt.length(); i++) {
						final String pruefling = Character.toString(gesammt
								.charAt(i));

						if (!pruefling.matches("[a-zA-Z]")) {
							endPosition = i - 1;

							break;
						}
						endPosition = i;
					}

					nachEinfuegePosition = gesammt.substring(einfuegePosition,
							endPosition + 1);
				} else {
					nachEinfuegePosition = "";
				}
			}

			if (!(startPosition >= gesammt.length())) {

				if (einfuegePosition >= gesammt.length()) {
					endPosition = einfuegePosition;
				}

				if (einfuegePosition >= 0) {
					vorEinfuegePosition = gesammt.substring(startPosition,
							einfuegePosition);

				}
				re = vorEinfuegePosition + einfuegen + nachEinfuegePosition;
				if (startPosition >= endPosition) {
					re = null;
				}
			}
		}

		/*
		 * Korrektur berflssiges Leerzeichen!!!
		 */
		if (re != null) {
			final String z = Character.toString(re.charAt(re.length() - 1));
			if (" ".equals(z)) {
				re = re.substring(0, re.length() - 1);
			}
		}

		return re;
	}

	/**
	 * Prft, ob ein String zu einer {@link ZeitKlasse} zugehrig ist. Bei
	 * Bezeichnung und Mehrzahl der {@link ZeitKlasse} wird dabei
	 * Case-Insensetive geprft.
	 * 
	 * @param wert
	 *            Zu prfender Wert.
	 * @return Liefert <code>true</code>, wenn es sich um eine gltige
	 *         {@link ZeitKlasse} handelt, sonst <code>false</code>.
	 */
	boolean istGueltigeZeitklasse(final String wert) {
		boolean re = false;

		for (final ZeitKlasse zeitKlasse : ZeitKlasse.values()) {

			final String bezeichnung = zeitKlasse.getBezeichnung()
					.toLowerCase();
			final String mehrzahl = zeitKlasse.getMehrzahl().toLowerCase();
			final String kurz = zeitKlasse.getKurzform();

			final StringBuffer chk = new StringBuffer();

			if (wert != null) {
				for (int i = 0; i < wert.length(); i++) {
					chk.append(wert.charAt(i));
				}
			}

			final String chkStr = chk.toString();
			if (bezeichnung.contains(chkStr.toLowerCase())) {
				re = true;
				break;
			}

			if (mehrzahl.contains(chkStr.toLowerCase())) {
				re = true;
				break;
			}

			if ((wert != null) && (wert.length() == 1) && kurz.equals(chkStr)) {
				re = true;
				break;
			}
		}

		return re;
	}

	/**
	 * Bestimmt die gltigen Felder und erzeugt einen im Feld anzeigbaren
	 * String. Fehlerhafte Werde werden separat bermittelt.
	 * 
	 * @param text
	 *            Der zu validierende String.
	 * @return Liefert eine Instanz der Klasse {@link ValidierungsInfo}, welche
	 */
	ValidierungsInfo validiereFelder(final String text) {
		ValidierungsInfo re = null;

		/** Liste der Felder einer Zeitdauer */
		final LinkedList<FeldInfo> felder = new LinkedList<FeldInfo>();

		/** Liste von nicht bestimmbaren Feldern. */
		final LinkedList<String> nichtBestimmbar = new LinkedList<String>();

		final String[] split = text.split(" ");
		if (split.length >= 2) {

			for (int i = 0; i < split.length; i++) {

				if (split[i].matches("\\d*")) {
					final int zahl = Integer.valueOf(split[i]);

					if ((i + 1) < (split.length)) {
						final ZeitKlasse zeitKlasse = bestimmeZeitklassenZugehoerigkeit(split[i + 1]);
						if (zeitKlasse == null) {
							/*
							 * Zeitklasse war nicht bestimmbar
							 */
							final String meldung = split[i] + " "
									+ split[i + 1];
							nichtBestimmbar.add(meldung);

						} else {
							/*
							 * Kurzform verwenden? Wird an Hand des Strings
							 * festgemacht.
							 */
							FeldInfo feldInfo = null;

							if (split[i + 1].length() == 1) {
								feldInfo = new FeldInfo(zeitKlasse, true, zahl);
							} else {
								feldInfo = new FeldInfo(zeitKlasse, false, zahl);
							}

							if (felder.isEmpty()) {
								felder.add(feldInfo);
							} else {
								/*
								 * Prfen ob Feld vom gleichen Typ schon
								 * existiert.
								 */
								boolean schonda = false;
								final Iterator<FeldInfo> it = felder.iterator();
								while (it.hasNext()) {
									final FeldInfo vglFeld = it.next();
									if (vglFeld
											.getTyp()
											.getBezeichnung()
											.equals(feldInfo.getTyp()
													.getBezeichnung())) {
										schonda = true;
										break;
									}
								}

								if (schonda) {
									ZeitDauer.LOGGER
											.fine("Folgendes Feld ist als Typ schon vorhanden: "
													+ feldInfo.getTyp()
															.getBezeichnung());
								} else {
									felder.add(feldInfo);
								}
							}
						}

						i = i + 1;
					} else {
						/* Kein Nachfolgerfeld fr eine Zahl */
						nichtBestimmbar.add(split[i]);
					}
				}
			}
		}

		/*
		 * Erzeugung des Rckgabewertes
		 */
		re = new ValidierungsInfo(felder, nichtBestimmbar);

		/*
		 * Gerade Feldanzahl? Wenn nein eine entsprechende sonstige Meldung
		 * setzen.
		 */
		if ((split.length % 2) > 0) {
			final String msg = "Die Anzahl der Felder muss gerade sein und dem Muster Zahl Leerzeichen\n"
					+ "Zeitklasse folgen. Das erste Feld muss eine Zahl sein und nach dem\n"
					+ "letzten Zeitklassenfeld darf keine Zahl folgen.";

			re.setSonstigeInfos(msg);
		}

		return re;

	}

	void setAusgangsZeitpunkt(final long ausgangsZeitPunkt) {
		this.ausgangsZeitpunkt = ausgangsZeitPunkt;
	}

	long getAusgangsZeitpunkt() {
		return ausgangsZeitpunkt;
	}

	/**
	 * Berechnet auf Grund der gesetzten Felder jahre, monate, tage, stunden,
	 * minuten, sekunden und millisekunden die Zeitdauer.
	 * 
	 * @param textFeld
	 *            die textliche Beschreibung der Zeitdauer
	 * @return Zeitdauer in Millisekunden
	 */
	long getZeitdauerInBezugAufAnfangszeitpunkt(final String textFeld) {

		int millisekunden = 0;
		int sekunden = 0;
		int minuten = 0;
		int stunden = 0;
		int tage = 0;
		int monate = 0;
		int jahre = 0;

		final ValidierungsInfo felder = validiereFelder(textFeld);

		final List<FeldInfo> feldInfos = felder.getFeldInfos();
		final Iterator<FeldInfo> it = feldInfos.iterator();
		while (it.hasNext()) {
			final FeldInfo info = it.next();

			final ZeitKlasse zeitKlasse = info.getTyp();

			switch (zeitKlasse) {
			case MILLSEKUNDE:
				millisekunden = info.getWert();
				break;
			case SEKUNDE:
				sekunden = info.getWert();
				break;
			case MINUTE:
				minuten = info.getWert();
				break;
			case STUNDEN:
				stunden = info.getWert();
				break;
			case TAG:
				tage = info.getWert();
				break;
			case MONAT:
				monate = info.getWert();
				break;
			case JAHR:
				jahre = info.getWert();
				break;
			default:
				break;
			}
		}

		final GregorianCalendar gcAnfang = new GregorianCalendar();
		gcAnfang.setTimeInMillis(ausgangsZeitpunkt);

		final GregorianCalendar gcEnde = (GregorianCalendar) gcAnfang.clone();

		gcEnde.add(Calendar.MILLISECOND, millisekunden);
		gcEnde.add(Calendar.SECOND, sekunden);
		gcEnde.add(Calendar.MINUTE, minuten);
		gcEnde.add(Calendar.HOUR_OF_DAY, stunden);
		gcEnde.add(Calendar.DAY_OF_MONTH, tage);
		gcEnde.add(Calendar.MONTH, monate);
		gcEnde.add(Calendar.YEAR, jahre);

		return gcEnde.getTimeInMillis() - gcAnfang.getTimeInMillis();
	}

	/**
	 * Berechnet die Basis einer Zeitdauer in Abhngigkeit von einem
	 * Startzeitpunkt.
	 * 
	 * @param startZeitpunkt
	 *            Startzeitpunkt, auf dessen Basis die Zeitdauer in JMThmst
	 *            bestimmt wird.
	 * @param zeitDauer
	 *            Zeitdauer in Millisekunden
	 * @return String fr das Textfeld
	 * 
	 */
	String setZeitdauerInBezugAufAnfangszeitpunkt(final long startZeitpunkt,
			final long zeitDauer) {
		final long ende = startZeitpunkt + zeitDauer;

		return setZeitdauerInBezugAufEndpunkt(zeitDauer, ende);
	}

	/**
	 * Berechnet die Basis einer Zeitdauer in Abhngigkeit von einem
	 * Endzeitpunkt den Anfangszeitpunkt und die zwischen Anfangszeitpunkt und
	 * Endzeitpunkt liegenden Jahre, Monate, Tage, Stunden, Minuten, Sekunden
	 * und Millisekunden. Schaltjahre und unterschiedliche Monatslngen werden
	 * bercksichtigt.
	 * 
	 * @param zeitDauer
	 *            Zeitdauer in Millisekunden.
	 * @param ende
	 *            Endzeitpunkt, von dem Anfangszeitpunkt und JTMhmst bis zum
	 *            Ende berechnet werden sollen.
	 * @return String fr das Textfeld
	 */
	String setZeitdauerInBezugAufEndpunkt(final long zeitDauer, final long ende) {
		String re = "";

		final long zeitEnde = ende;
		final long zeitAnfang = zeitEnde - zeitDauer;

		int jahreWert = 0;
		int monateWert = 0;
		int tageWert = 0;
		int stundenWert = 0;
		int minutenWert = 0;
		int sekundenWert = 0;
		int tausendstelWert = 0;

		final GregorianCalendar gcEnde = new GregorianCalendar();
		gcEnde.setTimeInMillis(zeitEnde);

		final GregorianCalendar gcAnfangWirdzuEnde = new GregorianCalendar();
		gcAnfangWirdzuEnde.setTimeInMillis(zeitAnfang);

		int zaehlerJahre = 0;
		boolean jahre = true;

		while (jahre) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.YEAR, zaehlerJahre + 1);

			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerJahre++;

			} else {
				jahre = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.YEAR, zaehlerJahre);

		int zaehlerMonate = 0;
		boolean monate = true;
		while (monate) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MONTH, zaehlerMonate + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerMonate++;

			} else {
				monate = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MONTH, zaehlerMonate);

		int zaehlerTage = 0;
		boolean tage = true;
		while (tage) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.DAY_OF_MONTH, zaehlerTage + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerTage++;

			} else {
				tage = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.DAY_OF_MONTH, zaehlerTage);

		int zaehlerStunden = 0;
		boolean stunden = true;
		while (stunden) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.HOUR_OF_DAY, zaehlerStunden + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerStunden++;
			} else {
				stunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.HOUR_OF_DAY, zaehlerStunden);

		int zaehlerMinuten = 0;
		boolean minuten = true;
		while (minuten) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MINUTE, zaehlerMinuten + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerMinuten++;
			} else {
				minuten = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MINUTE, zaehlerMinuten);

		int zaehlerSekunden = 0;
		boolean sekunden = true;
		while (sekunden) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.SECOND, zaehlerSekunden + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerSekunden++;
			} else {
				sekunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.SECOND, zaehlerSekunden);

		int zaehlerTausendstel = 0;
		boolean millisekunden = true;
		while (millisekunden) {
			final GregorianCalendar tempGcAnfang = (GregorianCalendar) gcAnfangWirdzuEnde
					.clone();
			tempGcAnfang.add(Calendar.MILLISECOND, zaehlerTausendstel + 1);
			if (tempGcAnfang.getTimeInMillis() <= gcEnde.getTimeInMillis()) {
				zaehlerTausendstel++;
			} else {
				millisekunden = false;
			}
		}
		gcAnfangWirdzuEnde.add(Calendar.MILLISECOND, zaehlerTausendstel);

		jahreWert = zaehlerJahre;
		monateWert = zaehlerMonate;
		tageWert = zaehlerTage;
		stundenWert = zaehlerStunden;
		minutenWert = zaehlerMinuten;
		sekundenWert = zaehlerSekunden;
		tausendstelWert = zaehlerTausendstel;

		re = baueZeitString(jahreWert, monateWert, tageWert, stundenWert,
				minutenWert, sekundenWert, tausendstelWert);
		return re;
	}

	/**
	 * Normalisiert die Zeitdauer in Bezug auf den Anfangszeitpunkt an Hand des
	 * {@link GregorianCalendar}. So werden z.B. auf 61 Sekunden 1 Minute und 2
	 * Sekunden.
	 * 
	 * @param textFeld
	 *            Inhalt der Textfeldes, welches normalisiert werden soll.
	 * @return Normalisierter Inhalt fr das Textfeld.s
	 */
	String normalisieren(final String textFeld) {
		final String re = setZeitdauerInBezugAufAnfangszeitpunkt(
				ausgangsZeitpunkt,
				getZeitdauerInBezugAufAnfangszeitpunkt(textFeld));
		return re;
	}

	/**
	 * Baut einen Zeitstring fr das Textfeld zusammen. Die Parameter mssen >=
	 * 0 sein, bei < 0 wird das entsprechende Feld nicht dargestellt.
	 * 
	 * @param jahre
	 *            Anzahl der Jahre.
	 * @param monate
	 *            Anzahl der Monate
	 * @param tage
	 *            Anzahl der Tage
	 * @param stunden
	 *            Anzahl der Stunden
	 * @param minuten
	 *            Anzahl der Minuten
	 * @param sekunden
	 *            Anzahl der Sekunden
	 * @param milliSecs
	 *            Anzahl der Tausendstel
	 * @return String fr das Textfeld.
	 */
	String baueZeitString(final int jahre, final int monate, final int tage,
			final int stunden, final int minuten, final int sekunden,
			final int milliSecs) {
		final StringBuilder result = new StringBuilder();

		if ((darzustellendeElemente.contains(ZeitKlasse.JAHR) || (jahre > 0))
				&& (jahre > -1)) {
			result.append(ZeitKlasse.JAHR.format(jahre, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.MONAT) || (monate > 0))
				&& (monate > -1)) {
			result.append(ZeitKlasse.MONAT.format(monate, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.TAG) || (tage > 0))
				&& (tage > -1)) {
			result.append(ZeitKlasse.TAG.format(tage, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.STUNDEN) || (stunden > 0))
				&& (stunden > -1)) {
			result.append(ZeitKlasse.STUNDEN.format(stunden, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.MINUTE) || (minuten > 0))
				&& (minuten > -1)) {
			result.append(ZeitKlasse.MINUTE.format(minuten, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.SEKUNDE) || (sekunden > 0))
				&& (sekunden > -1)) {
			result.append(ZeitKlasse.SEKUNDE.format(sekunden, kurzDarstellung));
			result.append(' ');
		}

		if ((darzustellendeElemente.contains(ZeitKlasse.MILLSEKUNDE) || (milliSecs > 0))
				&& (milliSecs > -1)) {
			result.append(ZeitKlasse.MILLSEKUNDE.format(milliSecs,
					kurzDarstellung));
			result.append(' ');

		}

		if (result.length() > 0) {
			return result.toString().substring(0, result.length() - 1);
		}

		return result.toString();
	}

	private int getFeldWert(final ZeitKlasse klasse, final String textFeld) {
		int re = 0;
		final ValidierungsInfo validiereFelder = validiereFelder(textFeld);
		for (final FeldInfo feldInfo : validiereFelder.getFeldInfos()) {
			if (feldInfo.getTyp() == klasse) {
				re = feldInfo.getWert();
				break;
			}
		}
		return re;
	}

	/**
	 * Liefert die Anzahl der Jahre in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Jahre.
	 */
	int getJahre(final String textFeld) {
		return getFeldWert(ZeitKlasse.JAHR, textFeld);
	}

	String setJahre(final String textFeld, final int jahre) {
		String re = "";

		final int monate = getMonate(textFeld);
		final int tage = getTage(textFeld);
		final int stunden = getStunden(textFeld);
		final int minuten = getMinuten(textFeld);
		final int sekunden = getSekunden(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Monate in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Monate.
	 */
	int getMonate(final String textFeld) {
		return getFeldWert(ZeitKlasse.MONAT, textFeld);
	}

	String setMonate(final String textFeld, final int monate) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int tage = getTage(textFeld);
		final int stunden = getStunden(textFeld);
		final int minuten = getMinuten(textFeld);
		final int sekunden = getSekunden(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Tage in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Tage.
	 */
	int getTage(final String textFeld) {
		return getFeldWert(ZeitKlasse.TAG, textFeld);
	}

	String setTage(final String textFeld, final int tage) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int monate = getMonate(textFeld);
		final int stunden = getStunden(textFeld);
		final int minuten = getMinuten(textFeld);
		final int sekunden = getSekunden(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Stunden in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Stunden.
	 */
	int getStunden(final String textFeld) {
		return getFeldWert(ZeitKlasse.STUNDEN, textFeld);
	}

	String setStunden(final String textFeld, final int stunden) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int monate = getMonate(textFeld);
		final int tage = getTage(textFeld);
		final int minuten = getMinuten(textFeld);
		final int sekunden = getSekunden(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Minuten in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Minuten.
	 */
	int getMinuten(final String textFeld) {
		return getFeldWert(ZeitKlasse.MINUTE, textFeld);
	}

	String setMinuten(final String textFeld, final int minuten) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int monate = getMonate(textFeld);
		final int tage = getTage(textFeld);
		final int stunden = getStunden(textFeld);
		final int sekunden = getSekunden(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Sekunden in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Sekunden.
	 */
	int getSekunden(final String textFeld) {
		return getFeldWert(ZeitKlasse.SEKUNDE, textFeld);
	}

	String setSekunden(final String textFeld, final int sekunden) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int monate = getMonate(textFeld);
		final int tage = getTage(textFeld);
		final int stunden = getStunden(textFeld);
		final int minuten = getMinuten(textFeld);
		final int milliSecs = getTausendstel(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Liefert die Anzahl der Tausendstel in einem String.
	 * 
	 * @param textFeld
	 *            String
	 * @return Anzahl der Tausendstel.
	 */
	int getTausendstel(final String textFeld) {
		return getFeldWert(ZeitKlasse.MILLSEKUNDE, textFeld);
	}

	String setTausendstel(final String textFeld, final int milliSecs) {
		String re = "";

		final int jahre = getJahre(textFeld);
		final int monate = getMonate(textFeld);
		final int tage = getTage(textFeld);
		final int stunden = getStunden(textFeld);
		final int minuten = getMinuten(textFeld);
		final int sekunden = getSekunden(textFeld);

		re = baueZeitString(jahre, monate, tage, stunden, minuten, sekunden,
				milliSecs);

		return re;
	}

	/**
	 * Bestimmt die {@link ZeitDauerGranularitaet} an Hand des Textfeldes.
	 * 
	 * @param textFeld
	 *            der die Zeitdauer beschreibende Text
	 * @return {@link ZeitDauerGranularitaet}
	 */
	ZeitDauerGranularitaet getGranularitaet(final String textFeld) {
		ZeitDauerGranularitaet re = null;

		final ValidierungsInfo validiereFelder = validiereFelder(textFeld);

		if (validiereFelder.getFeldInfos() != null) {
			for (final FeldInfo feldInfo : validiereFelder.getFeldInfos()) {
				if (re != null) {
					break;
				}
				switch (feldInfo.getTyp()) {
				case JAHR:
					re = ZeitDauerGranularitaet.jahr;
					break;
				case MILLSEKUNDE:
					re = ZeitDauerGranularitaet.tausendstel;
					break;
				case MINUTE:
					re = ZeitDauerGranularitaet.minute;
					break;
				case MONAT:
					re = ZeitDauerGranularitaet.monat;
					break;
				case SEKUNDE:
					re = ZeitDauerGranularitaet.sekunde;
					break;
				case STUNDEN:
					re = ZeitDauerGranularitaet.stunde;
					break;
				case TAG:
					re = ZeitDauerGranularitaet.tag;
					break;
				default:
					break;
				}
			}
		}

		if (re == null) {
			re = ZeitDauerGranularitaet.jahr;
		}
		return re;
	}

	void setZeitElementDarstellung(final ZeitKlasse klasse,
			final boolean darstellen) {
		if (darstellen) {
			darzustellendeElemente.add(klasse);
		} else {
			darzustellendeElemente.remove(klasse);
		}
	}
}
