/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse {@link EintragRaeumlicheGueltigkeit} reprsentiert die Datenstze
 * zur rumlichen Gltigkeit, die in der Liste der rumlichen Gltigkeiten
 * dargestellt werden.
 * 
 * Implementiert das Interface {@link Cloneable}.
 * 
 * TODO Warum?
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: EintragRaeumlicheGueltigkeit.java 47183 2013-11-18 09:25:32Z peuker $
 */
class EintragRaeumlicheGueltigkeit implements Cloneable {

	/** Name. */
	private String name;

	/** Info. */
	private String info;

	/** SystemObject. */
	private SystemObject systemObjekt;

	/**
	 * Konstruktor zur Erezugung eines Datensatzes der rumlichen Gltigkeit.
	 * 
	 * @param name
	 *            Name der rumlichen Gltigkeit.
	 * @param info
	 *            Info zur rumlichen Gltigkeit.
	 * @param systemObjekt
	 *            das Objekt fr das die Gltigkeit bestimmt wedren soll
	 */
	public EintragRaeumlicheGueltigkeit(final String name, final String info,
			final SystemObject systemObjekt) {
		this.name = name;
		this.info = info;
		this.systemObjekt = systemObjekt;
	}

	/**
	 * Liefert den Namen der rumlichen Gltigkeit.
	 * 
	 * @return Name der rumlichen Gltigkeit.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setzt den Namen der rumlichen Gltigkeit.
	 * 
	 * @param name
	 *            Name der rumlichen Gltigkeit.
	 */
	public void setName(final String name) {
		this.name = name;
	}

	/**
	 * Liefert die Info zur rumlichen Gltigkeit.
	 * 
	 * @return Info zur rumlichen Gltigkeit.
	 */
	public String getInfo() {
		return info;
	}

	/**
	 * Setzt die Info zur rumlichen Gltigkeit.
	 * 
	 * @param info
	 *            Info zur rumlichen Gltigkeit.
	 */
	public void setInfo(final String info) {
		this.info = info;
	}

	/**
	 * Liefert einen Clone der Instanz dieser Klasse.
	 * 
	 * @return Clone der Instanz dieser Klasse.
	 */
	@Override
	public EintragRaeumlicheGueltigkeit clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}

		final EintragRaeumlicheGueltigkeit raeumlicheGueltigkeit = new EintragRaeumlicheGueltigkeit(
				name, info, systemObjekt);
		return raeumlicheGueltigkeit;
	}

	public SystemObject getSystemObjekt() {
		return systemObjekt;
	}

	public void setSystemObjekt(final SystemObject systemObjekt) {
		this.systemObjekt = systemObjekt;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((info == null) ? 0 : info.hashCode());
		result = (prime * result) + ((name == null) ? 0 : name.hashCode());
		result = (prime * result)
				+ ((systemObjekt == null) ? 0 : systemObjekt.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {

		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (!(obj instanceof EintragRaeumlicheGueltigkeit)) {
			return false;
		}

		boolean flagge = false;
		final EintragRaeumlicheGueltigkeit eintrag = (EintragRaeumlicheGueltigkeit) obj;
		if (systemObjekt.equals(eintrag.getSystemObjekt())) {
			flagge = true;
		}
		return flagge;
	}
}
