/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

/**
 * Die Klasse {@link EintragVerkehrlicheGueltigkeit} reprsentiert die
 * Datenstze zur verkehrlichen Gltigkeit.
 * 
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: EintragVerkehrlicheGueltigkeit.java 46953 2013-11-05 10:05:57Z peuker $
 */
class EintragVerkehrlicheGueltigkeit implements Cloneable {
	/**
	 * Enumerator zur Definition des Bezugszeitpunktes "von". Dient der
	 * relativen Angabe der zeitlichen Gltigkeit.
	 * 
	 * @author HofmannJ
	 */
	public enum _Bezugszeitpunkt {
		/**
		 * Die Relation gibt einen Zeitpunkt vor dem Anfang des Zeitintervalls
		 * an.
		 */
		vorAnfang,

		/**
		 * Die Relation gibt einen Zeitpunkt nach dem Anfang des Zeitintervalls
		 * an.
		 */
		nachAnfang,

		/**
		 * Die Relation gibt einen Zeitpunkt vor dem Ende des Zeitintevalls an.
		 */
		vorEnde,

		/**
		 * Die Relation gibt einen Zeitpunkt nach dem Ende des Zeitintervalls
		 * an.
		 */
		nachEnde
	}

	/** Startzeit der verkehrlichenGltigkeit. */
	private long von;

	/** Von-Bezug. */
	private _Bezugszeitpunkt vonBezug;

	/** Endzeit der verkehrlichenGltigkeit. */
	private long bis;

	/** Bis-Bezug. */
	private _Bezugszeitpunkt bisBezug;

	/**
	 * Konstruktor fr einen Datensatz zur verkehrlichen Gltigkeit.
	 * 
	 * @param von
	 *            Startzeitpunkt in Millisekunden seit dem 1.1.1970.
	 * @param vonBezug
	 *            relative Angabe zur zeitlichen Gltigkeit.
	 * @param bis
	 *            Endzeitpunkt in Millisekunden sei dem 1.1.1970.
	 * @param bisBezug
	 *            relative Angabe zur zeitlichen Gltigkeit.
	 */
	public EintragVerkehrlicheGueltigkeit(final long von,
			final _Bezugszeitpunkt vonBezug, final long bis,
			final _Bezugszeitpunkt bisBezug) {
		this.von = von;
		this.vonBezug = vonBezug;
		this.bis = bis;
		this.bisBezug = bisBezug;
	}

	/**
	 * Liefert den Bezugszeitpunkt von.
	 * 
	 * @return Bezugszeitpunkt von.
	 */
	public long getVon() {
		return von;
	}

	/**
	 * Setzt den Bezugszeitpunkt von.
	 * 
	 * @param von
	 *            Zu setzender Bezugszeitpunkt von.
	 */
	public void setVon(final long von) {
		this.von = von;
	}

	/**
	 * Liefert die relative Angabe zur zeitlichen Gltigkeit von.
	 * 
	 * @return Relative Angabe zur zeitlichen Gltigkeit von.
	 */
	public _Bezugszeitpunkt getVonBezugszeitpunkt() {
		return vonBezug;
	}

	/**
	 * Setzt die relative Angabe zur zeitlichen Gltigkeit von.
	 * 
	 * @param vonBezugszeitpunkt
	 *            Zu setzende relative Angabe zur zeitlichen Gltigkeit von.
	 */
	public void setVonBezugszeitpunkt(final _Bezugszeitpunkt vonBezugszeitpunkt) {
		vonBezug = vonBezugszeitpunkt;
	}

	/**
	 * Liefert den Bezugszeitpunkt bis.
	 * 
	 * @return Bezugszeitpunkt bis.
	 */
	public long getBis() {
		return bis;
	}

	/**
	 * Setzt den Bezugszeitpunkt bis.
	 * 
	 * @param bis
	 *            Zu setzender Bezugszeitpunkt bis.
	 */
	public void setBis(final long bis) {
		this.bis = bis;
	}

	/**
	 * Liefert die relative Angabe zur zeitlichen Gltigkeit bis.
	 * 
	 * @return Relative Angabe zur zeitlichen Gltigkeit bis.
	 */
	public _Bezugszeitpunkt getBisBezugszeitpunkt() {
		return bisBezug;
	}

	/**
	 * Setzt die relative Angabe zur zeitlichen Gltigkeit bis.
	 * 
	 * @param bisBezugszeitpunkt
	 *            Zu setzende relative Angabe zur zeitlichen Gltigkeit bis.
	 */
	public void setBisBezugszeitpunkt(final _Bezugszeitpunkt bisBezugszeitpunkt) {
		bisBezug = bisBezugszeitpunkt;
	}

	/**
	 * Liefert einen Clone der Instanz dieser Klasse.
	 * 
	 * @return Clone der Instanz dieser Klasse.
	 */
	@Override
	public EintragVerkehrlicheGueltigkeit clone() {
		try {
			super.clone();
		} catch (final CloneNotSupportedException e) {
			throw new IllegalStateException(e);
		}

		final EintragVerkehrlicheGueltigkeit verkehrlicheGueltigkeit = new EintragVerkehrlicheGueltigkeit(
				von, vonBezug, bis, bisBezug);
		return verkehrlicheGueltigkeit;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + (int) (bis ^ (bis >>> 32));
		result = (prime * result)
				+ ((bisBezug == null) ? 0 : bisBezug.hashCode());
		result = (prime * result) + (int) (von ^ (von >>> 32));
		result = (prime * result)
				+ ((vonBezug == null) ? 0 : vonBezug.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {

		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (!(obj instanceof EintragVerkehrlicheGueltigkeit)) {
			return false;
		}

		final EintragVerkehrlicheGueltigkeit eintrag = (EintragVerkehrlicheGueltigkeit) obj;

		boolean flagge = false;

		if ((von == eintrag.getVon())
				&& (vonBezug == eintrag.getVonBezugszeitpunkt())
				&& (bis == eintrag.getBis())
				&& (bisBezug == eintrag.getBisBezugszeitpunkt())) {
			flagge = true;
		}

		return flagge;
	}
}
