/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.buv.rw.basislib.kalender.IEintragBereich;

/**
 * Kapselt die Bereiche von Ereigniskalendereintrgen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: ErkaBereich.java 45393 2013-08-07 21:32:57Z peuker $
 */
public class ErkaBereich implements IEintragBereich {

	/** Speichert den Startzeitpunkt. */
	private final Long von;

	/** Speichert den Endzeitpunkt. */
	private final Long bis;

	/** Formatierer fr das Datum. */
	private final SimpleDateFormat dateFormat = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm:ss,SSS");

	ErkaBereich(final Long von, final Long bis) {
		this.von = von;
		this.bis = bis;
	}

	@Override
	public Long getBis() {
		return bis;
	}

	@Override
	public String getBisLesbar() {
		return dateFormat.format(new Date(bis));
	}

	@Override
	public Long getVon() {
		return von;
	}

	@Override
	public String getVonLesbar() {
		return dateFormat.format(new Date(von));
	}

	@Override
	public boolean istAtomar() {
		return von.equals(bis);
	}
}
