/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.basislib.kalender.internal;

/**
 * Informationen zu einem Feld fr Kalendereintrge.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: FeldInfo.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class FeldInfo {

	/** Wert des Feldes. */
	private int wert = -1;

	/** Zeigt an, dass die Kurzform verwendet werden soll. */
	private final boolean kurzform;

	/** Typ des Feldes. Wird an Hand der {@link ZeitKlasse} festgelegt. */
	private final ZeitKlasse typ;

	/**
	 * Konstruktor.
	 * 
	 * @param typ
	 *            der Informationstyp
	 * @param kurzform
	 *            Kurzform verwenden?
	 * @param wert
	 *            der initiale Wert
	 */
	public FeldInfo(final ZeitKlasse typ, final boolean kurzform, final int wert) {
		this.typ = typ;
		this.kurzform = kurzform;
		this.wert = wert;
	}

	/**
	 * Liefert den Wert des Feldes.
	 * 
	 * @return Wert des Feldes
	 */
	public int getWert() {
		return wert;
	}

	/**
	 * Liefert den Typ des Feldes.
	 * 
	 * @return Typ des Feldes als {@link ZeitKlasse}
	 */
	public ZeitKlasse getTyp() {
		return typ;
	}

	/**
	 * Passt den Wert des Feldes in der Weise an, dass Min und Max Werte der
	 * {@link ZeitKlasse} nicht unter- oder berschritten werden.
	 */
	public void passeWertAn() {
		if (wert < typ.getBeschraenkungMin()) {
			wert = typ.getBeschraenkungMin();
		} else if (wert > typ.getBeschraenkungMax()) {
			wert = typ.getBeschraenkungMax();
		}
	}

	/**
	 * liefert den Inhalt des feld als Zeichenkette.
	 * 
	 * @return die Zeichenkette
	 */
	public String getFeldDarstellungsString() {
		final StringBuffer buffer = new StringBuffer(100);

		buffer.append(wert);
		buffer.append(' ');

		if (kurzform) {
			buffer.append(typ.getKurzform());
		} else {
			if (wert > 1) {
				buffer.append(typ.getMehrzahl());
			} else {
				buffer.append(typ.getBezeichnung());
			}
		}

		return buffer.toString();
	}

	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer(100);
		buffer.append("===FeldInfo===\n");
		buffer.append(typ.toString());
		buffer.append("\nWert: ");
		buffer.append(wert);
		buffer.append("\n==============\n");
		return buffer.toString();
	}
}
