/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender.internal;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.kalender.IEintragBereich;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * Kapsel fr Systemkalendereintrge.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: SkeKapsel.java 47112 2013-11-12 15:25:14Z peuker $
 */
public class SkeKapsel implements Comparable<SkeKapsel>, IEintragKapsel {

	/** Gekapselter Systemkalendereintrag. */
	private final SystemkalenderEintrag ske;

	/**
	 * Die Zeitbereiche dieses Systemkalendereintrags als Instanzen der Klasse
	 * {@link SkeBereich}.
	 */
	private final List<IEintragBereich> bereiche = new LinkedList<IEintragBereich>();

	/**
	 * Konstruktor, bernimmt einen {@link SystemkalenderEintrag} fr diese
	 * Kapsel.
	 * 
	 * @param ske
	 *            {@link SystemkalenderEintrag}
	 */
	public SkeKapsel(final SystemkalenderEintrag ske) {
		this.ske = ske;
	}

	@Override
	public String getPid() {
		return ske.getPid();
	}

	@Override
	public void addBereich(final Long von, final Long bis) {
		final SkeBereich bereich = new SkeBereich(von, bis);
		bereiche.add(bereich);
	}

	@Override
	public void addBereich(final IEintragBereich bereich) {
		this.bereiche.add(bereich);
	}

	@Override
	public List<IEintragBereich> getBereiche() {
		return bereiche;
	}

	@Override
	public int getBereicheAnzahl() {
		return bereiche.size();
	}

	@Override
	public String toString() {
		return ske.getPid();
	}

	@Override
	public String toStringLong() {
		final StringBuilder re = new StringBuilder();
		if (ske != null) {
			re.append("===");
			re.append(ske.getPid());
			re.append("===\n");

			if (bereiche.isEmpty()) {
				re.append("Keine Bereiche definiert!\n");
			} else {
				int counter = 0;

				final Iterator<IEintragBereich> it = bereiche.iterator();
				while (it.hasNext()) {
					re.append("Bereich ");
					re.append(counter);
					re.append(": ");
					re.append(it.next());
					re.append('\n');
					counter++;
				}

			}

			for (int i = 0; i < (ske.getPid().length() + 6); i++) {
				re.append('=');
			}
			re.append('\n');
		}

		return re.toString();
	}

	@Override
	public int compareTo(final SkeKapsel o) {
		final String pidThis = getPid();
		final String pidOther = o.getPid();
		return pidThis.compareTo(pidOther);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result)
				+ ((getPid() == null) ? 0 : getPid().hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final SkeKapsel other = (SkeKapsel) obj;
		if (getPid() == null) {
			if (other.getPid() != null) {
				return false;
			}
			return true;
		} else if (!getPid().equals(other.getPid())) {
			return false;
		}
		return true;
	}

}
