/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.legende;

import org.eclipse.swt.widgets.Control;

/**
 * Schnittstelle fr eine Legende, die im {@link LegendeWindow} angezeigt werden
 * kann.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: ILegende.java 45393 2013-08-07 21:32:57Z peuker $
 */
public interface ILegende {

	/**
	 * Die Ecke des Workbench Parts in der das Toolfenster ausgerichtet werden
	 * soll.
	 */
	static enum Corner {
		/** Oben links. */
		TopLeft("Oben links"),

		/** Oben rechts. */
		TopRight("Oben rechts"),

		/** Unten links. */
		BottomLeft("Unten links"),

		/** Unten rechts. */
		BottomRight("Unten rechts");

		private final String bezeichnung;

		private Corner(final String bezeichnung) {
			this.bezeichnung = bezeichnung;
		}

		public String getBezeichnung() {
			return bezeichnung;
		}
	}

	/**
	 * Gibt ein Control zurck, an dem das {@link LegendeWindow} ausgericht
	 * wird. Verndert das Control seine Position, dann folgt die Legende dem
	 * Control.
	 * 
	 * <p>
	 * Das Control, welches hier zurckgegeben wird, ist <em>nicht</em> das
	 * Control der Legende, sondern ein Control im, Workbench Part fr das die
	 * Legende angezeigt wird.
	 * 
	 * @return das Control oder <code>null</code>, wenn die Legende nicht folgen
	 *         soll.
	 */
	Control getControl();

	/**
	 * liefert die Ecke des Workbenchparts an dem die anzuzeigende Legende
	 * standardig ausgerichtet werden soll.
	 * 
	 * @return die Ecke
	 */
	Corner getDefaultCorner();

	/**
	 * liefert einen Titel zur Anzeige im Legendenfenster (optional).
	 * 
	 * @return den Titel oder <code>null</code>
	 */
	String getTitel();
}
