/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.legende;

import java.util.List;

import org.eclipse.swt.graphics.Image;

/**
 * Der Legendenbaustein stellt einen Teilbaum der Legende dar.
 * 
 * <p>
 * Die Legende wird rekursiv durch verschachtelung von {@link ILegendeBaustein}
 * -Objekten aufgebaut. Ist die Liste die {@link #getBausteine()} zurckgibt
 * leer, ist dieser Baustein ein Blatt im Baum, ist die Liste nicht leer ist der
 * Baustein ein innerer Knoten.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: ILegendeBaustein.java 45393 2013-08-07 21:32:57Z peuker $
 */
public interface ILegendeBaustein {

	/**
	 * Gibt die untergeordneten Legendenbausteine zurck. Gibt es keine, wird
	 * eine leere Liste zurckgegeben.
	 * 
	 * @return die Liste der untergeordneten Bausteine
	 */
	List<ILegendeBaustein> getBausteine();

	/**
	 * Gibt das kleines Bild des zu beschreibenden Objekts zurck.
	 * 
	 * @return das Image fr die Darstellung des Elements oder <code>null</code>
	 */
	Image getIcon();

	/**
	 * Gibt eine kurze Objektbeschreibung zurck.
	 * 
	 * @return die Beschreibung des Elements zur Darstellung im Legendenbaum
	 */
	String getText();

}
