/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 */

package de.bsvrz.buv.rw.basislib.legende;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

/**
 * Basisimplementierung der Schnittstelle.
 * 
 * <p>
 * Der Liste die {@link #getBausteine()} zurckgibt, knnen die anzuzeigenden
 * Legendenbausteine hinzugefgt werden.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: LegendeBaustein.java 45393 2013-08-07 21:32:57Z peuker $
 */
public class LegendeBaustein implements ILegendeBaustein {

	private final Image icon;
	private final String text;
	private List<ILegendeBaustein> bausteine;

	/**
	 * Initialisiert den Legendenbaustein mit seinem Icon und dem beschreibenden
	 * Text.
	 * 
	 * @param icon
	 *            das Image zur Darstellung des Bausteins
	 * @param text
	 *            der fr den Baustein anzuzeigende Text
	 */
	public LegendeBaustein(final Image icon, final String text) {
		this.icon = icon;
		this.text = text;
	}

	@Override
	public Image getIcon() {
		return icon;
	}

	@Override
	public String getText() {
		return text;
	}

	@Override
	public List<ILegendeBaustein> getBausteine() {
		if (bausteine == null) {
			bausteine = new ArrayList<ILegendeBaustein>();
		}

		return bausteine;
	}

}
