/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Erweitert die Klasse {@link DataTableObject} um eine Liste von {@link Zelle}n
 * vom Typ Plain sowie den zugehrigen Getter- und Settermethoden.
 * 
 * @author HofmannJ
 * 
 */
public class DataTableObjectErweitert extends DataTableObject {
	/**
	 * Wurzel-{@link Zelle}.
	 */
	private Zelle wurzelZelle;

	/**
	 * Plain-{@link Zelle}n.
	 */
	private LinkedList<Zelle> plainZellenDataListe = new LinkedList<Zelle>();

	/**
	 * Tiefste Ebene Data.
	 */
	private int tiefsteEbeneData;

	/**
	 * Start Offset der Zeilen in der Tabellendarstellung.
	 */
	private int startOffsetZeile;

	/**
	 * Flag das anzeigt, ob es sich um eine gerade oder um eine ungerade
	 * {@link DataTableObjectErweitert}-Zeile handelt. Hiernach wird bestimmt,
	 * wie die Zeilen einzufrben sind.
	 */
	private boolean geradeZeile;

	/**
	 * Flag das anzeigt, ob dieses {@link DataTableObjectErweitert} in der
	 * nderungsdarstellung angezeigt werden soll oder nicht.
	 */
	private boolean aenderungsDarstellung;

	/**
	 * bernimmt das {@link ResultData} und erzeugt daraus die entsprechende
	 * Struktur.
	 * 
	 * @param resultData
	 */
	public DataTableObjectErweitert(final ResultData resultData) {
		super(resultData);
	}

	/**
	 * Liefert die Wurzel{@link Zelle}.
	 * 
	 * @return Wurzel{@link Zelle}
	 */
	public Zelle getWurzelZelle() {
		return wurzelZelle;
	}

	/**
	 * Setzt die Wurzel{@link Zelle}.
	 * 
	 * @param wurzelZelle
	 *            Wurzel{@link Zelle}
	 */
	public void setWurzelZelle(final Zelle wurzelZelle) {
		this.wurzelZelle = wurzelZelle;
	}

	/**
	 * Liefert die Liste aller {@link Zelle}n vom Typ Plain.
	 * 
	 * @return Liste aller {@link Zelle}n vom Typ Plain
	 */
	public LinkedList<Zelle> getPlainZellen() {
		return plainZellenDataListe;
	}

	/**
	 * Setzt die Liste aller {@link Zelle}n vom Typ Plain.
	 * 
	 * @param plainZellen
	 *            Liste aller {@link Zelle}n vom Typ Plain
	 */
	public void setPlainZellen(final LinkedList<Zelle> plainZellen) {
		this.plainZellenDataListe = plainZellen;
	}

	/**
	 * Liefert die tiefste Ebene.
	 * 
	 * @return Tiefste Ebene.
	 */
	public int getTiefsteEbene() {
		return tiefsteEbeneData;
	}

	/**
	 * Setzt die tiefste Ebene.
	 * 
	 * @param tiefsteEbene
	 *            Tiefste Ebene.
	 */
	public void setTiefsteEbene(final int tiefsteEbene) {
		this.tiefsteEbeneData = tiefsteEbene;
	}

	/**
	 * Setzt das Start-Offset der Zeilen in der tabellarischen Darstellung.
	 * 
	 * @param offset
	 *            Start-Offset.
	 */
	public void setStartOffsetZeile(final int offset) {
		this.startOffsetZeile = offset;
	}

	/**
	 * Liefert das Start-Offset der Zeilen in der tabellarischen Darstellung.
	 * 
	 * @return Start-Offset
	 */
	public int getStartOffsetZeile() {
		return startOffsetZeile;
	}

	/**
	 * Liefert das End-Offset der Zeilen, dies berechnet sich aus dem
	 * Start-Offset + der tiefsten Ebene innerhalb der Zeilen. <br>
	 * <b>Achtung:</b> Die tiefste Ebene muss vorher, whrend der
	 * Initialisierungsphase, gesetzt werden.
	 * 
	 * @return End-Offset der Zeilen.
	 */
	public int getEndOffsetZeile() {
		final int endOffset = (tiefsteEbeneData + startOffsetZeile) - 1;
		return endOffset;
	}

	/**
	 * Prft, ob es sich um eine gerade oder ungerade
	 * {@link DataTableObjectErweitert}-Zeile handelt.
	 * 
	 * @return Liefert <code>true</code>, wenn es sich um eine gerade
	 *         {@link DataTableObjectErweitert}-Zeile handelt, sonst false.
	 */
	public boolean isGeradeZeile() {
		return geradeZeile;
	}

	/**
	 * Setzt diese {@link DataTableObjectErweitert}-Zeile auf gerade oder
	 * ungerade Zeile.
	 * 
	 * @param isGeradeZeile
	 *            Wenn <code>true</code>, dann ist die
	 *            {@link DataTableObjectErweitert}-Zeile gerade, bei
	 *            <code>false</code> ist sie ungerade.
	 */
	public void setGeradeZeile(final boolean isGeradeZeile) {
		this.geradeZeile = isGeradeZeile;
	}

	/**
	 * Liefert die Farbe dieses {@link DataTableObjectErweitert} zurck.
	 * 
	 * @return
	 */
	public Color getColor() {
		final Display display = Display.getDefault();

		Color re = null;
		if (getData() == null) {
			final DataState resultDataState = getDataState();

			if (resultDataState == DataState.NO_DATA) {
				// "keine Daten"
				re = display.getSystemColor(SWT.COLOR_GREEN);
			} else if (resultDataState == DataState.NO_SOURCE) {
				// "keine Daten (keine Quelle)"
				// Orange
				re = new Color(display, 255, 200, 0);
			} else if (resultDataState == DataState.NO_RIGHTS) {
				// "keine Daten (keine Rechte)";
				// rot
				re = display.getSystemColor(SWT.COLOR_RED);
			} else if (resultDataState == DataState.POSSIBLE_GAP) {
				// "keine Daten (potentielle Datenlcke)";
				// magenta
				re = display.getSystemColor(SWT.COLOR_MAGENTA);
			} else if (resultDataState == DataState.END_OF_ARCHIVE) {
				// "Ende des Archivanfragezeitraums";
				// cyan
				re = display.getSystemColor(SWT.COLOR_CYAN);
			} else if (resultDataState == DataState.DELETED_BLOCK) {
				// "Gelschter Bereich";
				// rot
				re = display.getSystemColor(SWT.COLOR_RED);
			} else if (resultDataState == DataState.UNAVAILABLE_BLOCK) {
				// "Ausgelagerter Bereich"
				// Gelb
				re = display.getSystemColor(SWT.COLOR_YELLOW);
			} else if (resultDataState == DataState.INVALID_SUBSCRIPTION) {
				// "keine Daten (fehlerhafte Anmeldung)";
				// rot
				re = display.getSystemColor(SWT.COLOR_RED);
			} else {
				// "keine Daten (Undefinierte Objektkodierung)"
				// rot
				re = display.getSystemColor(SWT.COLOR_RED);
			}
		} else {
			if (!isGeradeZeile()) {
				// Hellgrau
				re = new Color(display, 205, 205, 205);
			}
		}

		return re;
	}

	/**
	 * Liefert den Text dieses {@link DataTableObjectErweitert} zurck, wenn das
	 * Data == null ist.
	 * 
	 * @return Text dieses {@link DataTableObjectErweitert} zurck, wenn das
	 *         Data == null ist, sonst <code>null</code>.
	 */
	public String getText() {
		String re = "";
		if (getData() == null) {
			final DataState resultDataState = getDataState();

			if (resultDataState == DataState.NO_DATA) {
				re = "keine Daten";
			} else if (resultDataState == DataState.NO_SOURCE) {
				re = "keine Daten (keine Quelle)";
			} else if (resultDataState == DataState.NO_RIGHTS) {
				re = "keine Daten (keine Rechte)";
			} else if (resultDataState == DataState.POSSIBLE_GAP) {
				re = "keine Daten (potentielle Datenlcke)";
			} else if (resultDataState == DataState.END_OF_ARCHIVE) {
				re = "Ende des Archivanfragezeitraums";
			} else if (resultDataState == DataState.DELETED_BLOCK) {
				re = "Gelschter Bereich";
			} else if (resultDataState == DataState.UNAVAILABLE_BLOCK) {
				re = "Ausgelagerter Bereich";
			} else if (resultDataState == DataState.INVALID_SUBSCRIPTION) {
				re = "keine Daten (fehlerhafte Anmeldung)";
			} else {
				re = "keine Daten (Undefinierte Objektkodierung)";
			}
		}

		return re;
	}

	/**
	 * Prft ob es sich um ein {@link DataTableObjectErweitert} handelt, welches
	 * in der nderungsdarstellung angezeigt werden soll.
	 * 
	 * @return Liefert <code>true</code>, wenn dieses
	 *         {@link DataTableObjectErweitert} in der nderungsdarstellung
	 *         angezeigt werden soll, sonst <code>false</code>.
	 */
	public boolean isAenderungsDarstellung() {
		return aenderungsDarstellung;
	}

	/**
	 * Setzt, ob dieses {@link DataTableObjectErweitert} in der
	 * nderungsdarstellung dargestellt werden soll oder nicht.
	 * 
	 * @param aenderungsDarstellung
	 *            Ist <code>true</code>, wenn das
	 *            {@link DataTableObjectErweitert} in der nderungsdarstellung
	 *            angezeigt werden soll, sonst <code>false</code>.
	 */
	public void setAenderungsDarstellung(final boolean aenderungsDarstellung) {
		this.aenderungsDarstellung = aenderungsDarstellung;
	}

}
