/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

/**
 * Enumerator zur Definition der Darstellungsart innerhalb der Klasse
 * {@link TabellenModell}.
 * 
 * @author HofmannJ
 * 
 */
public enum EnumDarstellungsArt {
	/**
	 * Zustands Darstellung.
	 */
	zustandsDarstellung("Zustandsdarstellung",
			"Darstellung erfolgt in Zustandsdarstellung."),

	/**
	 * nderungs Darstellung.
	 */
	aenderungsDarstellung("nderungsdarstellung",
			"Darstellung erfolgt in nderungsdarstellung.");

	/**
	 * Speichert die Bezeichnung dieses Enumerators.
	 */
	private String bezeichnung;

	/**
	 * Speichert die Beschreibung dieses Enumerators.
	 */
	private String beschreibung;

	/**
	 * Konstruktor, bernimmt Bezeichnung und Beschreibung dieses Enumerators.
	 * 
	 * @param bezeichnung
	 *            Bezeichnung
	 * @param beschreibung
	 *            Beschreibung
	 */
	private EnumDarstellungsArt(final String bezeichnung,
			final String beschreibung) {
		this.bezeichnung = bezeichnung;
		this.beschreibung = beschreibung;
	}

	/**
	 * Liefert die Bezeichnung dieses Enumerators.
	 * 
	 * @return Bezeichnung
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * Liefert die Beschreibung dieses Enumerators.
	 * 
	 * @return Beschreibung
	 */
	public String getBeschreibung() {
		return beschreibung;
	}

	@Override
	public String toString() {
		return bezeichnung;
	}
}
