/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.basislib.listendarstellung.DataTableObjectErweitert;
import de.bsvrz.buv.rw.basislib.listendarstellung.TabellenModell;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableModel;

/**
 * {@link KTableCellRenderer} fr die Listendarstellung.
 * 
 * @author HofmannJ
 * 
 */
public class CellRenderer implements KTableCellRenderer {

	private final Display display;

	public CellRenderer() {
		display = Display.getCurrent();
	}

	@Override
	public void drawCell(final GC gc, final Rectangle rect, final int col,
			final int row, final Object content, final boolean focus,
			final boolean header, final boolean clicked, final KTableModel model) {

		Color textColor;
		Color backColor;
		Color borderColor;
		// TownExampleContent myContent = (TownExampleContent)content;

		Color zeilenFarbe = null;

		if (model instanceof TabellenModell) {
			final int zeile = row - model.getFixedHeaderRowCount();

			final TabellenModell tabellenModell = (TabellenModell) model;
			final DataTableObjectErweitert dataTableObjectErweitert = tabellenModell
					.getDtoImZeilenBereich(zeile);

			if (dataTableObjectErweitert != null) {
				// if (dataTableObjectErweitert.isGeradeZeile()) {
				// zeilenFarbe = _display.getSystemColor(SWT.COLOR_WHITE);
				// } else {
				// zeilenFarbe = new Color(_display, 255, 255, 190);
				// }

				zeilenFarbe = dataTableObjectErweitert.getColor();
			}
		}

		if (focus) {
			textColor = display.getSystemColor(SWT.COLOR_WHITE);
			backColor = display.getSystemColor(SWT.COLOR_BLUE);
		} else if (clicked) {
			textColor = display.getSystemColor(SWT.COLOR_WHITE);
			backColor = display.getSystemColor(SWT.COLOR_BLUE);
		} else {
			textColor = display.getSystemColor(SWT.COLOR_LIST_FOREGROUND);
			backColor = (display.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
			if (zeilenFarbe != null) {
				backColor = zeilenFarbe;
			}
		}

		borderColor = display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);

		gc.setForeground(borderColor);
		gc.drawLine(rect.x, rect.y + rect.height, rect.x + rect.width, rect.y
				+ rect.height);

		gc.setForeground(borderColor);
		gc.drawLine(rect.x + rect.width, rect.y, rect.x + rect.width, rect.y
				+ rect.height);

		// if (col == 0)
		// {
		// gc.setBackground(m_Display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		// textColor = m_Display.getSystemColor(SWT.COLOR_LIST_FOREGROUND);
		// gc.setForeground(textColor);
		//
		//
		// gc.drawImage((myContent.image),rect.x, rect.y);
		//
		// rect.y += 120;
		// rect.height -= 120;
		// gc.fillRectangle(rect);
		// gc.drawText((myContent.name),rect.x+25, rect.y+2);
		// }
		//
		//
		// else if (col == 1)
		// {
		// gc.setBackground(backColor);
		// gc.setForeground(textColor);
		//
		// gc.fillRectangle(rect);
		//
		// SWTX.drawTextImage(
		// gc,
		// myContent.country,
		// SWTX.ALIGN_HORIZONTAL_LEFT | SWTX.ALIGN_VERTICAL_TOP,
		// null,
		// SWTX.ALIGN_HORIZONTAL_LEFT | SWTX.ALIGN_VERTICAL_CENTER,
		// rect.x+3,
		// rect.y,
		// rect.width-3,
		// rect.height
		// );
		//
		// }
		//
		// else if (col == 2)
		// {
		gc.setBackground(backColor);
		gc.setForeground(textColor);

		gc.fillRectangle(rect);
		final Rectangle save = gc.getClipping();
		gc.setClipping(rect);
		gc.drawText((String) content, rect.x + 3, rect.y);
		gc.setClipping(save);

	}

	@Override
	public int getOptimalWidth(final GC gc, final int col, final int row,
			final Object content, final boolean fixed, final KTableModel model) {

		final int re = Math.max(gc.stringExtent(content.toString()).x + 8, 120);
		return re;
	}

}
