/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.mehrfachzwischenablage;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Liefert die Sicht der Mehrfachzwischenablage.
 * 
 * @author HofmannJ
 * 
 */
public class SichtMehrfachZwischenablage extends ViewPart implements
		IMehrfachZwischenablageEinfuegen {

	/** Der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Eltern-Composite.
	 */
	private Composite elternComposite;

	/**
	 * TableViewer zur Anzeige der Inhalte der Zwischenablage.
	 */
	private TableViewer tabelleObjekte;

	/**
	 * {@link MehrfachZwischenablageDaten#}.
	 */
	private MehrfachZwischenablageDaten zwischenablageDaten;

	@Override
	public void createPartControl(final Composite parent) {

		// this._elternComposite = parent;
		this.elternComposite = new Composite(parent, SWT.NONE);

		erzeugeTabelle();
		// parent.setLayout(new FillLayout());
		elternComposite.setLayout(new FillLayout());

		zwischenablageDaten = MehrfachZwischenablageDaten.getInstanz();

		final IMehrfachZwischenablageDatenZuhoerer datenZuhoerer = new IMehrfachZwischenablageDatenZuhoerer() {

			@Override
			public void eintraegeListeAktualisiert(
					final List<ZwischenablageDatensatz> eintraegeListe) {
				tabelleObjekte.setInput(eintraegeListe);

			}

		};

		zwischenablageDaten.hinzufuegenZuhoerer(datenZuhoerer);

		final List<ZwischenablageDatensatz> liste = zwischenablageDaten
				.getListe();
		if (liste != null) {
			tabelleObjekte.setInput(liste);
		}

		verankereKontextMenu();

		/*
		 * Hilfe Kontext
		 */
		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(elternComposite,
						"de.bsvrz.buv.rw.basislib.mehrfachzwischenablage");
	}

	public void erzeugeTabelle() {
		tabelleObjekte = new TableViewer(elternComposite, SWT.FULL_SELECTION
				| SWT.MULTI);

		// TableColumn tc1 = new TableColumn(_tabelleSystemObjekte.getTable(),
		// SWT.CENTER);
		// tc1.setText("Lfd. Nr.:");
		// tc1.setWidth(50);

		final TableColumn tc2 = new TableColumn(tabelleObjekte.getTable(),
				SWT.LEFT);
		tc2.setText("ObjektTyp:");
		tc2.setWidth(200);

		final TableColumn tc3 = new TableColumn(tabelleObjekte.getTable(),
				SWT.LEFT);
		tc3.setText("Weitere Informationen:");
		tc3.setWidth(300);

		tabelleObjekte.getTable().setHeaderVisible(true);
		tabelleObjekte.getTable().setLinesVisible(true);

		tabelleObjekte.setContentProvider(new TabelleInhaltsVersorger());
		tabelleObjekte.setLabelProvider(new TabelleDarstellungsVersorger());

		tabelleObjekte
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {

						final IStructuredSelection selection = (IStructuredSelection) event
								.getSelection();

						final List<ZwischenablageDatensatz> tempSelektion = new LinkedList<ZwischenablageDatensatz>();

						final Iterator<ZwischenablageDatensatz> it = selection
								.iterator();
						while (it.hasNext()) {
							final ZwischenablageDatensatz object = it.next();
							if (object != null) {
								tempSelektion.add(object);
							}
						}

						zwischenablageDaten
								.setSelektionMehrfachZwischenablageIntern(tempSelektion);
					}

				});

	}

	/**
	 * Verankert das Kontextmen.
	 * 
	 */
	private void verankereKontextMenu() {
		SichtMehrfachZwischenablage.LOGGER.fine(getClass().getName()
				+ "::verankereContextMenu => Verankere Kontextmen");

		final MenuManager menuMgr = new MenuManager(
				"#PopupMenuMehrfachZwischenablage");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {

				SichtMehrfachZwischenablage.LOGGER.fine(getClass().getName()
						+ "::verankereKontextMenu \n\t => menuaboutToShow\n");

				fuelleKontextMenu(manager);
			}
		});

		final Menu menu = menuMgr
				.createContextMenu(tabelleObjekte.getControl());
		tabelleObjekte.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, tabelleObjekte);
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *            Der {@link IMenuManager}, der die Struktur hinter dem
	 *            Kontextmen aufnimmt.
	 */
	private void fuelleKontextMenu(final IMenuManager manager) {
		/**
		 * Liste der Actions, die im Kontextmen dargestellt werden sollen
		 */
		final List<IAction> actions = new LinkedList<IAction>();

		// Action lschen
		final Action actionLoeschen = new Action() {
			@Override
			public void run() {
				SichtMehrfachZwischenablage.LOGGER.fine("AKTION BEREIT");
				aktionSelektiertesElementLoeschen();
				// Object firstElement = ((IStructuredSelection)
				// _tabelleObjekte.getSelection()).getFirstElement();
				// if (firstElement instanceof Element)
				// {
				//
				// }
			}
		};
		actionLoeschen.setText("Lschen");
		actionLoeschen.setToolTipText("Lscht das Element.");
		actionLoeschen.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		actions.add(actionLoeschen);

		// Hinzufgen der Actions zum Menmanager
		final Iterator<IAction> it = actions.iterator();
		int i = -1;
		while (it.hasNext()) {
			i++;
			manager.add(it.next());
			if (i == 2) {
				manager.add(new Separator());
			}
		}
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	/**
	 * Lscht ein Element aus der MehrfachZwischenablage.
	 */
	private void aktionSelektiertesElementLoeschen() {
		// _zwischenablageDaten.entferneEintrag(_zwischenablageDaten.);
		SichtMehrfachZwischenablage.LOGGER.fine("LSCHE");
		zwischenablageDaten.entferneSelektierteEintraege();
	}

	@Override
	public void setFocus() {
		if (elternComposite != null) {
			elternComposite.setFocus();
		}
	}

	private class TabelleInhaltsVersorger implements IStructuredContentProvider {
		@Override
		@SuppressWarnings("unchecked")
		public Object[] getElements(final Object inputElement) {
			Object[] listenObjekte = null;

			if (inputElement instanceof List) {
				final List<Object> tempList = (List<Object>) inputElement;

				listenObjekte = new Object[tempList.size()];

				final Iterator<Object> it = tempList.iterator();

				int i = -1;
				while (it.hasNext()) {
					i++;
					final Object object = it.next();

					if (object != null) {
						SichtMehrfachZwischenablage.LOGGER
								.fine(getClass().getName()
										+ "::getElements \n\t=> I: " + i
										+ " Typ: "
										+ object.getClass().getName() + "\n");

						listenObjekte[i] = object;
					} else {
						SichtMehrfachZwischenablage.LOGGER.fine(getClass()
								.getName()
								+ "::getElements \n\t=> I: "
								+ i
								+ " <= NULL Object\n");
						listenObjekte[i] = "NULL Objekt";
					}

				}

			}

			return listenObjekte;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird global gehandelt?
		}

	}

	private static class TabelleDarstellungsVersorger implements
			ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object object, final int columnIndex) {
			String returnString = "";
			ZwischenablageDatensatz zwischenablageDatensatz = null;

			// ID
			if (columnIndex == 0) {

				if (object instanceof ZwischenablageDatensatz) {
					zwischenablageDatensatz = (ZwischenablageDatensatz) object;
				}

				String tempString = "";
				if (zwischenablageDatensatz != null) {
					tempString = zwischenablageDatensatz.getWert().getClass()
							.getName();
				} else {
					tempString = object.getClass().getName();
				}

				if (tempString.length() > 20) {
					final int first = tempString.indexOf('.');
					returnString = returnString
							+ tempString.substring(0, first) + "..";
					final int last = tempString.lastIndexOf('.');
					returnString = returnString + tempString.substring(last);
				} else {
					returnString = tempString;
				}

			} else if (columnIndex == 1) {
				// Bezeichnung
				if (object instanceof ZwischenablageDatensatz) {
					zwischenablageDatensatz = (ZwischenablageDatensatz) object;
					returnString = zwischenablageDatensatz.getWert().toString();
				} else {
					returnString = object.toString();
				}
			} else {
				returnString = "Fr diese Spalte findet keine Versorgung statt.";
			}

			return returnString;
		}

		@Override
		public void addListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}

		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}
	}

	@Override
	public void einfuegen(final List<Object> objekte) {
		final LinkedList<ZwischenablageDatensatz> tempListe = new LinkedList<ZwischenablageDatensatz>();
		final Iterator<Object> it = objekte.iterator();
		while (it.hasNext()) {
			final Object object = it.next();
			tempListe.add(new ZwischenablageDatensatz(object));
			if (object instanceof String) {
				SichtMehrfachZwischenablage.LOGGER.fine("STRING: "
						+ (String) object);
			}
		}

		zwischenablageDaten.hinzufuegenEintraege(tempListe);
	}

}
