/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.List;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

public class ObjektzusammenstellungFallAdapter extends ViewerDropAdapter {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Liste der SystemObjekte.
	 */
	private final List<SystemObject> listeObjekte;

	/**
	 * Ziel.
	 */
	private Object ziel;

	/**
	 * Syso Flag.
	 */
	private boolean zielIstSystemObjekt = true;

	private SpeicherTransfer speicherTransfer;

	/**
	 * Initialisierung des FallAdapters.
	 * 
	 * @param viewer
	 *            Viewer, auf den sich der Falladapter bezieht.
	 * @param listeObjekte
	 *            Liste der Systemobjekte, mit denen der Viewer gefttert wird.
	 */
	protected ObjektzusammenstellungFallAdapter(final Viewer viewer,
			final List<SystemObject> listeObjekte) {
		super(viewer);
		this.listeObjekte = listeObjekte;
	}

	@Override
	public boolean performDrop(final Object data) {
		ObjektzusammenstellungFallAdapter.LOGGER.fine(getClass().getName()
				+ "::performDrop \n\t=>TYP: " + data.getClass() + "\n");

		if (data instanceof String) {
			final String tempString = (String) data;

			// Prfen auf [PID] am Anfang des Strings:
			if (tempString.length() > 5) {
				final String vergleich = tempString.substring(0, 5);

				if ("[PID]".equals(vergleich)) {

					final String tempPid = tempString.substring(5);
					ObjektzusammenstellungFallAdapter.LOGGER.fine("tempPid: "
							+ tempPid);

					speicherTransfer = SpeicherTransfer.getInstanz();
					if (speicherTransfer.getPid() != null) {
						if (speicherTransfer.getPid().equals(tempPid)) {
							ObjektzusammenstellungFallAdapter.LOGGER
									.fine(getClass().getName()
											+ "::performDrop \n\t=>Target: "
											+ ziel.getClass().getName() + "\n");

							final int neuePosition = listeObjekte.indexOf(ziel);
							speicherTransfer.setNeuePosition(neuePosition);
							final int altePosition = speicherTransfer
									.getAltePosition();

							ObjektzusammenstellungFallAdapter.LOGGER
									.fine(getClass().getName()
											+ "::performDrop \n\t=> POS - ALT: "
											+ altePosition + " NEU: "
											+ neuePosition + "\n");

							if ((neuePosition > -1) && (altePosition > -1)) {
								listeObjekte.remove(altePosition);

								if (neuePosition < altePosition) {
									ObjektzusammenstellungFallAdapter.LOGGER
											.fine(getClass().getName()
													+ "::performDrop \n\t=> neuePosition < altePosition");

									listeObjekte.add(neuePosition,
											speicherTransfer.getSystemObjekt());
								} else if (neuePosition > altePosition) {
									ObjektzusammenstellungFallAdapter.LOGGER
											.fine(getClass().getName()
													+ "::performDrop \n\t=> neuePosition > altePosition");

									listeObjekte.add(neuePosition,
											speicherTransfer.getSystemObjekt());
								}
							} else if ((altePosition == -1)
									&& (neuePosition > -1)) {
								ObjektzusammenstellungFallAdapter.LOGGER
										.fine(getClass().getName()
												+ "::PerformDrop \n\t=> (altePosition == -1 ) && (neuePosition > -1)");

								listeObjekte.add(neuePosition,
										speicherTransfer.getSystemObjekt());
							} else if (neuePosition < 0) {
								ObjektzusammenstellungFallAdapter.LOGGER
										.fine(getClass().getName()
												+ "::PerformDrop \n\t=> neuePosition < 0");

								listeObjekte.add(speicherTransfer
										.getSystemObjekt());
							} else if (!zielIstSystemObjekt) {
								ObjektzusammenstellungFallAdapter.LOGGER
										.fine(getClass().getName()
												+ "::target \n\t=> Ziel ist kein SystemObjekt. Anhngen an Tabelle.");

								if (altePosition > -1) {
									listeObjekte.remove(altePosition);
								}
								listeObjekte.add(speicherTransfer
										.getSystemObjekt());
							} else {
								ObjektzusammenstellungFallAdapter.LOGGER
										.fine(getClass().getName()
												+ "::target \n\t=> Sonst...");

								listeObjekte.add(speicherTransfer
										.getSystemObjekt());
							}

							// _listeObjekte.add(speicher.getSystemObject());
							speicherTransfer.zuruecksetzenSpeicher();

							getViewer().setInput(listeObjekte);
						} else {
							ObjektzusammenstellungFallAdapter.LOGGER
									.info(getClass().getName()
											+ "::performDrop \n\t=> Die Pid des in "
											+ speicherTransfer.getClass()
													.getName()
											+ "hinterlegten SystemObjekt(PID="
											+ speicherTransfer.getPid()
											+ ") entspricht nicht der Pid des zu droppenden SystemObjekts(PID="
											+ tempPid
											+ "). Drop Abgebrochen.\n");
						}

					} else {
						ObjektzusammenstellungFallAdapter.LOGGER
								.info(getClass().getName()
										+ "::performDrop \n\t=> In "
										+ speicherTransfer.getClass().getName()
										+ " ist KEIN SystemObjekt hinterlegt. Drop Abgebrochen.\n");
					}

				} else {
					ObjektzusammenstellungFallAdapter.LOGGER.info(getClass()
							.getName()
							+ "::performDrop \n\t=> KEIN SYSTEMOBJEKT\n");
				}
			}

		}

		return false;
	}

	@Override
	public boolean validateDrop(final Object target, final int operation,
			final TransferData transferType) {

		this.ziel = target;

		if (target instanceof SystemObject) {
			ObjektzusammenstellungFallAdapter.LOGGER.fine("SYSO");
			zielIstSystemObjekt = true;
			return true;
		}
		ObjektzusammenstellungFallAdapter.LOGGER.fine("KEINSYSO");
		zielIstSystemObjekt = false;
		return true;

	}

}
