/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.simulationsvariante;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.sys.funclib.debug.Debug;

public final class Simulationsvariante {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Speichert die Singleton-Instanz dieser Klasse.
	 */
	private static Simulationsvariante instanz;

	/**
	 * Speichert die aktuell eingestellte Simulationsvariante. Default-Wert ==
	 * 0.
	 */
	private int simulationsVariante;

	/**
	 * Diese Liste nimmt die Zuhrer auf, die sich ber die aktuelle
	 * Simulationsvariante informieren mchten.
	 */
	private final List<IZuhoererSimulationsvariante> zuhoererListe = new LinkedList<IZuhoererSimulationsvariante>();

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Sinleton-Instanz dieser Klasse
	 */
	public static Simulationsvariante getInstanz() {
		if (Simulationsvariante.instanz == null) {
			Simulationsvariante.instanz = new Simulationsvariante();
		}
		return Simulationsvariante.instanz;
	}

	/**
	 * Setzt die aktuelle Simulationsvariante.
	 * 
	 * @param i
	 *            Zu setzende Simulationsvariante.
	 * @throws IllegalArgumentException
	 *             Wird geworfen, wenn die bergebene Simulationsvariante <0
	 *             bzw. >999 ist.
	 */
	public void setSimulationsvariante(final int i) {
		if ((i >= 0) && (i <= 999)) {
			this.simulationsVariante = i;
			benachrichtigeZuhoerer();
		} else {
			throw new IllegalArgumentException(
					"Es kann keine Simulationsvariante < 0 bzw. > 999. Abbruch.");
		}
	}

	public int getSimulationsvariante() {
		return simulationsVariante;
	}

	/**
	 * Fgt einen {@link IZuhoererSimulationsvariante} zur Liste der Zuhrer
	 * hinzu.
	 * 
	 * @param zuhoerer
	 *            Hinzuzufgender {@link IZuhoererSimulationsvariante}.
	 */
	public void hinzufuegenZuhoererSimulationsvariante(
			final IZuhoererSimulationsvariante zuhoerer) {
		if (!zuhoererListe.contains(zuhoerer)) {
			zuhoererListe.add(zuhoerer);
		} else {
			Simulationsvariante.LOGGER
					.finest("Listener existiert bereits und wird nicht erneut hinzugefgt.");
		}
	}

	/**
	 * Entfernt einen {@link IZuhoererSimulationsvariante} von der Liste der
	 * Zuhrer.
	 * 
	 * @param zuhoerer
	 *            Zu entfernender {@link IZuhoererSimulationsvariante}.
	 */
	public void entferneZuhoererSimulationsvariante(
			final IZuhoererSimulationsvariante zuhoerer) {
		if (zuhoererListe.contains(zuhoerer)) {
			zuhoererListe.remove(zuhoerer);
		} else {
			Simulationsvariante.LOGGER
					.error(getClass().getName()
							+ "::entferneZuhoererSimulationsvariante \n\t=> Der bergebene Zuhrer vom Typ "
							+ zuhoerer.getClass().getName()
							+ " existiert nicht in der Liste der Zuhrer. Er kann nicht entfernt werden.");

		}
	}

	/**
	 * Benachrichtigt alle registrierten {@link IZuhoererSimulationsvariante}
	 * ber die aktuell eingestellte Simulationsvariante.
	 * 
	 */
	private void benachrichtigeZuhoerer() {
		Simulationsvariante.LOGGER.fine(getClass().getName()
				+ "::benachrichtigeZuhoerer \n\t=> Benachrichtige "
				+ zuhoererListe.size() + " Zuhrer.");

		final Iterator<IZuhoererSimulationsvariante> it = zuhoererListe
				.iterator();
		while (it.hasNext()) {
			final IZuhoererSimulationsvariante simulationsvarianteListener = it
					.next();
			simulationsvarianteListener
					.aktuelleSimulationsvariante(simulationsVariante);
		}

	}

	/**
	 * Private Konstruktor.
	 * 
	 */
	private Simulationsvariante() {
	}
}
