/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.FilterAusdruck;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur.TabellenFilter;
import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;

public class DialogFilterEditor extends Dialog {

	private static final String LABEL_INDEX = "Idx:";
	private static final String LABLE_TITEL = "Titel der Spalte:";
	private static final String LABEL_REGEX = "Regulrer Ausdruck:";

	private Shell shell;
	private Table tabelleUebersicht;
	private TableViewer tableViewer;

	private TabellenFilter tabellenFilter;
	private boolean bearbeitet;

	/**
	 * Konstruktor dieser Klasse.
	 * 
	 * @param parent
	 *            Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *            soll.
	 */
	public DialogFilterEditor(final Shell parent) {
		super(parent, 0);
	}

	/**
	 * ffnet diesen Dialog zur Bearbeitung eines {@link TabellenFilter}s.
	 * 
	 * @param neuerTabellenFilter
	 *            {@link TabellenFilter} zur Bearbeitung. Dieser wird gecloned,
	 *            so dass das Original nicht verndert wird.
	 * @return Liefert den bearbeiteten {@link TabellenFilter} zurck oder
	 *         <code>null</code>, wenn keine Bearbeitung statt gefunden hat.
	 */
	public TabellenFilter oeffne(final TabellenFilter neuerTabellenFilter) {
		/*
		 * Tiefe Kopie des TabellenFilters erzeugen
		 */
		if (neuerTabellenFilter != null) {
			this.tabellenFilter = (TabellenFilter) neuerTabellenFilter.clone();
		}

		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL
				| SWT.RESIZE);
		shell.setText("Filter Editor");

		shell.setImage(RwCompatActivator.getDefault().getImage(
				"icons/sonstiges/text_editor_16x16.png"));

		erzeugeDarstellung();

		if (neuerTabellenFilter != null) {
			tableViewer.setInput(neuerTabellenFilter);

			final CellEditor[] editors = new CellEditor[3];
			editors[0] = null;
			editors[1] = null;
			editors[2] = new TextCellEditor(tabelleUebersicht);
			tableViewer.setCellEditors(editors);
			tableViewer.setCellModifier(new CellMod());

		}

		shell.setSize(new Point(400, 300));
		shell.open();

		final Display display = getParent().getDisplay();

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		final Rectangle bounds = shell.getBounds();
		final Point cursorLocation = Display.getDefault().getCursorLocation();
		final Rectangle newBounds = new Rectangle(cursorLocation.x,
				cursorLocation.y, bounds.width, bounds.height);
		shell.setBounds(newBounds);

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		if (bearbeitet) {
			return neuerTabellenFilter;
		}
		return null;
	}

	/**
	 * Erzeugung der Darstellung.
	 */
	private void erzeugeDarstellung() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;

		erzeugeGruppeFilter();
		shell.setLayout(gridLayout);
		createBereichUnten();

	}

	/**
	 * Erzeugung von gruppeFilter.
	 * 
	 */
	private void erzeugeGruppeFilter() {
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final Group gruppeFilter = new Group(shell, SWT.NONE);
		gruppeFilter.setLayout(new GridLayout());
		gruppeFilter.setLayoutData(gridData);
		gruppeFilter.setText("Filter");
		tabelleUebersicht = new Table(gruppeFilter, SWT.FULL_SELECTION
				| SWT.BORDER);
		tabelleUebersicht.setHeaderVisible(true);
		tabelleUebersicht.setLayoutData(gridData2);
		tabelleUebersicht.setLinesVisible(true);
		tableViewer = new TableViewer(tabelleUebersicht);

		/*
		 * Spalten erzeugen
		 */
		final TableColumn tcIndex = new TableColumn(tabelleUebersicht,
				SWT.CENTER);
		tcIndex.setText(DialogFilterEditor.LABEL_INDEX);
		tcIndex.setWidth(35);

		final TableColumn tcSpalte = new TableColumn(tabelleUebersicht,
				SWT.CENTER);
		tcSpalte.setText(DialogFilterEditor.LABLE_TITEL);
		tcSpalte.setWidth(120);

		final TableColumn tcRegEx = new TableColumn(tabelleUebersicht,
				SWT.CENTER);
		tcRegEx.setText(DialogFilterEditor.LABEL_REGEX);
		tcRegEx.setWidth(200);

		/*
		 * ColumnProperties setzen
		 */
		tableViewer.setColumnProperties(new String[] {
				DialogFilterEditor.LABEL_INDEX, DialogFilterEditor.LABLE_TITEL,
				DialogFilterEditor.LABEL_REGEX });

		final ContentProvider cp = new ContentProvider();
		tableViewer.setContentProvider(cp);

		final LabelProvider lp = new LabelProvider();
		tableViewer.setLabelProvider(lp);

		tableViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {
					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final IStructuredSelection selection = (IStructuredSelection) event
								.getSelection();
						final Object firstElement = selection.getFirstElement();
						if (firstElement instanceof FilterAusdruck) {
							// TODO Was soll das?
							// System.out.println("YEHA");
						}
					}

				});

	}

	/**
	 * Aktion hinter der Schaltflche bernehmen. Das Flag
	 * {@link DialogFilterEditor#_bearbeitet} wird auf <code>true</code>
	 * gesetzt.
	 */
	private void aktionUebernehmen() {
		bearbeitet = true;
		aktionSchliessen();
	}

	/**
	 * Aktion hinter der Schaltflche Schlieen. Das
	 * {@link DialogFilterEditor#_bearbeitet} wird auf <code>false</code>, so
	 * dass von der {@link DialogFilterEditor#oeffne(TabellenFilter)}-Methode
	 * nur ein <code>null</code> zurckgeliefert wird.
	 */
	private void aktionSchliessen() {
		shell.setVisible(false);
		shell.dispose();
	}

	/**
	 * Aktion hinter der Schaltflche Bearbeiten. Bearbeitet den Regulren
	 * Ausdruck der aktuellen Selektion in der Tabelle.
	 */
	private void aktionBearbeiten() {
		final ISelection selection = tableViewer.getSelection();
		if (selection instanceof IStructuredSelection) {
			final IStructuredSelection iss = (IStructuredSelection) selection;
			final Object firstElement = iss.getFirstElement();
			if (firstElement instanceof FilterAusdruck) {
				final FilterAusdruck filterAusdruck = (FilterAusdruck) firstElement;
				final EditorRegulaererAusdruck era = new EditorRegulaererAusdruck(
						shell);
				final String oeffne = era.oeffne(filterAusdruck.getFilter());
				if (oeffne != null) {
					filterAusdruck.setFilter(oeffne);
					bearbeitet = true;
					tableViewer.setInput(tabellenFilter);
				}
			}
		}
	}

	/**
	 * ffnet die Hilfe zu diesem Dialog.
	 */
	private void aktionHilfe() {
		final String helpContextId = RwCompatActivator.PLUGIN_ID
				+ ".filterDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * Initialisierung des unteren Bereichs.
	 * 
	 */
	private void createBereichUnten() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.verticalAlignment = GridData.CENTER;
		final RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		final Composite bereichUnten = new Composite(shell, SWT.NONE);
		bereichUnten.setLayout(rowLayout);
		bereichUnten.setLayoutData(gridData1);
		final Button schaltflaecheUebernehmen = new Button(bereichUnten,
				SWT.NONE);
		schaltflaecheUebernehmen.setText("bernehmen");
		schaltflaecheUebernehmen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionUebernehmen();
			}
		});

		final Button schaltflaecheBearbeiten = new Button(bereichUnten,
				SWT.NONE);
		schaltflaecheBearbeiten.setText("Bearbeiten");
		schaltflaecheBearbeiten.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionBearbeiten();
			}
		});

		final Button schaltflaecheAbbrechen = new Button(bereichUnten, SWT.NONE);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionSchliessen();
			}
		});

		final Button schaltflaecheHilfe = new Button(bereichUnten, SWT.NONE);
		schaltflaecheHilfe.setText("Hilfe");
		schaltflaecheHilfe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionHilfe();
			}
		});
	}

	/**
	 * Liefert einen {@link IStructuredContentProvider} fr die Tabelle zur
	 * Darstellung der {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ContentProvider implements IStructuredContentProvider {

		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;

			if (inputElement instanceof TabellenFilter) {
				final TabellenFilter neuerTabellenFilter = (TabellenFilter) inputElement;
				final LinkedList<FilterAusdruck> filterAusdruecke = neuerTabellenFilter
						.getFilterAusdruecke();
				re = new Object[filterAusdruecke.size()];
				int i = 0;
				final Iterator<FilterAusdruck> it = filterAusdruecke.iterator();
				while (it.hasNext()) {
					re[i] = it.next();
					i++;
				}
			}

			return re;
		}

		@Override
		public void dispose() {
			// keine freizugebenden Ressourcen
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird global betrachtet
		}

	}

	/**
	 * Liefert einen {@link ITableLabelProvider} fr die Darstellung der Tabelle
	 * der {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class LabelProvider implements ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {
			String re = "";

			if (element instanceof FilterAusdruck) {
				final FilterAusdruck filterAusdruck = (FilterAusdruck) element;
				if (columnIndex == 0) {
					re = "" + filterAusdruck.getSpalte();
				} else if (columnIndex == 1) {
					re = "" + filterAusdruck.getBezeichnung();
				} else if (columnIndex == 2) {
					re = "" + filterAusdruck.getFilter();
				}
			}

			return re;
		}

		@Override
		public void addListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			return false;
		}

		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// nicht untersttzt
		}

	}

	/**
	 * Liefert einen {@link ICellModifier} zur Bearbeitung von Regulren
	 * Ausdrcken in der Tabelle zur Darstellung von {@link TabellenFilter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CellMod implements ICellModifier {

		@Override
		public boolean canModify(final Object element, final String property) {
			boolean re = false;

			if (property.equals(DialogFilterEditor.LABEL_REGEX)) {
				re = true;
			}

			return re;
		}

		@Override
		public Object getValue(final Object element, final String property) {
			Object re = null;

			if (property.equals(DialogFilterEditor.LABEL_REGEX)) {
				if (element instanceof FilterAusdruck) {
					final FilterAusdruck filterAusdruck = (FilterAusdruck) element;
					re = filterAusdruck.getFilter();
				}
			}

			return re;
		}

		@Override
		public void modify(final Object element, final String property,
				final Object value) {
			if (property.equals(DialogFilterEditor.LABEL_REGEX)) {
				if (element instanceof TableItem) {
					final TableItem ti = (TableItem) element;
					final FilterAusdruck tfi = (FilterAusdruck) ti.getData();

					if (value instanceof String) {
						final String wert = (String) value;

						if (!tfi.getFilter().equals(wert)) {
							tfi.setFilter(wert);
							bearbeitet = true;
							tableViewer.setInput(tabellenFilter);
						}
					}

				}
			}
		}

	}

}
