/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;

public class DragAdapter extends DragSourceAdapter {

	/**
	 * Speichert den {@link TableViewer}, auf den sich diese Drag-Fkt. bezieht.
	 */
	private final TableViewer tableViewer;

	/**
	 * Speichert die {@link TabellenDatenStruktur}, auf welche sich diese
	 * Drag-Funktionalitt bezieht.
	 */
	private final TabellenDatenStruktur tds;

	/**
	 * Konstruktor.
	 * 
	 * @param tds
	 *            {@link TabellenDatenStruktur}, auf welche sich die
	 *            Drag-Operationen beziehen sollen.
	 */
	public DragAdapter(final TableViewer tableViewer,
			final TabellenDatenStruktur tds) {
		this.tableViewer = tableViewer;
		this.tds = tds;
	}

	@Override
	public void dragStart(final DragSourceEvent event) {
		/*
		 * Wenn die Selektion nicht leer ist, dann darf der DRAG-Vorgang
		 * beginnen.
		 */
		event.doit = !tableViewer.getSelection().isEmpty();
	}

	@Override
	public void dragSetData(final DragSourceEvent event) {
		/*
		 * Daten der Selektion bestimmen
		 */
		final IStructuredSelection selection = (IStructuredSelection) tableViewer
				.getSelection();

		/*
		 * Selektiertes Element bestimmen und Zwischenspeichern
		 */
		final Object firstElement = selection.getFirstElement();
		if (firstElement instanceof ITabellenZeile) {
			/*
			 * Zwichenspeichern der Zeile
			 */
			final ITabellenZeile dragZeile = (ITabellenZeile) firstElement;
			tds.setDNDSpeicherZeile(dragZeile);

			/*
			 * Daten fr das Event setzen
			 */
			final String daten = "[ITabellenZeile]";
			event.data = daten;
		}
	}

	@Override
	public void dragFinished(final DragSourceEvent event) {
		if (!event.doit) {
			return;
		}

		if (event.detail == DND.DROP_MOVE) {
			tableViewer.refresh();
		}

	}

}
