/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.TextTransfer;

import de.bsvrz.sys.funclib.debug.Debug;

public class DragSourceAdapterTabellenSpalten extends DragSourceAdapter {
	/**
	 * Der Debugger.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Viewer, auf dem gearbeitet wird.
	 */
	private final StructuredViewer viewer;

	public DragSourceAdapterTabellenSpalten(final StructuredViewer viewer) {
		super();
		this.viewer = viewer;
	}

	@Override
	public void dragFinished(final DragSourceEvent event) {

		if (!event.doit) {
			return;
		}
		// if the gadget was moved, remove it from the source viewer
		if (event.detail == DND.DROP_MOVE) {
			// Lschen
			viewer.refresh();
		}
	}

	/**
	 * Liefert die per dragging selektierten Elemente des TableViewers.
	 */
	@Override
	public void dragSetData(final DragSourceEvent event) {
		final IStructuredSelection selection = (IStructuredSelection) viewer
				.getSelection();

		final Object selectedObject = selection.getFirstElement();

		if (selectedObject instanceof SpaltenElement) {
			final SpaltenElement spaltenElement = (SpaltenElement) selectedObject;
			final String id = spaltenElement.toString();
			SpaltenElementSpeicher.getInstanz().setSpaltenElement(
					spaltenElement);

			if (TextTransfer.getInstance().isSupportedType(event.dataType)) {
				event.data = id;
			}
		}

		DragSourceAdapterTabellenSpalten.LOGGER.fine(getClass().getName()
				+ "::dragSetData => Ende.");

	}

	/**
	 * Wird whrend des Starts der Drag-Vorgangs aufgerufen. Bestimmt den
	 * Datentyp und stellt sicher, dass keine leere Selektion "gedraged" werden
	 * kann.
	 */
	@Override
	public void dragStart(final DragSourceEvent event) {
		event.doit = !viewer.getSelection().isEmpty();
	}
}
