/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.compatibility.internal.RwCompatActivator;

/**
 * "Editor" fr regulre Ausdrcke. Liefert ein "groes" Textfeld, in dem der
 * regulre Ausdruck bearbeitet werden kann. <br>
 * 
 * Die Darstellung besteht aus:
 * <ul>
 * <li>Einem Textfeld, in dem der regulre Ausdruck bearbeitet werden kann.</li>
 * <li>Einem OK-Button, der die nderungen am regulren Ausdruck bernimmt.</li>
 * <li>Einem Abbrechen-Button, der die nderungen am regulren Ausruck verwirft.
 * </li>
 * </ul>
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und
 * Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @version $Revision: 1.3 $ / $Date: 2010/02/04 06:12:25 $ / ($Author: HofmannJ
 *          $)
 * @author HofmannJ
 * 
 */
public class EditorRegulaererAusdruck extends Dialog {

	/** Speichert den regulren Ausdruck. */
	private String regAusdruck;

	private Shell shell;

	private Text textEingabe;

	private boolean modified;

	/**
	 * Konstruktor, liefert eine neue Instanz der Klasse
	 * {@link EditorRegulaererAusdruck}.
	 * 
	 * @param parent
	 *            Eltern-Shell, die fr die Darstellung benutzt werden soll.
	 */
	public EditorRegulaererAusdruck(final Shell parent) {
		super(parent);
	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren/Anlegen
	 * eines neuen regulren Ausdrucks.
	 * 
	 * @return Regulrer Ausdruck als String.
	 */
	public String oeffne() {
		shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL
				| SWT.MIN | SWT.MAX | SWT.CLOSE);
		shell.setText("Editor regulrer Ausdruck");
		shell.setImage(RwCompatActivator.getDefault().getImage(
				"icons/sonstiges/text_editor_16x16.png"));

		shell.setLayout(new GridLayout());
		erzeugeBereichOben();
		erzeugeBereichUnten();
		shell.setSize(new Point(436, 259));

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		final Rectangle bounds = shell.getBounds();
		final Point cursorLocation = Display.getDefault().getCursorLocation();
		final Rectangle newBounds = new Rectangle(cursorLocation.x,
				cursorLocation.y, bounds.width, bounds.height);
		shell.setBounds(newBounds);

		shell.open();

		final Display display = getParent().getDisplay();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return regAusdruck;

	}

	/**
	 * ffnet den Dialog {@link EditorRegulaererAusdruck} zum Editieren eines
	 * bereits vorhandenen regulren Ausdrucks.
	 * 
	 * @param regAusdruck
	 *            Regulrer Ausdruck, der bearbeitet werden soll.
	 * 
	 * @return Liefert den bearbeiteten, regulren Ausduck. Der Dialog
	 *         {@link FilterDefinitionsdialog} kann die Darstellung des
	 *         regulren Ausdrucks des jeweiligen Filterelements hiermit
	 *         aktuallisieren.
	 * 
	 */
	public String oeffne(final String newRegAusdruck) {

		regAusdruck = newRegAusdruck;

		shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL
				| SWT.MIN | SWT.MAX | SWT.CLOSE);
		shell.setText("Editor regulrer Ausdruck");
		shell.setImage(RwCompatActivator.getDefault().getImage(
				"icons/sonstiges/text_editor_16x16.png"));

		shell.setLayout(new GridLayout());
		erzeugeBereichOben();
		erzeugeBereichUnten();
		shell.setSize(new Point(436, 259));

		/*
		 * Positionieren des Dialogs bei der Position des Mauszeigers
		 */
		final Rectangle bounds = shell.getBounds();
		final Point cursorLocation = Display.getDefault().getCursorLocation();
		final Rectangle newBounds = new Rectangle(cursorLocation.x,
				cursorLocation.y, bounds.width, bounds.height);
		shell.setBounds(newBounds);

		shell.open();
		textEingabe.setText(regAusdruck);

		final Display display = getParent().getDisplay();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

		return regAusdruck;
	}

	/**
	 * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionOk() {
		if (modified) {
			regAusdruck = textEingabe.getText();
			modified = false;
		}
		shell.dispose();
	}

	/**
	 * Verwirft die nderungen am regulren Ausdruck und schliet den Dialog.
	 * 
	 */
	private void aktionAbbrechen() {
		regAusdruck = null;
		shell.dispose();
	}

	/**
	 * ffnet die Hilfe zu diesem Editor.
	 */
	private void aktionHilfe() {
		final String helpContextId = RwCompatActivator.PLUGIN_ID
				+ ".filterDialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * This method initializes compositeTop.
	 * 
	 */
	private void erzeugeBereichOben() {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final Composite compositeTop = new Composite(shell, SWT.NONE);
		compositeTop.setLayout(new GridLayout());
		erzeugeGruppe(compositeTop);
		compositeTop.setLayoutData(gridData);
	}

	/**
	 * This method initializes compositeBottom.
	 * 
	 */
	private void erzeugeBereichUnten() {
		final RowLayout rowLayout = new RowLayout();
		rowLayout.justify = false;
		rowLayout.wrap = true;
		rowLayout.pack = false;
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.CENTER;
		gridData1.grabExcessHorizontalSpace = false;
		gridData1.verticalAlignment = GridData.CENTER;
		final Composite compositeBottom = new Composite(shell, SWT.NONE);
		compositeBottom.setLayoutData(gridData1);
		compositeBottom.setLayout(rowLayout);
		final Button schaltflaecheOk = new Button(compositeBottom, SWT.NONE);
		schaltflaecheOk.setText("Ok");
		schaltflaecheOk.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionOk();
			}
		});

		final Button schaltflaecheAbbrechen = new Button(compositeBottom,
				SWT.NONE);
		schaltflaecheAbbrechen.setText("Abbrechen");
		schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionAbbrechen();
			}
		});

		final Button schaltflaecheHilfe = new Button(compositeBottom, SWT.NONE);
		schaltflaecheHilfe.setText("Hilfe");
		schaltflaecheHilfe.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				aktionHilfe();
			}
		});
	}

	/**
	 * This method initializes group.
	 * 
	 */
	private void erzeugeGruppe(final Composite parent) {
		final GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.verticalAlignment = GridData.FILL;
		final GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.grabExcessVerticalSpace = true;
		gridData2.verticalAlignment = GridData.FILL;
		final Group gruppeBearbeiten = new Group(parent, SWT.NONE);
		gruppeBearbeiten.setLayout(new GridLayout());
		gruppeBearbeiten.setText("Regulrer Ausdruck");
		gruppeBearbeiten.setLayoutData(gridData2);
		textEingabe = new Text(gruppeBearbeiten, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		textEingabe.setLayoutData(gridData3);
		textEingabe.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(final ModifyEvent e) {
				modified = true;
			}
		});
	}
}
