/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.tabellen;

import java.util.LinkedList;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.TransferData;


/**
 * Liefert den {@link ViewerDropAdapter} mit dem es mglich wird, in dem
 * TableViewer des SichtEditorStatusleistenElemente per Drag & Drop
 * Statusleistenelemente in Ihrer XML-Reprsentation fallen zu lassen.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:19:28 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * 
 */
public class ViewerDropAdapterSpaltenReihenfolge extends ViewerDropAdapter {
	private final DialogSpaltenSortierungsReihenfolge dialogSpaltenSortierungsReihenfolge;

	protected ViewerDropAdapterSpaltenReihenfolge(
			final TableViewer viewer,
			final DialogSpaltenSortierungsReihenfolge dialogSpaltenSortierungsReihenfolge) {
		super(viewer);
		this.dialogSpaltenSortierungsReihenfolge = dialogSpaltenSortierungsReihenfolge;
	}

	@Override
	public boolean performDrop(final Object data) {
		final Object target = getCurrentTarget();

		if (target == null) {
			final LinkedList<SpaltenElement> sortierung = dialogSpaltenSortierungsReihenfolge
					.getSortierung();

			if (!sortierung.contains(SpaltenElementSpeicher.getInstanz()
					.getSpaltenElement())) {
				sortierung.add(SpaltenElementSpeicher.getInstanz()
						.getSpaltenElement());
				// TODO Was macht das hier?
				// final Iterator<SpaltenElement> it = sortierung.iterator();
				// while (it.hasNext()) {
				// final SpaltenElement se = it.next();
				// }

				dialogSpaltenSortierungsReihenfolge
						.aktuelleSortierungInTabelleDarstellen();
			}

		}

		return true;

	}

	@Override
	public boolean validateDrop(final Object target, final int operation,
			final TransferData transferType) {
		return true;
	}
}
