/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.  
 *   
 ******************************************************************************
 *
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.urlasser;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse liefert ein {@link UrlasserInfo}-Objekt, welches die Inhalte der
 * Urlasserabfrage, bestehend aus:
 * <ul>
 * <li>Benutzername,</li>
 * <li>Passwort,</li>
 * <li>Ursache und</li>
 * <li>Veranlasser</li>
 * </ul>
 * beschreibt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker,HofmannJ
 * @version $Id: UrlasserInfo.java 46673 2013-10-16 14:24:41Z peuker $
 */
public class UrlasserInfo {

	private static final String KEIN_BENUTZER_STRING = "<Kein Benutzer>";

	/** Benutzer. */
	private final SystemObject benutzer;

	/** Passwort. */
	private final String passwort;

	/** Ursache. */
	private final String ursache;

	/** Veranlasser. */
	private final String veranlasser;

	/**
	 * Konstruktor. bernimmt Benutzer, Ursache und den Veranlasser
	 * 
	 * @param benutzer
	 *            das Benutzerobjekt
	 * @param passwort
	 *            das Passwort
	 * @param ursache
	 *            die Ursache
	 * @param veranlasser
	 *            der Veranlasser
	 */
	public UrlasserInfo(final SystemObject benutzer, final String passwort,
			final String ursache, final String veranlasser) {
		this.benutzer = benutzer;
		this.passwort = passwort;
		this.veranlasser = veranlasser;
		this.ursache = ursache;
	}

	/**
	 * Liefert den Benutzer.
	 * 
	 * @return Benutzer. Kann <code>null</code> sein, wenn kein Benutzer gesetzt
	 *         wurde.
	 */
	public SystemObject getBenutzer() {
		return benutzer;
	}

	/**
	 * Liefert die Ursache oder einen Leerstring, wenn keine Ursache angegeben
	 * wurde.
	 * 
	 * @return die Ursache oder einen Leerstring
	 */
	public String getUrsache() {
		if (ursache == null) {
			return "";
		}
		return ursache;
	}

	/**
	 * Liefert den Veranlasser oder einen Leerstring, wenn keiner angegeben
	 * wurde.
	 * 
	 * @return den Veranlasser oder einen Leerstring.
	 */
	public String getVeranlasser() {
		if (veranlasser == null) {
			return "";
		}
		return veranlasser;
	}

	/**
	 * liefert den Name des Benutzers. Wenn kein Benutzer enthalten ist wird der
	 * {@value #KEIN_BENUTZER_STRING} geliefert.
	 * 
	 * @return den Name oder {@value #KEIN_BENUTZER_STRING}
	 */
	public String getBenutzerName() {
		if (benutzer == null) {
			return UrlasserInfo.KEIN_BENUTZER_STRING;
		}
		return benutzer.getNameOrPidOrId();
	}

	/**
	 * liefert das bergebene Passwort oder einen Leerstring, wenn keines
	 * bergeben wurde.
	 * 
	 * @return das Passwort oder einen Leerstring
	 */
	public String getPasswort() {
		if (passwort == null) {
			return "";
		}
		return passwort;
	}
}
