/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.basislib.util;

import org.eclipse.core.expressions.PropertyTester;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.internal.RahmenwerkService;

/**
 * Property-Tester, zum Prfen der Simulationsvariante.
 * 
 * Die ID des Testers ist: de.bsvrz.buv.rw.basislib.simulationsvariante.wert fr
 * die Prfung einer angegebenen Simulationsvariante oder
 * de.bsvrz.buv.rw.basislib.simulationsvariante.keineSimulation, um zu Prfen ob
 * eine Simulation verwendet wird
 * 
 * Die Argumente knnen leer bleiben.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungTester.java 46069 2013-09-17 12:11:12Z peuker $
 */
public class SimulationsVarianteTester extends PropertyTester {

	@Override
	public boolean test(final Object receiver, final String property,
			final Object[] args, final Object expectedValue) {
		boolean result = false;

		final Rahmenwerk rahmenwerk = RahmenwerkService.getService()
				.getRahmenWerk();
		if (rahmenwerk.isOnline()) {
			final short simulationVariant = rahmenwerk.getDavVerbindung()
					.getClientDavParameters().getSimulationVariant();

			if ("wert".equals(property)) {
				result = Short.valueOf(simulationVariant).equals(expectedValue);
			} else if ("keineSimulation".equals(property)) {
				result = simulationVariant <= 0;
			}
		}

		return result;
	}

}
