/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */
package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ColumnViewerEditor;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationEvent;
import org.eclipse.jface.viewers.ColumnViewerEditorActivationStrategy;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.jface.window.Window;
import org.eclipse.nebula.jface.gridviewer.GridTreeViewer;
import org.eclipse.nebula.jface.gridviewer.GridViewerColumn;
import org.eclipse.nebula.jface.gridviewer.GridViewerEditor;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.IWorkbenchCommandConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.CollapseAllHandler;
import org.eclipse.ui.handlers.ExpandAllHandler;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.urlasser.DefaultDatenSender;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserDatenDialog;
import de.bsvrz.buv.rw.basislib.util.RahmenwerkUtil;
import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungenBerechtigungsFunktion;
import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungenImpl;
import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungsParameterDatensatz;
import de.bsvrz.buv.rw.rw.berechtigungen.RwBerechtigungen;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Ansicht zum Bearbeiten der Berechtigungen innerhalb des Rahmenwerks.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungenEditor.java 51067 2014-07-30 09:23:27Z peuker $
 */
public class BerechtigungenEditor extends ViewPart implements ISaveablePart,
		IExpandable {

	/** die ID des Views (Editors). */
	public static final String VIEW_ID = "de.bsvrz.buv.rw.rw.berechtigungen.view.rechteeditor";

	/** Handler fr spezielle Tastaturcodes innerhalb des Grids. */
	private final class BerechtigungenEditorKeyHandler extends KeyAdapter {

		private Object spaltenObjekt;
		private Object zeilenObjekt;
		private int columnIndex = -1;

		@Override
		public void keyPressed(final KeyEvent e) {

			spaltenObjekt = null;
			zeilenObjekt = null;
			columnIndex = -1;

			final ColumnViewerEditor editor = berechtigungsMatrix
					.getColumnViewerEditor();
			if ((editor != null) && (editor.getFocusCell() != null)) {
				columnIndex = editor.getFocusCell().getColumnIndex();
				spaltenObjekt = berechtigungsMatrix.getGrid()
						.getColumn(columnIndex).getData("klasse");
				zeilenObjekt = ((IStructuredSelection) berechtigungsMatrix
						.getSelection()).getFirstElement();
			}

			if (e.keyCode == SWT.DEL) {
				handleDelKey();
			} else if ((columnIndex == 0)
					&& ((e.keyCode == SWT.SPACE) || (e.keyCode == SWT.CR))) {
				if (berechtigungsMatrix.isExpandable(zeilenObjekt)
						&& (!berechtigungsMatrix.getExpandedState(zeilenObjekt))) {
					berechtigungsMatrix.expandToLevel(zeilenObjekt,
							AbstractTreeViewer.ALL_LEVELS);
				} else {
					berechtigungsMatrix.collapseToLevel(zeilenObjekt,
							AbstractTreeViewer.ALL_LEVELS);
				}
			}
		}

		private void handleDelKey() {

			if (editierbar) {

				if (spaltenObjekt == null) {
					if (zeilenObjekt instanceof Funktion) {
						final Funktion funktion = (Funktion) zeilenObjekt;
						if (!funktion.isAvailable()) {
							if (MessageDialog
									.openConfirm(
											getSite().getShell(),
											"Funktion entfernen",
											"Soll die Funktion \""
													+ funktion
													+ "\" tatschlich entfernt werden?")) {
								datensatz.entferneFunktion(funktion);
								refreshTable();
								berechtigungsMatrix.refresh();
								setDirty(true);
							}
						}
					}
				} else if (spaltenObjekt instanceof BerechtigungsKlasse) {
					final BerechtigungsKlasse klasse = (BerechtigungsKlasse) spaltenObjekt;
					if (!klasse.isAvailable()) {
						if (MessageDialog.openConfirm(
								getSite().getShell(),
								"Berechtigunsklasse entfernen",
								"Soll die Berechtigungsklasse \""
										+ klasse.getName()
										+ "\" tatschlich entfernt werden?")) {
							datensatz.entferneKlasse(klasse);
							refreshTable();
							berechtigungsMatrix.refresh();
							setDirty(true);
						}
					}
				}
			}
		}
	}

	/**
	 * Editorsupport fr die Matrixzellen, es wird sofort eine Umschaltung
	 * vorgenommen mit den Zustnden FREI, GESPERRT, VERERBT.
	 */
	private final class BerechtigungsZuordnungEditorSupport extends
			EditingSupport {
		private final BerechtigungsKlasse klasse;

		private BerechtigungsZuordnungEditorSupport(final ColumnViewer viewer,
				final BerechtigungsKlasse klasse) {
			super(viewer);
			this.klasse = klasse;
		}

		@Override
		protected void setValue(final Object element, final Object value) {
			// wird nicht ausgefhrt, Umschaltung erfolgt unmittelbar
		}

		@Override
		protected Object getValue(final Object element) {
			return null;

		}

		@Override
		protected CellEditor getCellEditor(final Object element) {
			return null;
		}

		@Override
		protected boolean canEdit(final Object element) {
			if (editierbar) {
				if (element instanceof Funktion) {
					final Funktion funktion = (Funktion) element;
					final BerechtigungsKey key = new BerechtigungsKey(klasse,
							funktion);
					final Boolean status = datensatz.getFreigabeStatus(key);
					if (status == null) {
						datensatz.setFreigabeStatus(key, true);
					} else if (status) {
						datensatz.setFreigabeStatus(key, false);
					} else {
						datensatz.setFreigabeStatus(key, null);
					}
					berechtigungsMatrix.refresh(true);
					setDirty(true);
				}
			}
			return false;
		}

	}

	private GridTreeViewer berechtigungsMatrix;
	private BerechtigungsParameterDatensatz datensatz;
	private BerechtigungsGruppenHeaderRenderer headerRenderer;
	private Boolean dirty = Boolean.FALSE;
	private boolean editierbar;

	@Override
	public void createPartControl(final Composite parent) {

		editierbar = BerechtigungenBerechtigungsFunktion.BERECHTIGUNG_AENDERN
				.isFreigegeben() || RahmenwerkUtil.isUserAdmin();

		final Composite treePanel = new Composite(parent, SWT.NONE);
		treePanel.setLayout(new GridLayout());

		berechtigungsMatrix = new GridTreeViewer(treePanel, SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
		GridDataFactory.fillDefaults().grab(true, true)
				.applyTo(berechtigungsMatrix.getControl());

		berechtigungsMatrix.getGrid().setHeaderVisible(true);
		berechtigungsMatrix.getGrid().setLinesVisible(true);

		berechtigungsMatrix.getGrid().setCellSelectionEnabled(true);
		berechtigungsMatrix.setSorter(new BerechtigungenSorter());
		berechtigungsMatrix
				.setFilters(new ViewerFilter[] { new BerechtigungenFilter() });
		berechtigungsMatrix
				.setContentProvider(new BerechtigungsParameterContentProvider());

		final ColumnViewerEditorActivationStrategy actSupport = new ColumnViewerEditorActivationStrategy(
				berechtigungsMatrix) {
			@Override
			protected boolean isEditorActivationEvent(
					final ColumnViewerEditorActivationEvent event) {
				return (event.eventType == ColumnViewerEditorActivationEvent.TRAVERSAL)
						|| (event.eventType == ColumnViewerEditorActivationEvent.MOUSE_DOUBLE_CLICK_SELECTION)
						|| ((event.eventType == ColumnViewerEditorActivationEvent.KEY_PRESSED) && ((event.keyCode == SWT.CR) || (event.keyCode == SWT.SPACE)))
						|| (event.eventType == ColumnViewerEditorActivationEvent.PROGRAMMATIC);
			}
		};

		GridViewerEditor.create(berechtigungsMatrix, actSupport,
				ColumnViewerEditor.TABBING_HORIZONTAL
						| ColumnViewerEditor.TABBING_MOVE_TO_ROW_NEIGHBOR
						| ColumnViewerEditor.TABBING_VERTICAL
						| ColumnViewerEditor.KEYBOARD_ACTIVATION);

		BerechtigungenImpl.getInstance().addDatenSatzListener(
				new PropertyChangeListener() {
					@Override
					public void propertyChange(final PropertyChangeEvent evt) {
						if (BerechtigungenImpl.PROP_DATENSATZ.equals(evt
								.getPropertyName())) {
							if (!dirty) {
								datensatz = new BerechtigungsParameterDatensatz(
										BerechtigungenImpl.getInstance(),
										BerechtigungenImpl.getInstance()
												.getBerechtigungsDatenSatz());
								refreshTable();
							}
						}
					}
				});

		headerRenderer = new BerechtigungsGruppenHeaderRenderer();

		berechtigungsMatrix.getGrid().addKeyListener(
				new BerechtigungenEditorKeyHandler());
		ColumnViewerToolTipSupport.enableFor(berechtigungsMatrix,
				ToolTip.NO_RECREATE);

		datensatz = new BerechtigungsParameterDatensatz(
				BerechtigungenImpl.getInstance(), BerechtigungenImpl
						.getInstance().getBerechtigungsDatenSatz());
		refreshTable();

		final IHandlerService service = (IHandlerService) getSite().getService(
				IHandlerService.class);
		if (service != null) {
			service.activateHandler(IWorkbenchCommandConstants.NAVIGATE_EXPAND_ALL,
					new ExpandAllHandler(berechtigungsMatrix));
			service.activateHandler(IWorkbenchCommandConstants.NAVIGATE_COLLAPSE_ALL,
					new CollapseAllHandler(berechtigungsMatrix));
		}

		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(berechtigungsMatrix.getControl(),
						RwBerechtigungen.PLUGIN_ID + ".berechtigungen_editor");

	}

	private void refreshTable() {

		if ((berechtigungsMatrix != null)
				&& (!berechtigungsMatrix.getControl().isDisposed())) {
			berechtigungsMatrix.getControl().getDisplay()
					.asyncExec(new Runnable() {
						@Override
						public void run() {
							if ((berechtigungsMatrix != null)
									&& (!berechtigungsMatrix.getControl()
											.isDisposed())) {

								final Object[] expandedElements = berechtigungsMatrix
										.getExpandedElements();

								for (final GridColumn column : berechtigungsMatrix
										.getGrid().getColumns()) {
									column.dispose();
								}

								GridViewerColumn column = new GridViewerColumn(
										berechtigungsMatrix, SWT.NONE);

								column.getColumn().setWidth(300);
								column.getColumn().setText("Funktionen");
								column.getColumn()
										.setHeaderTooltip(
												"Die Funktionen, denen Berechtigungen zugewiesen werden knnen");
								column.setLabelProvider(new BerechtigungsParameterLabelProvider());
								column.getColumn().setTree(true);

								column = new GridViewerColumn(
										berechtigungsMatrix, SWT.NONE);
								column.getColumn().setWidth(50);
								column.getColumn().setHeaderRenderer(
										headerRenderer);
								column.getColumn().getCellRenderer()
										.setAlignment(SWT.CENTER);

								column.setEditingSupport(new BerechtigungsZuordnungEditorSupport(
										berechtigungsMatrix,
										datensatz
												.getWildcardBerechtigungsKlasse()));
								column.getColumn()
										.setText(
												datensatz
														.getWildcardBerechtigungsKlasse()
														.getName());
								column.getColumn()
										.setHeaderTooltip(
												"Funktion ist fr alle Berechtigunsklassen freigegegen oder gesperrt!");
								column.setLabelProvider(new BerechtigungsParameterLabelProvider(
										datensatz,
										datensatz
												.getWildcardBerechtigungsKlasse()));

								final List<BerechtigungsKlasse> berechtigungsKlassen = new ArrayList<BerechtigungsKlasse>(
										datensatz.getBerechtigungsKlassen());
								Collections.sort(berechtigungsKlassen,
										new Comparator<BerechtigungsKlasse>() {

											@Override
											public int compare(
													final BerechtigungsKlasse o1,
													final BerechtigungsKlasse o2) {
												return o1.getName().compareTo(
														o2.getName());
											}
										});

								for (final BerechtigungsKlasse klasse : berechtigungsKlassen) {
									column = new GridViewerColumn(
											berechtigungsMatrix, SWT.NONE);
									column.getColumn().setWidth(50);
									column.getColumn().setHeaderRenderer(
											headerRenderer);
									column.getColumn().getCellRenderer()
											.setAlignment(SWT.CENTER);
									column.getColumn()
											.setData("klasse", klasse);

									column.setEditingSupport(new BerechtigungsZuordnungEditorSupport(
											berechtigungsMatrix, klasse));

									if (klasse.isAvailable()) {
										column.getColumn().setText(
												klasse.getName());
									} else {
										column.getColumn().setText(
												"* " + klasse.getName());
									}
									column.getColumn().setHeaderTooltip(
											klasse.getName() + "("
													+ klasse.getId() + ")");
									column.setLabelProvider(new BerechtigungsParameterLabelProvider(
											datensatz, klasse));
								}

								berechtigungsMatrix.getGrid()
										.recalculateHeader();
								berechtigungsMatrix.getGrid().getParent()
										.layout();

								berechtigungsMatrix.setInput(datensatz);
								berechtigungsMatrix
										.setExpandedElements(expandedElements);
							}
						}
					});
		}
	}

	@Override
	public void setFocus() {
		if ((berechtigungsMatrix != null)
				&& (!berechtigungsMatrix.getGrid().isDisposed())) {
			berechtigungsMatrix.getGrid().setFocus();
		}
	}

	@Override
	public void doSave(final IProgressMonitor monitor) {
		if (editierbar) {
			if (datensatz != null) {
				final ResultData resultData = datensatz.erzeugeDatenSatz();
				if (resultData != null) {
					final UrlasserDatenDialog dialog = new UrlasserDatenDialog(
							getViewSite().getShell(), new DefaultDatenSender(
									Collections.singleton(resultData)));
					dialog.setTitle("Berechtigungen sichern");
					dialog.setInitialUrsache("Berechtigungen aktualisiert");
					dialog.setInitialVeranlasser(BerechtigungenImpl
							.getInstance().getRahmenwerk().getBenutzerName());
					if (dialog.open() == Window.OK) {
						setDirty(false);
					}
				}
			}
		} else {
			MessageDialog
					.openError(getViewSite().getShell(),
							"Speichern der Berechtigungen",
							"Sie haben keine Berechtigung zum ndern der Berechtigungen!");
		}
	}

	@Override
	public void doSaveAs() {
		// nicht untersttzt
	}

	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public boolean isSaveOnCloseNeeded() {
		return editierbar && dirty;
	}

	private void setDirty(final boolean state) {
		if (editierbar) {
			if (state != dirty) {
				dirty = state;
				firePropertyChange(ISaveablePart.PROP_DIRTY);
			}
		}
	}
}
