/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridHeaderRenderer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Transform;

/**
 * Renderer fr die um 90 Grad gedrehte Anzeige der Berechtigungsklassen im
 * Header der Matrix.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsGruppenHeaderRenderer.java 45978 2013-09-13 11:41:11Z peuker $
 */
public class BerechtigungsGruppenHeaderRenderer extends GridHeaderRenderer {

	@Override
	public boolean notify(final int event, final Point point, final Object value) {
		return false;
	}

	@Override
	public void paint(final GC gc, final Object value) {

		final GridColumn column = (GridColumn) value;
		final Point textExtent = gc.textExtent(column.getText());

		gc.setBackground(column.getDisplay().getSystemColor(SWT.COLOR_GRAY));
		gc.setForeground(column.getDisplay().getSystemColor(SWT.COLOR_BLACK));
		gc.fillRectangle(getBounds().x, getBounds().y, getBounds().width - 1,
				getBounds().height);
		gc.drawRectangle(getBounds().x, getBounds().y, getBounds().width - 1,
				getBounds().height);

		final Transform oldTransform = new Transform(column.getDisplay());
		gc.getTransform(oldTransform);

		final Transform newTransform = new Transform(column.getDisplay());
		newTransform
				.translate(
						((getBounds().x - (textExtent.y / 2)) + (getBounds().width / 2)),
						(getBounds().x + textExtent.x + 5));
		newTransform.rotate(270);
		gc.setTransform(newTransform);
		gc.drawString(column.getText(), getBounds().x, getBounds().y);

		gc.setTransform(oldTransform);
	}

	@Override
	public Point computeSize(final GC gc, final int wHint, final int hHint,
			final Object value) {
		final GridColumn column = (GridColumn) value;
		final Point textExtent = gc.textExtent(column.getText());
		return new Point(textExtent.y + 10, textExtent.x + 10);
	}
}
