/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Reprsentation einer Berechtigungsklasse in der Visualisierung der
 * Berechtigungen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsKlasse.java 46166 2013-09-24 11:17:07Z peuker $
 */
public class BerechtigungsKlasse implements Comparable<BerechtigungsKlasse> {

	private final String id;
	private final String name;
	private final SystemObject klassenObjekt;

	/**
	 * Konstruktor, erstellt die Reprsentation der Berechtigungsklasse mit der
	 * bergebenen Id.
	 * 
	 * @param id
	 *            die Id
	 */
	public BerechtigungsKlasse(final String id) {
		this.id = id;
		this.name = id;
		this.klassenObjekt = null;
	}

	/**
	 * Konstruktor, erstellt die Reprsentation der Berechtigungsklasse fr das
	 * bergebene Systemobjekt.
	 * 
	 * @param klasse
	 *            die Berchtigungsklasse als SystemObject
	 */
	public BerechtigungsKlasse(final SystemObject klasse) {
		this.id = klasse.getPid();
		this.name = klasse.getName();
		this.klassenObjekt = klasse;
	}

	@Override
	public int compareTo(final BerechtigungsKlasse o) {
		return id.compareTo(o.id);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = (prime * result) + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final BerechtigungsKlasse other = (BerechtigungsKlasse) obj;
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		return true;
	}

	/**
	 * liefert die Id der Klassenreprsentation.
	 * 
	 * @return die Id
	 */
	public String getId() {
		return id;
	}

	/**
	 * liefert den Name der Klassenreprsentation.
	 * 
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * die Benutzerklasse gilt als verfgbar, wenn sie durch ein entsprechendes
	 * Datenverteiler-Objekt reprsentiert wird.
	 * 
	 * @return die Verfgbarkeit
	 */
	public boolean isAvailable() {
		return klassenObjekt != null;
	}

	@Override
	public String toString() {
		return getName();
	}
}
