/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 */

package de.bsvrz.buv.rw.rw.berechtigungen.editor;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import de.bsvrz.buv.rw.rw.berechtigungen.BerechtigungsParameterDatensatz;

/**
 * COntentprovider, der die Informationen aus dem Parameterdatensatz zur
 * Definition der Berechtigungen bereitstellt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: BerechtigungsParameterContentProvider.java 45978 2013-09-13 11:41:11Z peuker $
 */
public class BerechtigungsParameterContentProvider implements
		ITreeContentProvider {

	private BerechtigungsParameterDatensatz berechtigungsParameterDatensatz;

	@Override
	public void dispose() {
		// es sind keine Ressourcen freizugeben
	}

	@Override
	public void inputChanged(final Viewer viewer, final Object oldInput,
			final Object newInput) {

		berechtigungsParameterDatensatz = null;
		if (newInput instanceof BerechtigungsParameterDatensatz) {
			berechtigungsParameterDatensatz = (BerechtigungsParameterDatensatz) newInput;
		}
	}

	@Override
	public Object[] getElements(final Object inputElement) {
		final Collection<Object> result = new ArrayList<Object>();
		if (berechtigungsParameterDatensatz != null) {
			if (inputElement instanceof BerechtigungsParameterDatensatz) {
				result.add(berechtigungsParameterDatensatz
						.getWildcardFunktion());
				result.addAll(berechtigungsParameterDatensatz.getPlugins());
				result.add(berechtigungsParameterDatensatz
						.getNichtZugeordneteFunktionen());
			}
		}
		return result.toArray();
	}

	@Override
	public Object[] getChildren(final Object parentElement) {
		final Collection<Object> result = new ArrayList<Object>();
		if (parentElement instanceof PluginMitBerechtigungen) {
			final PluginMitBerechtigungen plugin = (PluginMitBerechtigungen) parentElement;
			result.addAll(plugin.getFunktionen());
		}
		return result.toArray();
	}

	@Override
	public Object getParent(final Object element) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasChildren(final Object element) {
		return getChildren(element).length > 0;
	}
}
