/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import java.util.LinkedList;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse stellt zur Laufzeit der RCP-Applikation die Daten des aktuell
 * angemeldeten Benutzers zur Verfgung. Damit zur Laufzeit der RCP-Anwendung
 * nur eine Instanz dieser Klasse existiert, ist sie als Singleton
 * implementiert.<br>
 * <br>
 * Desweitern knnen sich Zuhrer vom Typ {@link IBenutzerZuhoerer} registrieren
 * und werden dann darber Informiert, dass sich ein neuer Benutzer angemeldet
 * hat.
 * 
 * @author HofmannJ
 * 
 * @deprecated die Daten werden vom RahmenwerkService bereitgestellt
 */
@Deprecated
public final class BenutzerAngemeldet {

	/** der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	/** Benutzername. */
	// private String name;

	/**
	 * Liste der {@link IBenutzerZuhoerer}, die ber einen neue angemeldeten
	 * Benuter informiert werden mchten.
	 */
	private final LinkedList<IBenutzerZuhoerer> benutzerZuhoerer = new LinkedList<IBenutzerZuhoerer>();

	private final Rahmenwerk rahmenwerk;

	/**
	 * Privater Konstruktor dieser Singleton-Klasse.
	 * 
	 */
	private BenutzerAngemeldet() {
		rahmenwerk = RahmenwerkService.getService().getRahmenWerk();
		rahmenwerk.addDavVerbindungsListener(new DavVerbindungsListener() {
			@Override
			public void verbindungHergestellt(final DavVerbindungsEvent event) {
				// TODO Auto-generated method stub

			}

			@Override
			public void verbindungGetrennt(final DavVerbindungsEvent event) {
				// TODO Auto-generated method stub

			}

			@Override
			public boolean verbindungHalten(final DavVerbindungsEvent event) {
				// TODO Auto-generated method stub
				return false;
			}
		});
	}

	/**
	 * Speichert die Singleton-Instanz dieser Klasse.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen {
		private static BenutzerAngemeldet instanz = new BenutzerAngemeldet();
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse zurck.
	 * 
	 * @return Singleton-Instanz von {@link BenutzerAngemeldet}.
	 */
	public static BenutzerAngemeldet getInstanz() {
		return Innen.instanz;
	}

	/*
	 * Setter und Getter
	 */

	/**
	 * Setzt den angemeldeter Benutzer.
	 * 
	 * @param name
	 *            Name des Benutzers
	 * @param passwort
	 *            Passwort des Benutzers
	 * @param ip
	 *            IP-Adresse des DaV
	 * @param port
	 *            Port-Nummer des DaV
	 * 
	 * @deprecated der angemeldete Nutzer wird vom Rahmenwerk gesetzt, das
	 *             Setzen von auen wird n icht mehr untersttzt
	 */
	@SuppressWarnings("unused")
	@Deprecated
	public void setAngemeldeterBenutzer(final String newName,
			final String newPasswort, final String ip, final int port) {
		BenutzerAngemeldet.LOGGER
				.warning("Die Daten des angemeldeten Benutzers knnen nicht direkt gesetzt werden, stattdessen wird versucht eine Datenverteilerverbindung aufzubauen!");
	}

	/**
	 * Fgt einen {@link IBenutzerZuhoerer} zur Liste der Zuhrer hinzu.
	 * 
	 * @param benutzerZuhoerer
	 *            Hinzuzufgender {@link IBenutzerZuhoerer}.
	 */
	public void hinzufuegenBenutzerZuhoerer(final IBenutzerZuhoerer listener) {
		if (!benutzerZuhoerer.contains(listener)) {
			benutzerZuhoerer.add(listener);
		} else {
			BenutzerAngemeldet.LOGGER
					.error(getClass().getName()
							+ "::hinzufuegenBenutzerZuhoerer \n\t=>Der BenutzerZuhrer ist bereits in der Zuhrerliste vorhanden und wird nicht erneut hinzugefgt.");
		}
	}

	/**
	 * Entfernt einen {@link IBenutzerZuhoerer} von der Liste der Zuhrer.
	 * 
	 * @param benutzerZuhoerer
	 *            Zu entfernender {@link IBenutzerZuhoerer}.
	 */
	public void entferneBenutzerZuhoerer(final IBenutzerZuhoerer listener) {
		if (benutzerZuhoerer.contains(listener)) {
			benutzerZuhoerer.remove(listener);
		} else {
			BenutzerAngemeldet.LOGGER
					.error(getClass().getName()
							+ "::entferneBenutzerZuhoerer=>Der BenutzerZuhrer ist in der Zuhrerliste nicht vorhanden und wird somit nicht entfernt.");
		}
	}

	/**
	 * Getter IP-Adresse.
	 * 
	 * @return IP-Adresse als String
	 */
	public String getIp() {
		if (rahmenwerk.isOnline()) {
			return rahmenwerk.getDavVerbindung().getClientDavParameters()
					.getDavCommunicationAddress();
		}
		return null;
	}

	/**
	 * Getter Benutzername.
	 * 
	 * @return Benutzername Liefert den Benutzernamen.
	 */
	public String getName() {
		return rahmenwerk.getBenutzerName();
	}

	/**
	 * Getter Passwort.
	 * 
	 * @return Liefert das Passwort.
	 */
	public String getPasswort() {
		BenutzerAngemeldet.LOGGER
				.warning("Passwort kann nicht ber die API abgefragt werden!");
		return null;
	}

	/**
	 * Getter IP-Adresse.
	 * 
	 * @return Liefert die IP-Adresse-
	 * 
	 */
	public int getTcpPort() {
		if (rahmenwerk.isOnline()) {
			return rahmenwerk.getDavVerbindung().getClientDavParameters()
					.getDavCommunicationSubAddress();
		}

		return -1;
	}

	/**
	 * Setzt alle Parameter zurck.
	 * 
	 */
	public void zuruecksetzen() {
		BenutzerAngemeldet.LOGGER
				.warning("Die Nutzerinformationen knnen hier nicht zurckgesetzt werden, das passiert, wenn die Datenverteilerverbindung beendet wird!");
	}
}
