/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

import java.util.EventObject;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.dav.daf.main.ClientDavInterface;

/**
 * Liefert den Event der ans die Zuhrer der DaV-Verbindung durch deren Listener
 * {@link IDaVVerbindungZuhoerer} bertragen wird. <br>
 * Dadruch kann z.B. die Verbindung oder der Grund fr den Verbindugnsabbau
 * abgefragt werden.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ
 *          $)
 * 
 * @author HofmannJ
 * @deprecated ersetzt durch {@link DavVerbindungsEvent} des
 *             {@link DavVerbindungsListener}
 * 
 */
@Deprecated
public class DaVVerbindungEreignis extends EventObject {

	/** DaV-Adresse. */
	private String host;

	/** DaV-Port. */
	private int port = -1;

	/** Benutzer-Name. */
	private String benutzer;

	/** Urlasser-Name. */
	private String urlasser;

	/** Verbindung zum DaV. */
	private ClientDavInterface connection;

	/** Gibt den Grund fr den Verbindungsabbau an. */
	private String grundVerbindungsabbau;

	/** Gibt den Grund fr den Verbindungsaufbau an. */
	private String grundVerbindungsaufbau;

	/** Generierte serielle Id. */
	private static final long serialVersionUID = -3901539737900085638L;

	/**
	 * Konstruktor, wenn eine Verbindung aufgebaut wurde.
	 * 
	 * @param source
	 *            Eventauslsendes Objekt.
	 * @param ipAdresse
	 *            IP-Adresse der DaV-Verbindung.
	 * @param port
	 *            Port der DaV-Verbindung.
	 * @param benutzer
	 *            Benutzer der DaV-Verbindung.
	 * @param urlasser
	 *            Urlasser-Benutzer der DaV-Verbindung.
	 * @param grundVerbindungsAufbau
	 *            Grund fr den Verbindungsaufbau.
	 */
	public DaVVerbindungEreignis(final Object source,
			final ClientDavInterface connection, final String ipAdresse,
			final int port, final String benutzer, final String urlasser,
			final String grundVerbindungsAufbau) {
		super(source);
		this.connection = connection;
		this.host = ipAdresse;
		this.port = port;
		this.benutzer = benutzer;
		this.urlasser = urlasser;
		this.grundVerbindungsaufbau = grundVerbindungsAufbau;
	}

	/**
	 * Konstruktor, wenn die Verbindung abgebaut wurde.
	 * 
	 * @param source
	 *            Eventauslsendes Objekt.
	 * @param grundVerbindungsAbbau
	 *            Grund fr den Verbindungsabbau.
	 */
	public DaVVerbindungEreignis(final Object source,
			final String grundVerbindungsAbbau) {
		super(source);
		this.grundVerbindungsabbau = grundVerbindungsAbbau;
	}

	/**
	 * Liefert den DaV-Benutzer.
	 * 
	 * @return DaV-Benutzer.
	 */
	public String getDaVBenutzer() {
		return benutzer;
	}

	/**
	 * Liefert den DaV-Urlasser-Benutzer.
	 * 
	 * @return DaV-Urlasser-Benutzer.
	 */
	public String getDaVUrlasser() {
		return urlasser;
	}

	/**
	 * Liefert die DaV-Verbindung.
	 * 
	 * @return DaV-Verbindung.
	 */
	public ClientDavInterface getDaVVerbindung() {
		return connection;
	}

	/**
	 * Liefert die DaV-IP Adresse.
	 * 
	 * @return DaV-IP Adresse.
	 */
	public String getDaVIpAdresse() {
		return host;
	}

	/**
	 * Liefert den DaV-Port.
	 * 
	 * @return DaV-Port.
	 */
	public int getDaVPort() {
		return port;
	}

	/**
	 * Liefert den Grund fr den Verbindungsabbau.
	 * 
	 * @return Grund fr den Verbindungsabbau.
	 */
	public String getGrundDaVVerbindunsAbbau() {
		return grundVerbindungsabbau;
	}

	public String getGrundDaVVerbindungsAufbau() {
		return grundVerbindungsaufbau;
	}

}
