/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav;

/**
 * Speichert die Daten des Urlassers, ist als Singleton ausgelegt.
 * 
 * @author HofmannJ
 * 
 */
public final class UrlasserAngemeldet {
	/**
	 * Benutzername.
	 */
	private String name;

	/**
	 * Passwort.
	 */
	private String passwort;

	/**
	 * Ursache.
	 */
	private String ursache;

	/**
	 * Veranlasser.
	 */
	private String veranlasser;

	/**
	 * Privater Konstruktor, kann von auen nicht aufgerufen werden.
	 * 
	 */
	private UrlasserAngemeldet() {
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link UrlasserAngemeldet}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen {
		private static final UrlasserAngemeldet INSTANZ = new UrlasserAngemeldet();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link UrlasserAngemeldet}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link UrlasserAngemeldet}
	 */
	public static UrlasserAngemeldet getInstanz() {
		return Innen.INSTANZ;
	}

	/*
	 * Setter und Getter
	 */

	/**
	 * Liefert den Namen des Urlassers.
	 * 
	 * @return Urlassername
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setzt den Namen des Urlassers.
	 * 
	 * @param name
	 *            Urlassername
	 */
	public void setName(final String name) {
		this.name = name;
	}

	/**
	 * Liefert das Urlasserpasswort.
	 * 
	 * @return Urlasserpasswort
	 */
	public String getPasswort() {
		return passwort;
	}

	/**
	 * Setzt das Urlasserpasswort.
	 * 
	 * @param passwort
	 *            Urlasserpaswort
	 */
	public void setPasswort(final String passwort) {
		this.passwort = passwort;
	}

	/**
	 * Liefert die Ursache.
	 * 
	 * @return Ursacher
	 */
	public String getUrsache() {
		return ursache;
	}

	/**
	 * Setzt die Ursache.
	 * 
	 * @param ursache
	 *            Ursache
	 */
	public void setUrsache(final String ursache) {
		this.ursache = ursache;
	}

	/**
	 * Liefert den Veranlasser.
	 * 
	 * @return Liefert den Veranlasser.
	 */
	public String getVeranlasser() {
		return veranlasser;
	}

	/**
	 * Setzt den Veranlasser.
	 * 
	 * @param veranlasser
	 */
	public void setVeranlasser(final String veranlasser) {
		this.veranlasser = veranlasser;
	}

	/**
	 * Setzt alle Parameter zurck.
	 * 
	 */
	public void zuruecksetzen() {
		name = null;
		passwort = null;
		ursache = null;
		veranlasser = null;
	}

}
