/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav.datenbasis.prelader;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;


/**
 * Dieses Interface liefert einen Listener, dessen Instanzen bei der Klasse
 * {@link DaVPreLader} angemeldet werden knnen. Man kann sich durch diesen
 * Listener darber informieren lassen, wenn eine aktuelle {@link GtmDatenBasis}
 * durch die Klasse {@link DaVPreLader} zur Verfgung steht bzw. wenn eine
 * Aktualisierung auf eine bereits vorhandene {@link GtmDatenBasis} durchgefhrt
 * wurde.
 * 
 * @author HofmannJ
 * 
 */
public interface IDaVPreLader {
	/**
	 * Diese Methode wird von {@link DaVPreLader} aufgerufen, wenn das initiale
	 * Laden der SystemObjects aus dem DaV abgeschlossen ist. Es ist zu
	 * beachten, dass der Aufruf dieser Methode durch einen separaten Thread
	 * erfolgt. Wurde das initiale Laden bereits ein mal durchgefhrt, so wird
	 * diese Methode sogleich nach dem Anmelden dieses Listeners bei der Klasse
	 * {@link DaVPreLader#hinzufuegenListenerPreLader(IDaVPreLader)} mit den
	 * aktuell vorhandenen SystemObes aufgerufen.
	 * 
	 * @param systemObjects
	 *            Liste von beim initialen Ladevorgang
	 */
	void systemObjekteInitial(GtmDatenBasis datenBasis);

	/**
	 * Diese Methode wird vom {@link DaVPreLader} aufgerufen, wenn eine
	 * Aktualisierung der Liste der SystemObjects aus dem DaV vorliegt. Es ist
	 * zu beachten, dass der Aufruf dieser Methode durch einen separaten Thread
	 * erfolgt.
	 * 
	 * @param systemObjects
	 *            Liste von beim Aktualisieren
	 */
	void systemObjekteAktualisierung(GtmDatenBasis datenBasis);
}
