/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 *
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.einstellungen;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungLocation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungOwnerType;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Konnektor zur bertragung von Einstellungsspeicherdaten zum und vom
 * Datenverteiler.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: DavConnector.java 51069 2014-07-30 13:31:37Z peuker $
 */
class DavConnector implements ClientReceiverInterface, ClientSenderInterface {

	private final AttributeGroup netzAtg;
	private final Aspect sollAsp;
	private final Aspect vorgabeAsp;
	private final AttributeGroup benutzerAtg;
	private final EinstellungenSpeichermanager speichermanager;
	private static final Debug LOGGER = Debug.getLogger();

	private static final String START_FRAME_EXPRESSION = "[\\s]*\\[START\\]";
	private static final String ENDE_FRAME_EXPRESSION = "\\[ENDE\\][\\s]*";
	private static final int MAX_LINE_LENGTH = 15000;

	static String entpackeString(final String string) {
		String result = string.trim();
		result = result.replaceAll(DavConnector.START_FRAME_EXPRESSION, "");
		result = result.replaceAll(DavConnector.ENDE_FRAME_EXPRESSION, "");

		return result;
	}

	private final ClientDavInterface dav;
	private final DataModel model;
	private final SystemObject oberflaechenObject;

	public DavConnector(final EinstellungenSpeichermanager speichermanager,
			final Rahmenwerk rahmenwerk) {
		if (!rahmenwerk.isOnline()) {
			throw new IllegalStateException(
					"Es besteht keine Rahmenwerkverbindung");
		}

		this.speichermanager = speichermanager;

		dav = rahmenwerk.getDavVerbindung();
		model = dav.getDataModel();
		oberflaechenObject = rahmenwerk.getOberflaechenObject();

		netzAtg = model
				.getAttributeGroup("atg.globaleEinstellungenOberflcheNetzweit");
		benutzerAtg = model
				.getAttributeGroup("atg.benutzerEinstellungenOberflcheNetzweit");
		sollAsp = model.getAspect("asp.parameterSoll");
		vorgabeAsp = model.getAspect("asp.parameterVorgabe");

		dav.subscribeReceiver(this, oberflaechenObject, new DataDescription(
				netzAtg, sollAsp), ReceiveOptions.normal(), ReceiverRole
				.receiver());

		dav.subscribeReceiver(this, oberflaechenObject, new DataDescription(
				benutzerAtg, sollAsp), ReceiveOptions.normal(), ReceiverRole
				.receiver());

		try {
			dav.subscribeSender(this, oberflaechenObject, new DataDescription(
					netzAtg, vorgabeAsp), SenderRole.sender());
		} catch (final OneSubscriptionPerSendData e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			dav.subscribeSender(this, oberflaechenObject, new DataDescription(
					benutzerAtg, vorgabeAsp), SenderRole.sender());
		} catch (final OneSubscriptionPerSendData e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void dataRequest(final SystemObject arg0,
			final DataDescription arg1, final byte arg2) {
		// TODO Auto-generated method stub

	}

	public void disconnect() {
		// TODO Auto-generated method stub
	}

	@Override
	public boolean isRequestSupported(final SystemObject arg0,
			final DataDescription arg1) {
		// TODO Auto-generated method stub
		return false;
	}

	public synchronized void saveBenutzerEinstellungsSpeicher(final String pid,
			final String data, final UrlasserInfo info)
			throws SendSubscriptionNotConfirmed {

		final SystemObject benutzerToSave = model.getObject(pid);
		if (benutzerToSave == null) {
			DavConnector.LOGGER.error("Der Benutzer mit der PID: " + pid
					+ " existiert nicht in der aktuellen Konfiguration!");
			return;
		}

		final ResultData benutzerDaten = dav.getData(oberflaechenObject,
				new DataDescription(benutzerAtg, sollAsp), 0);

		Data neueDaten;
		if ((benutzerDaten != null) && (benutzerDaten.getData() != null)) {
			neueDaten = benutzerDaten.getData().createModifiableCopy();
		} else {
			neueDaten = dav.createData(benutzerAtg);
		}

		final Data.Array benutzerArray = neueDaten
				.getArray("Benutzereinstellungen");

		int dataIdx;
		for (dataIdx = 0; dataIdx < benutzerArray.getLength(); dataIdx++) {
			final SystemObject benutzer = benutzerArray.getItem(dataIdx)
					.getReferenceValue("benutzer").getSystemObject();
			if (benutzerToSave.equals(benutzer)) {
				break;
			}
		}

		if (dataIdx >= benutzerArray.getLength()) {
			benutzerArray.setLength(dataIdx + 1);
		}

		benutzerArray.getItem(dataIdx).getReferenceValue("benutzer")
				.setSystemObject(benutzerToSave);
		final int lines = (data.length() / DavConnector.MAX_LINE_LENGTH) + 1;

		final TextArray textArray = benutzerArray.getItem(dataIdx)
				.getTextArray("Voreinstellungen");
		textArray.setLength(lines);
		for (int line = 0; line < lines; line++) {
			final int start = line * DavConnector.MAX_LINE_LENGTH;
			final int ende = Math.min(
					((line + 1) * DavConnector.MAX_LINE_LENGTH), data.length());
			final String substring = data.substring(start, ende);
			textArray.getTextValue(line).setText(substring);
		}

		if (info != null) {
			final Data urlasserDaten = neueDaten.getItem("Urlasser");
			urlasserDaten.getReferenceValue("BenutzerReferenz")
					.setSystemObject(info.getBenutzer());
			urlasserDaten.getTextValue("Ursache").setText(info.getUrsache());
			urlasserDaten.getTextValue("Veranlasser").setText(
					info.getVeranlasser());
		}

		final ResultData resultData = new ResultData(oberflaechenObject,
				new DataDescription(benutzerAtg, vorgabeAsp), dav.getTime(),
				neueDaten);
		dav.sendData(resultData);
	}

	public synchronized void saveSystemEinstellungsSpeicher(final String data,
			final UrlasserInfo info) throws SendSubscriptionNotConfirmed {

		final Data neueDaten = dav.createData(netzAtg);
		final TextArray textArray = neueDaten.getItem(
				"BenutzereinstellungenGlobal").getTextArray("Voreinstellungen");
		final int lines = (data.length() / DavConnector.MAX_LINE_LENGTH) + 1;
		textArray.setLength(lines);
		for (int line = 0; line < lines; line++) {
			final int start = line * DavConnector.MAX_LINE_LENGTH;
			final int ende = Math.min(
					((line + 1) * DavConnector.MAX_LINE_LENGTH), data.length());
			final String substring = data.substring(start, ende);
			textArray.getTextValue(line).setText(substring);
		}

		if (info != null) {
			final Data urlasserDaten = neueDaten.getItem("Urlasser");
			urlasserDaten.getReferenceValue("BenutzerReferenz")
					.setSystemObject(info.getBenutzer());
			urlasserDaten.getTextValue("Ursache").setText(info.getUrsache());
			urlasserDaten.getTextValue("Veranlasser").setText(
					info.getVeranlasser());
		}

		final ResultData resultData = new ResultData(oberflaechenObject,
				new DataDescription(netzAtg, vorgabeAsp), dav.getTime(),
				neueDaten);
		dav.sendData(resultData);
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData result : results) {
			final Data daten = result.getData();
			if (daten == null) {
				continue;
			}

			if (result.getDataDescription().getAttributeGroup().equals(netzAtg)) {
				try {
					final SAXParserFactory factory = SAXParserFactory
							.newInstance();
					final SAXParser parser = factory.newSAXParser();

					final StringBuilder strBuffer = new StringBuilder();
					final TextArray textArray = daten.getItem(
							"BenutzereinstellungenGlobal").getTextArray(
							"Voreinstellungen");
					for (final TextValue textValue : textArray.getTextValues()) {
						strBuffer.append(textValue.getText());
					}

					if (strBuffer.length() == 0) {
						DavConnector.LOGGER.fine("Es wurden keine globalen Benutzereinstellungen gefunden");
					} else {
						parser.parse(
								new InputSource(new StringReader(strBuffer
										.toString())),
								new EinstellungenXmlHandler(
										speichermanager,
										new SpeicherKey(
												EinstellungOwnerType.SYSTEM,
												EinstellungLocation.NETZWERKWEIT)));
					}
				} catch (final ParserConfigurationException e) {
					DavConnector.LOGGER.error(e.getLocalizedMessage());
				} catch (final SAXException e) {
					DavConnector.LOGGER.error(e.getLocalizedMessage());
				} catch (final IOException e) {
					DavConnector.LOGGER.error(e.getLocalizedMessage());
				}
			} else if (result.getDataDescription().getAttributeGroup()
					.equals(benutzerAtg)) {
				try {
					final SAXParserFactory factory = SAXParserFactory
							.newInstance();
					final SAXParser parser = factory.newSAXParser();

					final Data.Array benutzerArray = daten
							.getArray("Benutzereinstellungen");

					for (int idx = 0; idx < benutzerArray.getLength(); idx++) {
						final StringBuilder strBuffer = new StringBuilder();
						final SystemObject benutzer = benutzerArray
								.getItem(idx).getReferenceValue("benutzer")
								.getSystemObject();
						if (benutzer == null) {
							DavConnector.LOGGER
									.warning("Der Parameterdatensatz der Benutzereinstellungen enthlt an Position "
											+ idx
											+ " keine gltige Benutzerreferenz");
							continue;
						}
						final TextArray textArray = benutzerArray.getItem(idx)
								.getTextArray("Voreinstellungen");
						for (final TextValue textValue : textArray
								.getTextValues()) {
							strBuffer.append(textValue.getText());
						}

						try {
							final SpeicherKey key = new SpeicherKey(
									EinstellungOwnerType.BENUTZER,
									EinstellungLocation.NETZWERKWEIT,
									benutzer.getPid());
							parser.parse(new InputSource(new StringReader(
									strBuffer.toString())),
									new EinstellungenXmlHandler(
											speichermanager, key));
						} catch (final SAXException e) {
							DavConnector.LOGGER.error(e.getLocalizedMessage());
						} catch (final IOException e) {
							DavConnector.LOGGER.error(e.getLocalizedMessage());
						}
					}
				} catch (final ParserConfigurationException e) {
					DavConnector.LOGGER.error(e.getLocalizedMessage());
				} catch (final SAXException e) {
					DavConnector.LOGGER.error(e.getLocalizedMessage());
				}
			}
		}
	}
}
