/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */
package de.bsvrz.buv.rw.rw.handler;

import java.io.File;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.rw.rw.ui.bildschirmfangen.BildschirmBereichAuswahlDialog;
import de.bsvrz.buv.rw.rw.ui.bildschirmfangen.BildschirmBereichAuswahlDialog.BildschirmSicherungsBereich;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Handler zum Erzeugen einer Bildschirmkopie vom gesamten Bildschirnm oder dem
 * Rahmenwerk-Fenster.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: HardCopyHandler.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class HardCopyHandler extends AbstractHandler {

	private static final Debug LOGGER = Debug.getLogger();

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {
		final BildschirmBereichAuswahlDialog dialog = new BildschirmBereichAuswahlDialog(
				PlatformUI.getWorkbench().getDisplay().getActiveShell());

		if (dialog.open() == Window.OK) {
			final BildschirmSicherungsBereich bereichsWahl = dialog
					.getSicherungsBereich();

			final UIJob job = new UIJob(PlatformUI.getWorkbench().getDisplay(),
					"Hardcopy") {

				@Override
				public IStatus runInUIThread(final IProgressMonitor monitor) {
					if (bereichsWahl != null) {
						switch (bereichsWahl) {
						case fenster:
							aktionFangeInhaltAktuellesFenster();
							break;
						case bildschirm:
							aktionFangeInhaltGesamterBildschirm();
							break;
						default:
							break;
						}
					}

					return Status.OK_STATUS;
				}
			};
			job.schedule(1000);
		}

		return null;
	}

	/**
	 * Erzeugt den Screenshot des aktuellen Fensters.
	 */
	public void aktionFangeInhaltAktuellesFenster() {
		final Rectangle bereich = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getShell().getBounds();
		aktionBereichFangen(bereich);
	}

	/**
	 * Erzeugt einen Screenshot des gesamten Bildschirmbereiches.
	 */
	public void aktionFangeInhaltGesamterBildschirm() {
		final Rectangle bereich = PlatformUI.getWorkbench().getDisplay()
				.getBounds();
		aktionBereichFangen(bereich);
	}

	/**
	 * Liefert den Pfad, unter dem die Datei gespeichert werden soll.
	 * 
	 * @return Pfad zur Datei
	 */
	private String speichernUnterDialog() {
		final FileDialog dialog = new FileDialog(PlatformUI.getWorkbench()
				.getDisplay().getActiveShell(), SWT.SAVE);

		dialog.setFilterNames(new String[] { "Portable Network Graphics" });
		dialog.setFilterExtensions(new String[] { "*.png" });
		final String pfad = dialog.open();
		return pfad;
	}

	/**
	 * "Fngt" den durch das {@link Rectangle} angegebenen Bereichs des
	 * Bildschirms und speichert ihn in einer PNG-Datei ab.
	 * 
	 * @param bereich
	 *            Zu "fangender" Bereich.
	 */
	private void aktionBereichFangen(final Rectangle bereich) {
		final int x = bereich.x;
		final int y = bereich.y;
		final int breite = bereich.width;
		final int hoehe = bereich.height;

		final GC gc = new GC(PlatformUI.getWorkbench().getDisplay());
		final Image image = new Image(PlatformUI.getWorkbench().getDisplay(),
				new Rectangle(x, y, breite, hoehe));
		gc.copyArea(image, x, y);

		final ImageLoader loader = new ImageLoader();
		loader.data = new ImageData[] { image.getImageData() };

		final String pfad = speichernUnterDialog();

		if (pfad != null) {
			// loader.save("C:\\shot.png", SWT.IMAGE_PNG);

			boolean ueberschreiben = true;
			final File file = new File(pfad);
			if (file.exists()) {
				final MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
						.getDisplay().getActiveShell(), SWT.ICON_WARNING
						| SWT.YES | SWT.NO);
				mb.setText("Datei existiert bereits. berschreiben?");
				mb.setMessage("Die Datei\""
						+ pfad
						+ "\" existiert bereits.\nSoll diese berschrieben werden?");
				final int dlgSel = mb.open();
				if (dlgSel == SWT.YES) {
					ueberschreiben = true;
				} else if (dlgSel == SWT.NO) {
					ueberschreiben = false;
				}
			}

			if (ueberschreiben) {
				loader.save(pfad, SWT.IMAGE_PNG);
			}
		}
	}

	/**
	 * Liefert ein Vorschaufenster, in dem ein {@link Image} angezeigt werden
	 * kann.
	 * 
	 * @param img
	 *            Anzuzeigendes {@link Image}
	 */
	public void vorschauScreenShot(final Image img) {
		final Image image = img;

		final Shell popup = new Shell(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.SHELL_TRIM);
		popup.setLayout(new FillLayout());
		popup.setText("Image");
		popup.setBounds(50, 50, image.getBounds().width,
				image.getBounds().height);
		popup.addListener(SWT.Close, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				image.dispose();
			}
		});

		final ScrolledComposite sc = new ScrolledComposite(popup, SWT.V_SCROLL
				| SWT.H_SCROLL);
		final Canvas canvas = new Canvas(sc, SWT.NONE);
		sc.setContent(canvas);
		canvas.setBounds(PlatformUI.getWorkbench().getDisplay().getBounds());
		canvas.addPaintListener(new PaintListener() {
			@Override
			public void paintControl(final PaintEvent e) {
				e.gc.drawImage(image, 0, 0);
			}
		});
		popup.open();
	}
}
