/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */
package de.bsvrz.buv.rw.rw.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

import de.bsvrz.buv.rw.rw.menu.AbstractMenueLeiste;
import de.bsvrz.buv.rw.rw.menu.LadeSpeicherDialogMenu;
import de.bsvrz.buv.rw.rw.menu.MenueLeiste;
import de.bsvrz.buv.rw.rw.menu.editoren.MenueEditor;
import de.bsvrz.buv.rw.rw.menu.editoren.MenueEditorInput;
import de.bsvrz.buv.rw.rw.menu.editoren.PerspektiveMenuEditor;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Handler zur ffnen eines Meneditors.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: OpenMenuEditor.java 49030 2014-02-28 15:19:54Z peuker $
 */
public class OpenMenuEditor extends AbstractHandler {

	private static final Debug LOGGER = Debug.getLogger();

	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {

		final AbstractMenueLeiste leiste = new MenueLeiste();
		boolean loadMenue = true;

		final Object trigger = event.getTrigger();
		if (trigger instanceof Event) {
			if ((((Event) trigger).stateMask & SWT.MODIFIER_MASK) != 0) {
				loadMenue = false;
			}
		}

		final String perspectiveID = PerspektiveMenuEditor.PERSPECTIVE_ID;

		try {
			final IWorkbenchWindow window = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow();
			PlatformUI.getWorkbench().showPerspective(perspectiveID, window);

			final MenueEditorInput ei = new MenueEditorInput(leiste);
			final IWorkbenchPage page = window.getActivePage();
			try {

				if (loadMenue) {
					final LadeSpeicherDialogMenu dialog = new LadeSpeicherDialogMenu(
							window.getShell(), SWT.OPEN);
					if (dialog.open() == Window.OK) {
						final MenueEditor editor = (MenueEditor) page
								.openEditor(ei, MenueEditor.ID);
						editor.menueLeisteLaden(dialog.getSpeicherKey());
					}
				} else {
					page.openEditor(ei, MenueEditor.ID);
				}

			} catch (final PartInitException pie) {
				OpenMenuEditor.LOGGER
						.error("Der Menleisteneditor konnte nicht "
								+ "geffnet werden\n\t=> " + pie.toString());
			}

		} catch (final WorkbenchException wbe) {
			OpenMenuEditor.LOGGER
					.error("Die Men- und Symbolleisteneditorperspektive "
							+ "konnte nicht geffnet werden\n\t=> "
							+ wbe.toString());
		}

		return null;
	}
}
