/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.interfaces.darstellungsobjekte;

/**
 * Interface fr den Zugriff auf ZoomInformationen von DarstellungsObjektTypen.<br>
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/12/02 09:21:02 $ / ($Author: HofmannJ
 *          $)
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 */
public interface IDarstellungsObjektTypZoomInfo {
	/**
	 * Holen der ZoomStufe in Prozent, fr die die Ausdehnungsinformationen des
	 * DarstellungsObjektes {@link #getAusdehnungX()} und
	 * {@link #getAusdehnungY()} gelten.<br>
	 * 
	 * @return ZoomStufe in Prozent.
	 */
	int getZoomStufeFuerAusdehnung();

	/**
	 * Holen der Ausdehnung des DarstellungsObjektes in X-Richtung, abhngig von
	 * der bei der Bestimmung bergebenen ZoomStufe, in Pixel.<br>
	 * 
	 * @return Ausdehnung in X-Richtung in Pixel.
	 */
	int getAusdehnungX();

	/**
	 * Holen der Ausdehnung des DarstellungsObjektes in Y-Richtung, abhngig von
	 * der bei der Bestimmung bergebenen ZoomStufe, in Pixel.<br>
	 * 
	 * @return Ausdehnung in Y-Richtung in Pixel.
	 */
	int getAusdehnungY();

	/**
	 * Holen des ZoomVerhalten des DarstellungsObjektes.<br>
	 * 
	 * @return ZoomVerhalten des DarstellungsObjektes.
	 */
	DarstellungsObjektZoomVerhalten getZoomVerhalten();
}
