/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.internal;

import de.bsvrz.buv.rw.basislib.einstellungen.Einstellungen;

/**
 * Komponente zur Bereitstellung zustzlicher Services innerhalb des
 * Rahmenwerk-Plugins.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: InternalServices.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class InternalServices {

	private static InternalServices instance;
	private Einstellungen einstellungen;

	/**
	 * liefert die globale Instanz der Komponente.
	 * 
	 * @return die Instanz
	 */
	public static InternalServices getInstance() {
		return InternalServices.instance;
	}

	/**
	 * bindet den bergebenen Service {@link Einstellungen} an die Komponente.
	 * 
	 * @param service
	 *            der Service
	 */
	protected void bindEinstellungen(final Einstellungen service) {
		this.einstellungen = service;
	}

	/**
	 * entfernt den bergebenen Service {@link Einstellungen} von der
	 * Komponente.
	 * 
	 * @param service
	 *            der Service
	 */
	protected void unbindEinstellungen(final Einstellungen service) {
		this.einstellungen = null;
	}

	/** aktiviert die Komponente. */
	protected void activate() {
		InternalServices.instance = this;
	}

	/** deaktiviert die Komponente. */
	protected void deactivate() {
		InternalServices.instance = null;
	}

	/**
	 * liefert den gebundenen Service {@link Einstellungen} oder
	 * <code>null</code>, wenn keiner verfgbar ist.
	 * 
	 * @return den Service oder <code>null</code>
	 */
	public Einstellungen getEinstellungen() {
		return einstellungen;
	}
}
