/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.kommandozeilenparameter;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.core.runtime.Platform;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;

/**
 * Diese Klasse liefert alle Kommandozeilenparameter, die an das Rahmenwerk zum
 * Start bergeben wurden. Sie wurde im Haupt Plug-In des Rahmenwerkes
 * realisiert.
 * 
 * @author HofmannJ
 * @deprecated die Kommandozeilenparameter knnen ber die Schnittstelle
 *             {@link Rahmenwerk#getStartParameter()} ermittelt werden
 */
@Deprecated
public final class Kommandozeilenparameter {
	/**
	 * Speichert die Kommandozeilenargumente in der Weise, wie sie von der
	 * {@link Platform#getCommandLineArgs()} geliefert werden.
	 */
	private final String[] commandLineArgs;

	private final TreeMap<String, KmdParameter> hmParameter = new TreeMap<String, KmdParameter>();

	/**
	 * Privater Konstruktor.
	 */
	private Kommandozeilenparameter() {
		commandLineArgs = Platform.getCommandLineArgs();

		/*
		 * Auswertung
		 */
		for (int i = 0; i < commandLineArgs.length; i++) {
			final String par = commandLineArgs[i];
			if (par.contains("=")) {
				/*
				 * Parameter mit =
				 */
				final String[] split = par.split("=");
				if (split.length == 2) {
					if ((split[0] != null) && (split[1] != null)
							&& (split[0].length() > 0)
							&& (split[1].length() > 0)) {
						final KmdParameter kmdParameter = new KmdParameter(
								split[0], split[1]);
						hmParameter.put(kmdParameter.getSchluessel(),
								kmdParameter);
					}
				} else if (split.length == 1) {
					final KmdParameter kmdParameter = new KmdParameter(
							split[0], null);
					hmParameter.put(kmdParameter.getSchluessel(), kmdParameter);
				}
			} else {
				/*
				 * Parameter mit getrennten Argumenten
				 */
				final String parKey = par;
				String parValue = "";
				if ((i + 1) < commandLineArgs.length) {
					if (!commandLineArgs[i + 1].contains("=")) {
						// TODO berprfen, ob das so korrekt ist
						i = i + 1;
						parValue = commandLineArgs[i];
					}
				}
				final KmdParameter kmdParameter = new KmdParameter(parKey,
						parValue);
				hmParameter.put(kmdParameter.getSchluessel(), kmdParameter);
			}
		}

		setzeDefaultWerte();
	}

	private void setzeDefaultWerte() {
		if (!hmParameter.containsKey("-berechtigungsKonzept")) {
			final KmdParameter par = new KmdParameter("-berechtigungsKonzept",
					"false");
			hmParameter.put(par.getSchluessel(), par);
		}
	}

	/**
	 * Private innere Klasse. Speichert die Singleton-Instanz von
	 * {@link Kommandozeilenparameter}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen {
		private static final Kommandozeilenparameter INSTANZ = new Kommandozeilenparameter();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link Kommandozeilenparameter}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link Kommandozeilenparameter}
	 */
	public static Kommandozeilenparameter getInstanz() {
		return Innen.INSTANZ;
	}

	/**
	 * Liefert eine Liste von allen vorhandenen Kommandozeilenparametern zurck.
	 * 
	 * @return Liste von allen vorhandenen Kommandozeilenparametern
	 */
	public LinkedList<KmdParameter> getKommandoZeilenParameter() {
		final LinkedList<KmdParameter> re = new LinkedList<KmdParameter>();
		final Iterator<Entry<String, KmdParameter>> it = hmParameter.entrySet()
				.iterator();
		while (it.hasNext()) {
			final Entry<String, KmdParameter> entry = it.next();
			re.add(entry.getValue());
		}

		return re;
	}

	/**
	 * Liefert den {@link KmdParameter} zu einem bestimmten Schlssel. Existiert
	 * zu diesem Schlssel kein {@link KmdParameter}, so wird <code>null</code>
	 * zurckgeliefert werden.
	 * 
	 * @param schluessel
	 *            Schlssel, zu dem der zugehrige {@link KmdParameter} gesucht
	 *            werden soll.
	 * @return {@link KmdParameter} sofern zum angegebenen Schlssel vorhanden,
	 *         sonst <code>null</code>.
	 */
	public KmdParameter getKommandoZeilenParameter(final String schluessel) {
		KmdParameter re = null;
		if (hmParameter.containsKey(schluessel)) {
			re = hmParameter.get(schluessel);
		}
		return re;
	}

	/**
	 * Setzt einen Kommandozeilenparameter. <br>
	 * <b>Achtung:</b> Kommandozeilenparameter sollten ber die Laufzeitumgebung
	 * gesetzt werden!
	 * 
	 * @param kmdParameter
	 *            Der zu setzende Kommandozeilenparameter.
	 */
	public void setKommandoZeilenParameter(final KmdParameter kmdParameter) {
		if (kmdParameter != null) {
			hmParameter.put(kmdParameter.getSchluessel(), kmdParameter);
		}
	}
}
