/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.menu;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.services.IServiceLocator;

/**
 * Allgemeine Definition fr eine Action zur Einbindung in eine Toolbar oder das
 * Men des Rahmenwerks. Die Actions werden aus den (veralteten) ExtensionPoints
 * "ActionSet" eingelesen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwToolbarAction.java 46953 2013-11-05 10:05:57Z peuker $
 */
class RwToolbarAction extends RwToolbarElement {

	/**
	 * Stellvertreter-Aktion, die die durch die ExtensionPoints definierten
	 * Aktionen letztendlich umsetzt.
	 */
	private static class DelegatorAction extends Action {

		private final IWorkbenchWindowActionDelegate delegate;

		public DelegatorAction(final String name, final int style,
				final IWorkbenchWindowActionDelegate delegate) {
			super(name, style);
			this.delegate = delegate;
		}

		@Override
		public void run() {
			delegate.run(this);
		}
	}

	/**
	 * Konstruktor, erzeugt eine Elementdefinition auf Basis der bergebenen
	 * Konfigurationsdaten aus der entsprechenden Plugin-Definition.
	 * 
	 * @param configuration
	 *            die Konfigurationsdaten
	 */
	public RwToolbarAction(final IConfigurationElement configuration) {
		super(configuration);
	}

	@Override
	public IContributionItem createContributionItem(
			final IServiceLocator locator, final MenueAction element)
			throws CoreException {
		final Object extension = getConfiguration().createExecutableExtension(
				"class");
		// if (extension instanceof IAction) {
		// ((IAction) extension).setText(element.getVisibleName());
		// return new ActionContributionItem((IAction) extension);
		// }

		if (extension instanceof IWorkbenchWindowActionDelegate) {
			if (!(locator instanceof IWorkbenchWindow)) {
				throw new CoreException(
						new Status(
								IStatus.ERROR,
								getPluginId(),
								"Eine WorkbenchWindowActionDelegate kann nur in einem WorkbenchWindow eingebunden werden! - Locator ist vom Typ: "
										+ locator.getClass()));
			}

			final IConfigurationElement conf = getConfiguration();
			int style = IAction.AS_PUSH_BUTTON;
			final String styleAttr = conf.getAttribute("style");
			if ("toggle".equals(styleAttr)) {
				style = IAction.AS_CHECK_BOX;
			}

			((IWorkbenchWindowActionDelegate) extension)
					.init((IWorkbenchWindow) locator);
			final DelegatorAction delegatorAction = new DelegatorAction(
					element.getVisibleName(), style,
					(IWorkbenchWindowActionDelegate) extension);

			final String iconPath = conf.getAttribute("icon");
			if (iconPath != null) {
				delegatorAction.setImageDescriptor(AbstractUIPlugin
						.imageDescriptorFromPlugin(getPluginId(), iconPath));
			}

			return new ActionContributionItem(delegatorAction);
		}

		throw new CoreException(new Status(IStatus.ERROR, getPluginId(),
				"Extensions vom Typ: " + extension.getClass()
						+ " werden nicht untersttzt!"));
	}
}
