/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.menu.AbstractMenueLeiste;

/**
 * Editor-Input fr den Meneditor.
 * 
 * Eine Instanz der Klasse kapslet eine {@link AbstractMenueLeiste}, die aber
 * auch innerhalb des Meneditors ber die entsprechenden Kommandos ersetzt
 * werden kann.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: MenueEditorInput.java 45776 2013-08-30 10:36:23Z peuker $
 */
public class MenueEditorInput implements IEditorInput {
	/**
	 * Struktur der Men- bzw. Symbolleiste
	 */
	private final AbstractMenueLeiste menueLeiste;

	/**
	 * Konstruktor, bernimmt eine Instanz eine Menleiste(ndefinition).
	 * 
	 * @param menuLeiste
	 *            die Menleiste die initial als Inout an den Editor bergeben
	 *            wird
	 */
	public MenueEditorInput(final AbstractMenueLeiste menuLeiste) {
		this.menueLeiste = menuLeiste;
	}

	@Override
	public boolean exists() {
		return false;
	}

	@Override
	public ImageDescriptor getImageDescriptor() {
		return RahmenwerkActivator.getDefault().getImageDescriptor(
				"icons/obj16/menu.gif");
	}

	@Override
	public String getName() {
		return menueLeiste.getName();
	}

	@Override
	public IPersistableElement getPersistable() {
		return null;
	}

	@Override
	public String getToolTipText() {
		return "Struktur einer Menleiste";
	}

	@Override
	public Object getAdapter(final Class adapter) {
		return null;
	}

	/**
	 * Liefert die Struktur der Mens bzw. der Symbolleiste.
	 * 
	 * @return die Menleiste
	 */
	public AbstractMenueLeiste getMenueLeiste() {
		return menueLeiste;
	}

}
