/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.menu.editoren;

import java.util.Collection;

import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.menu.PluginCollection;
import de.bsvrz.buv.rw.rw.menu.RwPluginElemente;
import de.bsvrz.buv.rw.rw.menu.RwToolbarElement;

/**
 * View zur Darstellung aller registrierten Menu-Elemente des Rahmenwerks.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: ViewMenueEditorElemente.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class ViewMenueEditorElemente extends ViewPart {

	/** ID der View. */
	public static final String ID = "de.bsvrz.buv.rw.rw.view.menueditor.elemente";

	private String filterPattern;

	private TreeViewer treeViewer;

	@Override
	public void createPartControl(final Composite parent) {

		final Composite panel = new Composite(parent, SWT.NONE);
		panel.setLayout(new GridLayout(2, false));
		GridDataFactory.fillDefaults().grab(true, true).applyTo(panel);

		new Label(panel, SWT.NONE).setText("Filter: ");
		final Text filterText = new Text(panel, SWT.BORDER);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(filterText);

		filterText.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(final ModifyEvent e) {
				filterPattern = filterText.getText().trim().toLowerCase();
				if (treeViewer != null) {
					treeViewer.refresh();
				}
			}
		});

		createTreeViewerWithDND(panel);
		GridDataFactory.fillDefaults().grab(true, true).span(2, 1)
				.applyTo(treeViewer.getControl());

		treeViewer.setInput(new PluginCollection("Plug-Ins"));

		PlatformUI.getWorkbench().getHelpSystem()
				.setHelp(panel, "de.bsvrz.buv.rw.basislib.menuleisteneditor");
	}

	private void createTreeViewerWithDND(final Composite parent) {

		treeViewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL);
		ColumnViewerToolTipSupport.enableFor(treeViewer);
		treeViewer.setContentProvider(new CommandSelektorContentProvider());
		treeViewer.setLabelProvider(new CommandSelektorLabelProvider());
		treeViewer.setSorter(new ViewerSorter() {
			@Override
			public int compare(final Viewer viewer, final Object e1,
					final Object e2) {
				final String left = e1.toString().replaceAll("&", "");
				final String right = e2.toString().replaceAll("&", "");
				return left.compareTo(right);
			}
		});

		final ViewerFilter filter = new ViewerFilter() {
			@Override
			public boolean select(final Viewer viewer,
					final Object parentElement, final Object element) {

				if ((filterPattern != null) && (!filterPattern.isEmpty())) {
					if (element instanceof RwToolbarElement) {
						return element.toString().toLowerCase()
								.contains(filterPattern);
					} else if (element instanceof RwPluginElemente) {
						final Collection<RwToolbarElement> actions = ((RwPluginElemente) element)
								.getElemente();
						for (final RwToolbarElement action : actions) {
							if (action.toString().toLowerCase()
									.contains(filterPattern)) {
								treeViewer.setExpandedState(element, true);
								return true;
							}
						}
						return false;
					}
				}
				return true;
			}
		};

		treeViewer.setFilters(new ViewerFilter[] { filter });

		final int ops = DND.DROP_MOVE;
		final Transfer[] transfers = new Transfer[] { LocalSelectionTransfer
				.getTransfer() };
		treeViewer.addDragSupport(ops, transfers,
				new LocalViewerSelectionDragAdapter(treeViewer));
	}

	@Override
	@Focus
	public void setFocus() {
		if ((treeViewer != null) && (!treeViewer.getControl().isDisposed())) {
			treeViewer.getControl().setFocus();
		}
	}
}
