/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.preferences;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;

/**
 * Composite zum Vornehmen der Seiteneinstellungen.
 * 
 * TODO Benennung berdenken, eventuell andere Struktur fr Prferenzseiten?
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: Zusammenstellung.java 45420 2013-08-09 12:12:12Z peuker $
 */
public class Zusammenstellung extends Composite {

	private Composite compositeLinks;

	private Composite compositeRechts;

	private Group gruppeKopfzeile;

	private Group gruppeFusszeile;

	private Group groupPlatzhalter;

	private Label labelKopfzeileLinks;

	private Label labelKopfzeileMitte;

	private Label labelKopfzeileRechts;

	private Label labelFusszeileLinks;

	private Label labelFusszeileMitte;

	private Label labelFusszeileRechts;

	private List listPlatzhalter;

	private ListViewer listViewer;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Oben Links.
	 */
	private TextFeld textFeldKopfzeileLinks;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Oben Mitte.
	 */
	private TextFeld textFeldKopfzeileMitte;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Oben Rechts.
	 */
	private TextFeld textFeldKopfzeileRechts;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Unten Links.
	 */
	private TextFeld textFeldFusszeileLinks;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Unten Mitte.
	 */
	private TextFeld textFeldFusszeileMitte;

	/**
	 * {@link TextFeld} zur Erfassung der Kopfzeilen Einstellungen Unten Rechts.
	 */
	private TextFeld textFeldFusszeileRechts;

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            Eltern-{@link Composite}
	 * @param style
	 *            Stil
	 */
	public Zusammenstellung(final Composite parent, final int style) {
		super(parent, style);
		init();
	}

	/**
	 * Fhrt die Initialisierung durch.
	 */
	private void init() {
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		erzeugeCompositeLinks();
		this.setLayout(gridLayout);
		erzeugeCompositeRechts();
		setSize(new Point(517, 268));

		final String[] platzhalter = new String[] { "Seite", "Seitenanzahl",
				"Datum", "Uhrzeit", "Titel" };
		final ListeInhaltsAnbieter contentProvider = new ListeInhaltsAnbieter();
		listViewer.setContentProvider(contentProvider);
		listViewer.setInput(platzhalter);
	}

	/**
	 * Liefert in einem Array vom Typ String die getroffenen Einstellungen von
	 * Kopf- und Fusszeile.<br>
	 * <br>
	 * Die nachfolgende Tabelle gibt Aufschlu darber, welche Felder des
	 * Rckgabe Arrays welchem Wert entsprechen:<br>
	 * <table>
	 * <tr>
	 * <th>Feldindex:</th>
	 * <th>Wert:</th>
	 * </tr>
	 * <tr>
	 * <td>0</td>
	 * <td>Kopfzeile Links</td>
	 * </tr>
	 * <tr>
	 * <td>1</td>
	 * <td>Kopfzeile Mitte</td>
	 * </tr>
	 * <tr>
	 * <td>2</td>
	 * <td>Kopfzeile Rechts</td>
	 * </tr>
	 * <tr>
	 * <td>3</td>
	 * <td>Fuzeile Links</td>
	 * </tr>
	 * <tr>
	 * <td>4</td>
	 * <td>Fuzeile Mitte</td>
	 * </tr>
	 * <tr>
	 * <td>5</td>
	 * <td>Fuzeile Rechts</td>
	 * </tr>
	 * </table>
	 * 
	 * @return Array vom Typ {@link String} mit den Feldern der Kopf- und
	 *         Fuzeile
	 */
	public String[] getKopfFussZeilen() {
		final String[] re = new String[6];

		re[0] = textFeldKopfzeileLinks.getText();
		re[1] = textFeldKopfzeileMitte.getText();
		re[2] = textFeldKopfzeileRechts.getText();
		re[3] = textFeldFusszeileLinks.getText();
		re[4] = textFeldFusszeileMitte.getText();
		re[5] = textFeldFusszeileRechts.getText();

		return re;
	}

	/**
	 * Setzt die Felder von Kopf- und Fuzeile an Hand eines Arrays vom Typ
	 * {@link String}.<br>
	 * Das Array muss dabei wie folgt aufgebaut sein:<br>
	 * <table>
	 * <tr>
	 * <th>Feldindex:</th>
	 * <th>Wert:</th>
	 * </tr>
	 * <tr>
	 * <td>0</td>
	 * <td>Kopfzeile Links</td>
	 * </tr>
	 * <tr>
	 * <td>1</td>
	 * <td>Kopfzeile Mitte</td>
	 * </tr>
	 * <tr>
	 * <td>2</td>
	 * <td>Kopfzeile Rechts</td>
	 * </tr>
	 * <tr>
	 * <td>3</td>
	 * <td>Fuzeile Links</td>
	 * </tr>
	 * <tr>
	 * <td>4</td>
	 * <td>Fuzeile Mitte</td>
	 * </tr>
	 * <tr>
	 * <td>5</td>
	 * <td>Fuzeile Rechts</td>
	 * </tr>
	 * </table>
	 * 
	 * Wirft eine IllegalArgumentException, wenn das bergebene Array
	 * <code>null</code> ist oder die Anzahl der Felder des Arrays
	 * <code>!=6</code> ist.
	 * 
	 * @param kopfFussZeilen
	 *            Array mit den Inhalten von Kopf- und Fuzeile.
	 * 
	 */
	public void setKopfFussZeilen(final String[] kopfFussZeilen) {
		if ((kopfFussZeilen != null) && (kopfFussZeilen.length == 6)) {
			if (kopfFussZeilen[0] != null) {
				textFeldKopfzeileLinks.setText(kopfFussZeilen[0]);
			} else {
				textFeldKopfzeileLinks.setText("");
			}

			if (kopfFussZeilen[1] != null) {
				textFeldKopfzeileMitte.setText(kopfFussZeilen[1]);
			} else {
				textFeldKopfzeileMitte.setText("");
			}

			if (kopfFussZeilen[2] != null) {
				textFeldKopfzeileRechts.setText(kopfFussZeilen[2]);
			} else {
				textFeldKopfzeileRechts.setText("");
			}

			if (kopfFussZeilen[3] != null) {
				textFeldFusszeileLinks.setText(kopfFussZeilen[3]);
			} else {
				textFeldFusszeileLinks.setText("");
			}

			if (kopfFussZeilen[4] != null) {
				textFeldFusszeileMitte.setText(kopfFussZeilen[4]);
			} else {
				textFeldFusszeileMitte.setText("");
			}

			if (kopfFussZeilen[5] != null) {
				textFeldFusszeileRechts.setText(kopfFussZeilen[5]);
			} else {
				textFeldFusszeileRechts.setText("");
			}

		} else {
			throw new IllegalArgumentException(
					"Der bergebene Parameter fr Kopf- und Fuzeileneinstellungen "
							+ "entspricht nicht den Konventionen - er ist entweder NULL "
							+ "oder beinhaltet keine 6 Felder!");
		}
	}

	/**
	 * Initialisierung von compositeLinks.
	 * 
	 */
	private void erzeugeCompositeLinks() {
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		final GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 1;
		compositeLinks = new Composite(this, SWT.NONE);
		erzeugeGruppeKopfzeile();
		compositeLinks.setLayout(gridLayout1);
		compositeLinks.setLayoutData(gridData);
		erzeugeGruppeFusszeile();
	}

	/**
	 * Initialisierung von compositeRechts.
	 * 
	 */
	private void erzeugeCompositeRechts() {
		final GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.grabExcessHorizontalSpace = false;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		compositeRechts = new Composite(this, SWT.NONE);
		compositeRechts.setLayout(new GridLayout());
		compositeRechts.setLayoutData(gridData1);
		erzeugeGruppePlatzhalter();
	}

	/**
	 * Initialisierung von gruppeKopfzeile.
	 * 
	 */
	private void erzeugeGruppeKopfzeile() {
		final GridData gridDataLabelRechts = new GridData();
		gridDataLabelRechts.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelRechts.verticalAlignment = GridData.BEGINNING;
		final GridData gridDataLabelMitte = new GridData();
		gridDataLabelMitte.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelMitte.verticalAlignment = GridData.BEGINNING;
		final GridData gridDataLabelLinks = new GridData();
		gridDataLabelLinks.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelLinks.verticalAlignment = GridData.BEGINNING;
		final GridData gridDataAreaKopfzeileRechts = new GridData();
		gridDataAreaKopfzeileRechts.horizontalAlignment = GridData.FILL;
		gridDataAreaKopfzeileRechts.grabExcessHorizontalSpace = true;
		gridDataAreaKopfzeileRechts.grabExcessVerticalSpace = true;
		gridDataAreaKopfzeileRechts.verticalAlignment = GridData.FILL;
		final GridData gridDataTextFeldKopfzeileRechtes = new GridData();
		gridDataTextFeldKopfzeileRechtes.horizontalAlignment = GridData.FILL;
		gridDataTextFeldKopfzeileRechtes.grabExcessHorizontalSpace = true;
		gridDataTextFeldKopfzeileRechtes.grabExcessVerticalSpace = true;
		gridDataTextFeldKopfzeileRechtes.verticalAlignment = GridData.FILL;
		final GridData gridDataTextFeldKopfzeileLinks = new GridData();
		gridDataTextFeldKopfzeileLinks.horizontalAlignment = GridData.FILL;
		gridDataTextFeldKopfzeileLinks.grabExcessHorizontalSpace = true;
		gridDataTextFeldKopfzeileLinks.grabExcessVerticalSpace = true;
		gridDataTextFeldKopfzeileLinks.verticalAlignment = GridData.FILL;
		final GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 2;
		final GridData gridDataGruppeKopfzeile = new GridData();
		gridDataGruppeKopfzeile.horizontalAlignment = GridData.FILL;
		gridDataGruppeKopfzeile.grabExcessHorizontalSpace = true;
		gridDataGruppeKopfzeile.grabExcessVerticalSpace = true;
		gridDataGruppeKopfzeile.verticalAlignment = GridData.FILL;
		gruppeKopfzeile = new Group(compositeLinks, SWT.NONE);
		gruppeKopfzeile.setLayoutData(gridDataGruppeKopfzeile);
		gruppeKopfzeile.setLayout(gridLayout2);
		gruppeKopfzeile.setText("Kopfzeile");

		labelKopfzeileLinks = new Label(gruppeKopfzeile, SWT.NONE);
		labelKopfzeileLinks.setText("Links:");
		labelKopfzeileLinks.setLayoutData(gridDataLabelLinks);

		textFeldKopfzeileLinks = new TextFeld(gruppeKopfzeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldKopfzeileLinks.setLayoutData(gridDataTextFeldKopfzeileLinks);

		labelKopfzeileMitte = new Label(gruppeKopfzeile, SWT.NONE);
		labelKopfzeileMitte.setText("Mitte:");
		labelKopfzeileMitte.setLayoutData(gridDataLabelMitte);

		textFeldKopfzeileMitte = new TextFeld(gruppeKopfzeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldKopfzeileMitte.setLayoutData(gridDataTextFeldKopfzeileRechtes);

		labelKopfzeileRechts = new Label(gruppeKopfzeile, SWT.NONE);
		labelKopfzeileRechts.setText("Rechts:");
		labelKopfzeileRechts.setLayoutData(gridDataLabelRechts);

		textFeldKopfzeileRechts = new TextFeld(gruppeKopfzeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldKopfzeileRechts.setLayoutData(gridDataTextFeldKopfzeileRechtes);

	}

	/**
	 * Initialisierung von gruppeFusszeile.
	 * 
	 */
	private void erzeugeGruppeFusszeile() {

		final GridData gridDataLabelRechts = new GridData();
		gridDataLabelRechts.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelRechts.verticalAlignment = GridData.BEGINNING;
		final GridData gridDataAreaFusszeileRechts = new GridData();
		gridDataAreaFusszeileRechts.horizontalAlignment = GridData.FILL;
		gridDataAreaFusszeileRechts.grabExcessHorizontalSpace = true;
		gridDataAreaFusszeileRechts.grabExcessVerticalSpace = true;
		gridDataAreaFusszeileRechts.verticalAlignment = GridData.FILL;
		final GridData gridDataLabelMitte = new GridData();
		gridDataLabelMitte.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelMitte.verticalAlignment = GridData.BEGINNING;
		final GridData gridDataAreaFusszeileMitte = new GridData();
		gridDataAreaFusszeileMitte.horizontalAlignment = GridData.FILL;
		gridDataAreaFusszeileMitte.grabExcessHorizontalSpace = true;
		gridDataAreaFusszeileMitte.grabExcessVerticalSpace = true;
		gridDataAreaFusszeileMitte.verticalAlignment = GridData.FILL;
		final GridData gridDataLabelLinks = new GridData();
		gridDataLabelLinks.horizontalAlignment = GridData.BEGINNING;
		gridDataLabelLinks.verticalAlignment = GridData.BEGINNING;
		final GridData gridDatTextFeldFusszeileLinks = new GridData();
		gridDatTextFeldFusszeileLinks.horizontalAlignment = GridData.FILL;
		gridDatTextFeldFusszeileLinks.grabExcessHorizontalSpace = true;
		gridDatTextFeldFusszeileLinks.grabExcessVerticalSpace = true;
		gridDatTextFeldFusszeileLinks.verticalAlignment = GridData.FILL;
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		final GridData gridDataGruppeFusszeile = new GridData();
		gridDataGruppeFusszeile.horizontalAlignment = GridData.FILL;
		gridDataGruppeFusszeile.grabExcessHorizontalSpace = true;
		gridDataGruppeFusszeile.grabExcessVerticalSpace = true;
		gridDataGruppeFusszeile.verticalAlignment = GridData.FILL;
		gruppeFusszeile = new Group(compositeLinks, SWT.NONE);
		gruppeFusszeile.setLayoutData(gridDataGruppeFusszeile);
		gruppeFusszeile.setLayout(gridLayout);
		gruppeFusszeile.setText("Fuzeile");

		labelFusszeileLinks = new Label(gruppeFusszeile, SWT.NONE);
		labelFusszeileLinks.setText("Links:");
		labelFusszeileLinks.setLayoutData(gridDataLabelLinks);

		textFeldFusszeileLinks = new TextFeld(gruppeFusszeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldFusszeileLinks.setLayoutData(gridDatTextFeldFusszeileLinks);

		labelFusszeileMitte = new Label(gruppeFusszeile, SWT.NONE);
		labelFusszeileMitte.setText("Mitte:");
		labelFusszeileMitte.setLayoutData(gridDataLabelMitte);

		textFeldFusszeileMitte = new TextFeld(gruppeFusszeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldFusszeileMitte.setLayoutData(gridDataAreaFusszeileMitte);

		labelFusszeileRechts = new Label(gruppeFusszeile, SWT.NONE);
		labelFusszeileRechts.setText("Rechts:");
		labelFusszeileRechts.setLayoutData(gridDataLabelRechts);

		textFeldFusszeileRechts = new TextFeld(gruppeFusszeile, SWT.MULTI
				| SWT.WRAP | SWT.V_SCROLL);
		textFeldFusszeileRechts.setLayoutData(gridDataAreaFusszeileRechts);
	}

	/**
	 * Initialisierung von gruppePlatzhalter.
	 * 
	 */
	private void erzeugeGruppePlatzhalter() {
		final GridData gridData14 = new GridData();
		gridData14.horizontalAlignment = GridData.FILL;
		gridData14.grabExcessVerticalSpace = true;
		gridData14.grabExcessHorizontalSpace = true;
		gridData14.verticalAlignment = GridData.FILL;
		final GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.BEGINNING;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.verticalAlignment = GridData.FILL;
		groupPlatzhalter = new Group(compositeRechts, SWT.NONE);
		groupPlatzhalter.setLayout(new GridLayout());
		groupPlatzhalter.setLayoutData(gridData4);
		groupPlatzhalter.setText("Platzhalter");
		listPlatzhalter = new List(groupPlatzhalter, SWT.NONE);
		listPlatzhalter.setLayoutData(gridData14);

		listViewer = new ListViewer(listPlatzhalter);

		final int operations = DND.DROP_MOVE;

		final DragSource source = new DragSource(listPlatzhalter, operations);

		final Transfer[] types = new Transfer[] { TextTransfer.getInstance() };
		source.setTransfer(types);

		source.addDragListener(new DragSourceListener() {

			@Override
			public void dragFinished(final DragSourceEvent event) {
				// Hier gibt es nichts weiter zu tun. Es muss nichts gelscht
				// werden.
			}

			@Override
			public void dragSetData(final DragSourceEvent event) {

				int idx = listPlatzhalter.getSelectionIndex();
				if (idx < 0) {
					listPlatzhalter.setSelection(0);
					idx = 0;
				}

				String sel = "";
				final String[] selection = listPlatzhalter.getSelection();
				if (selection.length > 0) {
					sel = "[" + selection[0] + "]";
				}

				// event.data = "HALLO WELT!" + idx;

				event.data = sel;
			}

			@Override
			public void dragStart(final DragSourceEvent event) {
				event.doit = true;

			}

		});
	}

	/**
	 * Implementierung eines {@link IStructuredContentProvider} fr die Liste.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class ListeInhaltsAnbieter implements
			IStructuredContentProvider {
		@Override
		public Object[] getElements(final Object inputElement) {
			Object[] re = null;

			if (inputElement instanceof String[]) {
				final String[] input = (String[]) inputElement;

				if (input.length > 0) {
					re = new Object[input.length];

					for (int i = 0; i < input.length; i++) {
						re[i] = input[i];
					}
				}

			}

			return re;
		}

		@Override
		public void dispose() {
			// keine Ressourcen freizugeben
		}

		@Override
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			// Input wird vermmutlich global verwaltet?
		}

	}

	/** erweitert ein {@link StyledText} um Drop-Support von Texttransfers. */
	private static class TextFeld extends StyledText {

		public TextFeld(final Composite parent, final int style) {
			super(parent, style);
			addDropSupport();
		}

		private void addDropSupport() {
			final DropTarget dropTarget = new DropTarget(this, DND.DROP_MOVE);
			final Transfer[] types = new Transfer[] { TextTransfer
					.getInstance() };
			dropTarget.setTransfer(types);

			dropTarget.addDropListener(new DropTargetAdapter() {
				@Override
				public void drop(final DropTargetEvent event) {
					// A drop has occurred, copy over the data
					if (event.data == null) {
						// no data to copy, indicate failure in event.detail
						event.detail = DND.DROP_NONE;
						return;
					}

					// _debug.fine("ED: " + event.data.getClass().getName());

					if (event.data instanceof String) {
						final String einfuegen = (String) event.data;
						TextFeld.this.insert(einfuegen);
					}

				}
			});
		}

	}

} // @jve:decl-index=0:visual-constraint="10,10"
