/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 */

package de.bsvrz.buv.rw.rw.statusleiste;

import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.StatusLineLayoutData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsEvent;
import de.bsvrz.buv.rw.basislib.dav.DavVerbindungsListener;
import de.bsvrz.buv.rw.basislib.menu.AbstractRwMenuControl;
import de.bsvrz.buv.rw.basislib.util.RwColor;
import de.bsvrz.buv.rw.basislib.util.RwInfoDef;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;

/**
 * Statuszeilenelement zur Darstellung von Systeminformationen.
 * 
 * Es wird ein Offline- und ein Online-Format untersttzt, in dem Platzhalter
 * der Klasse {@link RwInfoDef} eingebaut werden knnen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: RwInfoStatusControl.java 50567 2014-05-27 13:03:22Z peuker $
 */
public class RwInfoStatusControl extends AbstractRwMenuControl {

	/**
	 * Datenverteiler-Verbindungslistener, ber den zwischen Online- und
	 * Offlinedarstellung umgeschaltet wird.
	 */
	private class VerbindungsListener implements DavVerbindungsListener {

		@Override
		public void verbindungHergestellt(final DavVerbindungsEvent event) {
			UIJob job = new UIJob("Statusline-Update") {

				@Override
				public IStatus runInUIThread(final IProgressMonitor monitor) {
					updateText();
					return Status.OK_STATUS;
				}
			};
			job.schedule();
		}

		@Override
		public void verbindungGetrennt(final DavVerbindungsEvent event) {
			updateText();
		}

		@Override
		public boolean verbindungHalten(final DavVerbindungsEvent event) {
			return false;
		}
	}

	/** die Parameter des Elements. */
	private static final String PARAM_HINTERGRUND = "hintergrund";
	private static final String PARAM_FARBE = "farbe";
	private static final String PARAM_FORMAT_ONLINE = "formatonline";
	private static final String PARAM_FORMAT_OFFLINE = "formatoffline";

	/** die aktuellen Parameter. */
	private String formatOnlineStr;
	private String formatOfflineStr;
	private String foreGround;
	private String backGround;

	private CLabel anzeigeText;
	private VerbindungsListener verbindungsListener;

	@Override
	public void fill(final Composite parent) {

		if (anzeigeText == null) {
			anzeigeText = new CLabel(parent, SWT.SHADOW_NONE | SWT.BORDER);

			Color color = new Color(anzeigeText.getDisplay(),
					RwColor.getRgbFromString(foreGround));
			anzeigeText.setForeground(color);
			color.dispose();

			if ((backGround == null) || backGround.trim().isEmpty()) {
				anzeigeText.setBackground((Color) null);
			} else {
				color = new Color(anzeigeText.getDisplay(),
						RwColor.getRgbFromString(backGround));
				anzeigeText.setBackground(color);
				color.dispose();
			}

			anzeigeText.setAlignment(SWT.CENTER);
			anzeigeText.addDisposeListener(new DisposeListener() {

				@Override
				public void widgetDisposed(final DisposeEvent e) {
					anzeigeText = null;
					if (verbindungsListener != null) {
						RahmenwerkImpl.getInstanz()
								.removeDavVerbindungsListener(
										verbindungsListener);
					}
					verbindungsListener = null;
				}
			});

			verbindungsListener = new VerbindungsListener();
			RahmenwerkImpl.getInstanz().addDavVerbindungsListener(
					verbindungsListener);

			updateText();
		}
	}

	private void updateText() {

		if (anzeigeText != null) {

			UIJob job = new UIJob("Refresh RwInfo") {

				@Override
				public IStatus runInUIThread(IProgressMonitor monitor) {
					String text;

					if ((anzeigeText != null) && (!anzeigeText.isDisposed())) {

						if (RahmenwerkImpl.getInstanz().isOnline()) {
							text = RwInfoDef.format(formatOnlineStr);
						} else {
							text = RwInfoDef.format(formatOfflineStr);
						}
						anzeigeText.setVisible((text != null)
								&& (!text.isEmpty()));

						anzeigeText.setText(text);
						final Point preferredSize = ((Control) anzeigeText)
								.computeSize(SWT.DEFAULT, SWT.DEFAULT);
						final int widthHint = preferredSize.x + 10;

						final StatusLineLayoutData data = new StatusLineLayoutData();
						data.widthHint = widthHint;
						anzeigeText.setLayoutData(data);
						anzeigeText.getParent().layout(true);
					}
					return Status.OK_STATUS;
				}
			};
			job.schedule();
		}
	}

	@Override
	public void setParameter(final Map<String, String> parameters) {
		for (final Entry<String, String> parameter : parameters.entrySet()) {
			if (RwInfoStatusControl.PARAM_FORMAT_ONLINE.equals(parameter
					.getKey())) {
				formatOnlineStr = parameter.getValue();
			} else if (RwInfoStatusControl.PARAM_FORMAT_OFFLINE
					.equals(parameter.getKey())) {
				formatOfflineStr = parameter.getValue();
			} else if (RwInfoStatusControl.PARAM_FARBE.equals(parameter
					.getKey())) {
				foreGround = parameter.getValue();
			} else if (RwInfoStatusControl.PARAM_HINTERGRUND.equals(parameter
					.getKey())) {
				backGround = parameter.getValue();
			}
		}
	}

	@Override
	public boolean useInStatusLine() {
		return true;
	}
}
