/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.statusleiste;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.rw.basislib.Rahmenwerk;
import de.bsvrz.buv.rw.compatibility.internal.RahmenwerkService;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse enthlt einen Verweis auf die globale Statusleiste und
 * ermglicht so den Zugriff. <br>
 * Diese Klasse ist als Singleton ausgelegt, da nur eine globale Statusleiste
 * verfgbar ist. <br>
 * Diese Klasse arbeitet auf den gngigen Eclipse-Klassen
 * {@link IContributionItem} sowie {@link IStatusLineManager}.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, HofmannJ (Dambach)
 * @version $Id: StatusLeisteZugriff.java 49030 2014-02-28 15:19:54Z peuker $
 * 
 * @deprecated Zugriff auf die Statusleiste sollte ber das
 *             {@link Rahmenwerk#getRwToolBarManager()} erfolgen.
 * 
 */
@Deprecated
public final class StatusLeisteZugriff {

	/** der Debugger. */
	private static final Debug LOGGER = Debug.getLogger();

	/** Privater Konstruktor dieser Singleton-Klasse. */
	private StatusLeisteZugriff() {
		// keine externen Instanzen gewnscht
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link StatusLeisteZugriff}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Innen {
		private static final StatusLeisteZugriff INSTANZ = new StatusLeisteZugriff();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link StatusLeisteZugriff}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link StatusLeisteZugriff}
	 */
	public static StatusLeisteZugriff getInstanz() {
		return Innen.INSTANZ;
	}

	/**
	 * Setzt den globalen StatusLineManager.
	 * 
	 * @param statusLineManager
	 *            Zu setzender StatusLineManager.
	 */
	public void setStatusZeilenManager(
			final IStatusLineManager statusLineManager) {
		StatusLeisteZugriff.LOGGER
				.warning("Der Statuszeilenmanger wird vom Rahmenwerk gesetzt und kann hier nicht berschrieben werden.");
	}

	/**
	 * Liefert den globalen StatuLineManager.
	 * 
	 * @return {@link IStatusLineManager#}
	 */
	public IStatusLineManager getStatusZeilenManager() {
		return RahmenwerkService.getService().getRahmenWerk()
				.getRwToolBarManager().getStatusLeisteManager();
	}

	/**
	 * Fgt ein Element zur Statuszeile hinzu.
	 * 
	 * @param statusleisteElement
	 *            Element, dass zur Statuszeile hinzugefgt werden soll.
	 */
	public void hinzufuegenStatusZeilenElement(
			final IContributionItem statusleisteElement) {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.add(statusleisteElement);
			statusZeilenManager.update(true);
		}
	}

	/**
	 * Entfernt ein Element aus der Statusleiste und liefert dieses als
	 * {@link IContributionItem} zurck. Wird das entsprechende Element nicht in
	 * der Statuszeile gefunden, wird <code>null</code> zurckgeliefert.
	 * 
	 * @param item
	 *            Element, dass aus der Statuszeile entfernt werden soll.
	 * @return Entferntes Element als {@link IContributionItem} oder
	 *         <code>null</code>.
	 */
	public IContributionItem entferneStatusZeilenElement(
			final ContributionItem item) {
		IContributionItem itemRemoved = null;
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			itemRemoved = statusZeilenManager.remove(item);
			statusZeilenManager.update(true);
		}
		return itemRemoved;
	}

	/**
	 * Entfernte alle Statuszeilenelemente aus der Statuszeile.
	 * 
	 */
	public void entferneAlleStatusZeilenElemente() {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.removeAll();
			statusZeilenManager.update(true);
			loescheStatusZeilenMeldung();
			loescheStatusZeilenFehlerMeldung();
		}
	}

	/**
	 * Setzt eine Meldung i.d. Statuszeile.<br>
	 * Diese Meldung ist <b>nur</b> sichtbar, wenn keine Fehlermeldung gesetzt
	 * ist.
	 * 
	 * @param meldungsText
	 *            Text der Meldung.
	 */
	public void setStatusZeilenMeldung(final String meldungsText) {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setMessage(meldungsText);
		}
	}

	/**
	 * Setzt eine Meldung mit Bild i.d. Statuszeile.<br>
	 * Diese Meldung ist <b>nur</b> sichtbar, wenn keine Fehlermeldung gesetzt
	 * ist.
	 * 
	 * @param meldungsText
	 *            Text der Meldung.
	 * @param meldungsBild
	 *            Bild der Meldung.
	 */
	public void setStatusZeilenMeldung(final String meldungsText,
			final Image meldungsBild) {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setMessage(meldungsBild, meldungsText);
		}
	}

	/**
	 * Setzt eine Fehlermeldung i.d. Stautzeile.<br>
	 * <b>berlargert</b> eine Meldung, wenn gesetzt.
	 * 
	 * @param fehlerMeldungsText
	 *            Text der Fehlermeldung.
	 */
	public void setStatusZeilenFehlerMeldung(final String fehlerMeldungsText) {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setErrorMessage(fehlerMeldungsText);
		}
	}

	/**
	 * Setzt eine Fehlermeldung mit Bild i.d. Statuszeile.<br>
	 * <b>berlargert</b> eine Meldung, wenn gesetzt.
	 * 
	 * @param fehlerMeldungsText
	 *            Text der Fehlermeldung.
	 * @param fehlerMeldungsBild
	 *            Bild der Fehlermeldung.
	 */
	public void setStatusZeilenFehlerMeldung(final String fehlerMeldungsText,
			final Image fehlerMeldungsBild) {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setErrorMessage(fehlerMeldungsBild,
					fehlerMeldungsText);
		}
	}

	/**
	 * Lscht den Meldungsbereich i.d. Statuszeile.
	 * 
	 */
	public void loescheStatusZeilenMeldung() {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setMessage(null);
		}
	}

	/**
	 * Lscht dern Fehlermeldungsbereich i.d. Statuszeile.
	 * 
	 */
	public void loescheStatusZeilenFehlerMeldung() {
		final IStatusLineManager statusZeilenManager = getStatusZeilenManager();
		if (statusZeilenManager != null) {
			statusZeilenManager.setErrorMessage(null);
		}
	}
}
