/*
 * This file is part of Rahmenwerk 2.0.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2013 BitCtrl Systems GmbH.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strae 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax:   +49-341-49067-15
 * mailto: support@bitctrl.de
 * 
 * Partly inspired by the original Rahmenwerk created by Dambach Werke GmbH.
 *   
 **************************************************************************************
 *
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.buv.rw.rw.ui.views;

import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.RahmenwerkActivator;
import de.bsvrz.buv.rw.rw.internal.RahmenwerkImpl;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;

/**
 * Ansicht zur Darstellung der an die Rahmenwerks-Applikation bergebenen
 * Startparameter.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: KommandoZeilenParameterView.java 46673 2013-10-16 14:24:41Z peuker $
 */
public class KommandoZeilenParameterView extends ViewPart {

	/** Labelprovider fr die Darstellung der Parameter. */
	private static class InternalLabelProvider extends LabelProvider implements
			ITableLabelProvider {

		@Override
		public Image getColumnImage(final Object element, final int columnIndex) {
			if (columnIndex == 0) {
				return RahmenwerkActivator.getDefault().getImage(
						"icons/view16/environment.gif");
			}
			return null;
		}

		@Override
		public String getColumnText(final Object element, final int columnIndex) {

			String result = null;

			if (element instanceof Argument) {
				final Argument argument = (Argument) element;
				if (columnIndex == 0) {
					result = argument.getName();
				} else if (columnIndex == 1) {
					if (argument.hasValue()) {
						result = argument.getValue();
					}
				}
			}

			if (result == null) {
				return "";
			}

			return result;
		}

	}

	/** ID der Ansicht. */
	public static final String ID = "de.bsvrz.buv.rw.rw.kommandozeilenparameter.KommandoZeilenParameterView";

	private TableViewer tableViewer;

	@Override
	public void createPartControl(final Composite parent) {

		final Composite panel = new Composite(parent, SWT.NONE);
		final TableColumnLayout tabLayout = new TableColumnLayout();
		panel.setLayout(tabLayout);

		tableViewer = new TableViewer(panel);

		TableColumn column = new TableColumn(tableViewer.getTable(), SWT.NONE);
		column.setText("Argument");
		tabLayout.setColumnData(column, new ColumnWeightData(1));

		column = new TableColumn(tableViewer.getTable(), SWT.NONE);
		column.setText("Wert");
		tabLayout.setColumnData(column, new ColumnWeightData(2));

		tableViewer.getTable().setHeaderVisible(true);
		tableViewer.getTable().setLinesVisible(true);

		tableViewer.setContentProvider(new ArrayContentProvider());
		tableViewer.setSorter(new ViewerSorter());

		tableViewer.setLabelProvider(new InternalLabelProvider());
		tableViewer.setInput(RahmenwerkImpl.getInstanz().getStartParameter()
				.fetchUnusedArguments());
	}

	@Override
	public void setFocus() {
		if ((tableViewer != null) && (!tableViewer.getControl().isDisposed())) {
			tableViewer.getControl().setFocus();
		}
	}
}
