/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
    
    Author: Friederich Kupzog  
    fkmk@kupzog.de
    www.kupzog.de/fkmk
 */
package de.kupzog.examples;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;

import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableModel;

/**
 * @author Friederich Kupzog
 */
public class PaletteExampleModel implements KTableModel {

	/*
	 * overridden from superclass
	 */
	@Override
	public Object getContentAt(final int col, final int row) {
		return new RGB(col * 16, row * 16, (col + row) * 8);
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellEditor getCellEditor(final int col, final int row) {
		return null;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setContentAt(final int col, final int row, final Object value) {
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowCount() {
		return 16;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getFixedHeaderRowCount() {
		return 0;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getColumnCount() {
		return 16;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getFixedHeaderColumnCount() {
		return 0;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getColumnWidth(final int col) {
		return 10;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public boolean isColumnResizable(final int col) {
		return false;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setColumnWidth(final int col, final int value) {
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowHeight(final int row) {
		return 10;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public boolean isRowResizable(final int row) {
		return false;
	}

	public void setFirstRowHeight(final int value) {
		// ignore since row resize is not allowed anyway.
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public int getRowHeightMinimum() {
		return 10;
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void setRowHeight(final int row, final int value) {
	}

	private static KTableCellRenderer myRenderer = new PaletteExampleRenderer();

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellRenderer getCellRenderer(final int col, final int row) {
		return PaletteExampleModel.myRenderer;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#belongsToCell(int, int)
	 */
	@Override
	public Point belongsToCell(final int col, final int row) {
		return new Point(col, row);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getTooltipAt(int, int)
	 */
	@Override
	public String getTooltipAt(final int col, final int row) {
		return (col * 16) + " / " + (row * 16) + " / " + ((col + row) * 8);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableRowCount()
	 */
	@Override
	public int getFixedSelectableRowCount() {
		// all fixed rows are non-selectable.
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableColumnCount()
	 */
	@Override
	public int getFixedSelectableColumnCount() {
		// all fixed columns are non-selectable.
		return 0;
	}

}
