/*
 * Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 
 Author: Friederich Kupzog  
 fkmk@kupzog.de
 www.kupzog.de/fkmk
 */
package de.kupzog.examples;

import java.util.HashMap;

import org.eclipse.swt.graphics.Point;

import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableDefaultModel;
import de.kupzog.ktable.editors.KTableCellEditorText;

/**
 * @author Friederich Kupzog
 */
public class SpanModelExample extends KTableDefaultModel {

	private final HashMap content = new HashMap();

	/**
	 * Initialize the underlying model
	 */
	public SpanModelExample() {
		// before initializing, you probably have to set some member values
		// to make all model getter methods work properly.
		initialize();
	}

	@Override
	public Object doGetContentAt(final int col, final int row) {
		if (isFixedCell(col, row)) {
			if ((col == 0) && (row < 4)) {
				return "Art";
			}

			if ((col == 1) && (row < 4)) {
				return "Zeit";
			}

			if ((col == 2) && (row < 4)) {
				return "Objekt";
			}

			if ((col > 2) && (col <= 12) && (row == 0)) {
				return "AnzeigeEigenschaftSoll";
			}

			if ((col > 2) && (col <= 11) && (row == 1)) {
				return "Eigenschaft";
			}

			if ((col == 12) && (row >= 1) && (row <= 3)) {
				return "SollZeitpunkt";
			}

			if ((col == 3) && (row >= 2) && (row <= 3)) {
				return "AnzeigeInhalt";
			}

			if ((col == 4) && (row >= 2) && (row <= 3)) {
				return "Blinken";
			}

			if ((col == 5) && (row >= 2) && (row <= 3)) {
				return "Blinktakt";
			}

			if ((col == 6) && (row >= 2) && (row <= 3)) {
				return "Passiviert";
			}

			if ((col == 7) && (row >= 2) && (row <= 3)) {
				return "GrafikDarstellungen [..]";
			}

			if ((col == 8) && (row >= 2) && (row <= 3)) {
				return "Wechseltext [..]";
			}

			if ((col >= 9) && (col <= 11) && (row == 2)) {
				return "DynamischerWechseltext";
			}

			if ((col == 9) && (row == 3)) {
				return "Font [..]";
			}

			if ((col == 10) && (row == 3)) {
				return "TextAusrichtung";
			}

			if ((col == 11) && (row == 3)) {
				return "Textdefinition [..]";
			}

			return "F" + col + "/" + row;
		}

		if ((col == 11) && (row == 4)) {
			return "zeichen.dWiSta.KlnPilot.Mittelschrift.H";
		}

		if ((col == 11) && (row == 5)) {
			return "zeichen.dWiSta.KlnPilot.Mittelschrift.a";
		}

		if ((col == 11) && (row == 6)) {
			return "zeichen.dWiSta.KlnPilot.Mittelschrift.l";
		}

		if ((col == 11) && (row == 7)) {
			return "zeichen.dWiSta.KlnPilot.Mittelschrift.l";
		}

		if ((col == 11) && (row == 8)) {
			return "zeichen.dWiSta.KlnPilot.Mittelschrift.o";
		}

		if ((col == 12) && (row == 4)) {
			return "zeichen 1";
		}

		if ((col == 12) && (row == 5)) {
			return "zeichen 2";
		}

		if ((col == 12) && (row == 6)) {
			return "zeichen 3";
		}

		if ((col == 0) && (row >= 4) && (row <= 8)) {
			return "OA";
		}

		if ((col > 2) && (row == 9)) {
			return "keine Daten";
		}

		if ((col > 2) && (row == 10)) {
			return "keine Daten (keine Quelle)";
		}

		// if
		// (((col-getFixedHeaderColumnCount()-getFixedSelectableColumnCount())%2==0
		// &&
		// (row-getFixedHeaderRowCount()-getFixedSelectableRowCount())%2==0))
		return "S" + col + "/" + row;

		// should never happen, since the spanning should prevent this:
		// System.err.println("A content was requested that is invalid: "+col+"/"+row);
		// return "Non-Valid: "+col+"/"+row;

	}

	/*
	 * overridden from superclass
	 */
	@Override
	public KTableCellEditor doGetCellEditor(final int col, final int row) {
		if (isFixedCell(col, row)) {
			return null; // no editor
		}

		// if
		// (((col-getFixedHeaderColumnCount()-getFixedSelectableColumnCount())%2==0
		// &&
		// (row-getFixedHeaderRowCount()-getFixedSelectableRowCount())%2==0))
		return new KTableCellEditorText();

		// should never happen:
		// System.err.println("A cell editor was requested that is invalid: "+col+"/"+row);
		// return null;
	}

	// Rendering
	@Override
	public KTableCellRenderer doGetCellRenderer(final int col, final int row) {
		if (isFixedCell(col, row)) {
			return KTableCellRenderer.defaultRenderer;
		}

		// if
		// (((col-getFixedHeaderColumnCount()-getFixedSelectableColumnCount())%2==0
		// &&
		// (row-getFixedHeaderRowCount()-getFixedSelectableRowCount())%2==0))
		// return KTableCellRenderer.defaultRenderer;

		// should never happen:
		// System.err.println("A cell renderer was requested that is invalid: "+col+"/"+row);

		// return KTableCellRenderer.defaultRenderer;

		return new SpanExampleRenderer();
	}

	/*
	 * overridden from superclass
	 */
	@Override
	public void doSetContentAt(final int col, final int row, final Object value) {
		System.out.println("Set val at: " + col + "/" + row + ": " + value);
		content.put(col + "/" + row, value);
	}

	/**
	 * Makes all cells span 2 rows and two colums. This is just a simple example
	 * to demonstrate how to work with this feature.
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#doBelongsToCell(int, int)
	 */
	@Override
	public Point doBelongsToCell(final int col, final int row) {
		if (isFixedCell(col, row)) {
			if ((col == 0) && (row < 4)) {
				return new Point(col, 0);
			}

			if ((col == 1) && (row < 4)) {
				return new Point(col, 0);
			}

			if ((col == 2) && (row < 4)) {
				return new Point(col, 0);
			}

			if ((col > 2) && (col <= 12) && (row == 0)) {
				return new Point(3, row);
			}

			if ((col > 2) && (col <= 11) && (row == 1)) {
				return new Point(3, row);
			}

			if ((col == 12) && (row >= 1) && (row <= 3)) {
				return new Point(12, 1);
			}

			if ((col == 3) && (row >= 2) && (row <= 3)) {
				return new Point(3, 2);
			}

			if ((col == 3) && (row >= 2) && (row <= 3)) {
				return new Point(3, 2);
			}

			if ((col == 4) && (row >= 2) && (row <= 3)) {
				return new Point(4, 2);
			}

			if ((col == 5) && (row >= 2) && (row <= 3)) {
				return new Point(5, 2);
			}

			if ((col == 6) && (row >= 2) && (row <= 3)) {
				return new Point(6, 2);
			}

			if ((col == 7) && (row >= 2) && (row <= 3)) {
				return new Point(7, 2);
			}

			if ((col == 8) && (row >= 2) && (row <= 3)) {
				return new Point(8, 2);
			}

			if ((col >= 9) && (col <= 11) && (row == 2)) {
				return new Point(9, row);
			}

			if ((col == 9) && (row == 3)) {
				return new Point(col, row);
			}

			if ((col == 10) && (row == 3)) {
				return new Point(col, row);
			}

			if ((col == 11) && (row == 3)) {
				return new Point(col, row);
			}

			if ((col == 0) && (row >= 4) && (row <= 8)) {
				return new Point(col, 4);
			}

			if ((col == 1) && (row >= 4) && (row <= 8)) {
				return new Point(col, 4);
			}

			if ((col == 2) && (row >= 4) && (row <= 8)) {
				return new Point(col, 4);
			}

			return new Point(col, row);
		}

		// col-=getFixedHeaderColumnCount()+getFixedSelectableColumnCount();
		// row-=getFixedHeaderRowCount()+getFixedSelectableRowCount();

		// if ((col==1 && row==1) && (col%2==0 && row%2==0))
		// return new Point(col, row);

		// find supercell to merge with:
		// int newCol = col;
		// int newRow = row;
		// if (col%2!=0)
		// newCol--;
		// if (row%2!=0)
		// newRow--;

		// System.out.println("col " + col + " row " + row);

		// return new
		// Point(newCol+getFixedHeaderColumnCount()+getFixedSelectableColumnCount(),
		// newRow+getFixedHeaderRowCount()+getFixedSelectableRowCount());

		if ((col == 3) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 4) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 5) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 6) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 7) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 8) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 9) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 10) && (row >= 4) && (row <= 8)) {
			return new Point(col, 4);
		}

		if ((col == 12) && (row >= 6) && (row <= 8)) {
			return new Point(col, 6);
		}

		if ((col > 2) && (row == 9)) {
			return new Point(3, 9);
		}

		if ((col > 2) && (row == 10)) {
			return new Point(3, 10);
		}

		return new Point(col, row);
	}

	// number of cells:
	@Override
	public int doGetRowCount() {
		return 1000; // 100+getFixedRowCount();
	}

	@Override
	public int doGetColumnCount() {
		return 13; // 100+getFixedColumnCount();
	}

	@Override
	public int getFixedHeaderRowCount() {
		return 4;
	}

	@Override
	public int getFixedHeaderColumnCount() {
		return 3;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableRowCount()
	 */
	@Override
	public int getFixedSelectableRowCount() {
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableModel#getFixedSelectableColumnCount()
	 */
	@Override
	public int getFixedSelectableColumnCount() {
		return 0;
	}

	@Override
	public boolean isColumnResizable(final int col) {
		// if (col < 3)
		// {
		// return false;
		// }

		return true;
	}

	@Override
	public int getInitialFirstRowHeight() {
		return 22;
	}

	@Override
	public boolean isRowResizable(final int row) {
		if (row < 4) {
			return false;
		}

		return true;
	}

	@Override
	public int getRowHeightMinimum() {
		return 18;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#getInitialColumnWidth(int)
	 */
	@Override
	public int getInitialColumnWidth(final int column) {
		if (column == 2) {
			return 60;
		}

		if (column == 12) {
			return 80;
		}

		return 35;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#getInitialRowHeight()
	 */
	@Override
	public int getInitialRowHeight(final int row) {
		// if (row == 4)
		// return 90;

		return 18;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.kupzog.ktable.KTableDefaultModel#doGetTooltipAt(int, int)
	 */
	@Override
	public String doGetTooltipAt(final int col, final int row) {
		// return "Test-Tooltip " + col + "/" + row;
		return (String) doGetContentAt(col, row);
	}
}
